@@ -1,20 +1,26 @@
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from flexget.utils.template import get_template
+
+from future.moves.urllib.parse import urlparse, parse_qsl
 
 import os
 import re
-import urlparse
+import logging
 from collections import defaultdict
 from datetime import datetime
 
 import jsonschema
-from jsonschema.compat import str_types
+from jsonschema.compat import str_types, int_types
 
 from flexget.event import fire_event
 from flexget.utils import qualities, template
-from flexget.utils.tools import parse_timedelta
+from flexget.utils.tools import parse_timedelta, parse_episode_identifier
 
 schema_paths = {}
 
+log = logging.getLogger('config_schema')
+
 
 # TODO: Rethink how config key and schema registration work
 def register_schema(path, schema):
@@ -57,7 +63,7 @@ def get_schema():
     return _root_config_schema
 
 
-def one_or_more(schema):
+def one_or_more(schema, unique_items=False):
     """
     Helper function to construct a schema that validates items matching `schema` or an array
     containing items matching `schema`.
@@ -67,7 +73,7 @@ def one_or_more(schema):
     schema.setdefault('title', 'single value')
     return {
         'oneOf': [
-            {'title': 'multiple values', 'type': 'array', 'items': schema, 'minItems': 1},
+            {'title': 'multiple values', 'type': 'array', 'items': schema, 'minItems': 1, 'uniqueItems': unique_items},
             schema
         ]
     }
@@ -77,11 +83,11 @@ def resolve_ref(uri):
     """
     Finds and returns a schema pointed to by `uri` that has been registered in the register_schema function.
     """
-    parsed = urlparse.urlparse(uri)
+    parsed = urlparse(uri)
     if parsed.path in schema_paths:
         schema = schema_paths[parsed.path]
         if callable(schema):
-            return schema(**dict(urlparse.parse_qsl(parsed.query)))
+            return schema(**dict(parse_qsl(parsed.query)))
         return schema
     raise jsonschema.RefResolutionError("%s could not be resolved" % uri)
 
@@ -107,7 +113,7 @@ def process_config(config, schema=None, set_defaults=True):
     # Customize the error messages
     for e in errors:
         set_error_message(e)
-        e.json_pointer = '/' + '/'.join(map(unicode, e.path))
+        e.json_pointer = '/' + '/'.join(map(str, e.path))
     return errors
 
 
@@ -130,7 +136,30 @@ def parse_interval(interval_string):
     return parse_timedelta(interval_string)
 
 
-## Public API end here, the rest should not be used outside this module
+def parse_percent(percent_input):
+    """Takes a size string from the config and turns it into int(bytes)."""
+    percent_input = percent_input.rstrip('%')
+    try:
+        return float(percent_input)
+    except ValueError:
+        raise ValueError("should be in format '0-x%'")
+
+
+def parse_size(size_input):
+    """Takes a size string from the config and turns it into int(bytes)."""
+    prefixes = [None, 'K', 'M', 'G', 'T', 'P']
+    try:
+        # Bytes
+        return int(size_input)
+    except ValueError:
+        size_input = size_input.upper().rstrip('IB')
+        value, unit = float(size_input[:-1]), size_input[-1:]
+        if unit not in prefixes:
+            raise ValueError("should be in format '0-x (KiB, MiB, GiB, TiB, PiB)'")
+        return int(1024 ** prefixes.index(unit) * value)
+
+
+# Public API end here, the rest should not be used outside this module
 
 
 class RefResolver(jsonschema.RefResolver):
@@ -156,7 +185,6 @@ def is_quality_req(instance):
     return qualities.Requirements(instance)
 
 
-
 @format_checker.checks('time', raises=ValueError)
 def is_time(time_string):
     if not isinstance(time_string, str_types):
@@ -171,6 +199,20 @@ def is_interval(interval_string):
     return parse_interval(interval_string) is not None
 
 
+@format_checker.checks('size', raises=ValueError)
+def is_size(size_string):
+    if not isinstance(size_string, (str_types, int_types)):
+        return True
+    return parse_size(size_string) is not None
+
+
+@format_checker.checks('percent', raises=ValueError)
+def is_percent(percent_string):
+    if not isinstance(percent_string, str_types):
+        return True
+    return parse_percent(percent_string) is not None
+
+
 @format_checker.checks('regex', raises=ValueError)
 def is_regex(instance):
     if not isinstance(instance, str_types):
@@ -204,16 +246,37 @@ def is_path(instance):
     raise ValueError('`%s` does not exist' % instance)
 
 
-#TODO: jsonschema has a format checker for uri if rfc3987 is installed, perhaps we should use that
+# TODO: jsonschema has a format checker for uri if rfc3987 is installed, perhaps we should use that
 @format_checker.checks('url')
 def is_url(instance):
     if not isinstance(instance, str_types):
         return True
-    regexp = ('(' + '|'.join(['ftp', 'http', 'https', 'file', 'udp']) +
+    regexp = ('(' + '|'.join(['ftp', 'http', 'https', 'file', 'udp', 'socks5']) +
               '):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?')
     return re.match(regexp, instance)
 
 
+@format_checker.checks('episode_identifier', raises=ValueError)
+def is_episode_identifier(instance):
+    if not isinstance(instance, (str_types, int)):
+        return True
+    return parse_episode_identifier(instance) is not None
+
+
+@format_checker.checks('episode_or_season_id', raises=ValueError)
+def is_episode_or_season_id(instance):
+    if not isinstance(instance, (str_types, int)):
+        return True
+    return parse_episode_identifier(instance, identify_season=True) is not None
+
+
+@format_checker.checks('file_template', raises=ValueError)
+def is_valid_template(instance):
+    if not isinstance(instance, str_types):
+        return True
+    return get_template(instance) is not None
+
+
 def set_error_message(error):
     """
     Create user facing error message from a :class:`jsonschema.ValidationError` `error`
@@ -221,7 +284,7 @@ def set_error_message(error):
     """
     # First, replace default error messages with our custom ones
     if error.validator == 'type':
-        if isinstance(error.validator_value, basestring):
+        if isinstance(error.validator_value, str):
             valid_types = [error.validator_value]
         else:
             valid_types = list(error.validator_value)
@@ -237,9 +300,9 @@ def set_error_message(error):
         error.message = 'Got `%s`, expected: %s' % (error.instance, valid_types)
     elif error.validator == 'format':
         if error.cause:
-            error.message = unicode(error.cause)
+            error.message = str(error.cause)
     elif error.validator == 'enum':
-        error.message = 'Must be one of the following: %s' % ', '.join(map(unicode, error.validator_value))
+        error.message = 'Must be one of the following: %s' % ', '.join(map(str, error.validator_value))
     elif error.validator == 'additionalProperties':
         if error.validator_value is False:
             extras = set(jsonschema._utils.find_additional_properties(error.instance, error.schema))
@@ -273,7 +336,7 @@ def select_child_errors(validator, errors):
         # Find the subschemas that did not have a 'type' error validating the instance at this path
         no_type_errors = dict(subschema_errors)
         valid_types = set()
-        for i, errors in subschema_errors.iteritems():
+        for i, errors in subschema_errors.items():
             for e in errors:
                 if e.validator == 'type' and not e.path:
                     # Remove from the no_type_errors dict
@@ -290,7 +353,7 @@ def select_child_errors(validator, errors):
         elif len(no_type_errors) == 1:
             # If one of the possible schemas did not have a 'type' error, assume that is the intended one and issue
             # all errors from that subschema
-            for e in no_type_errors.values()[0]:
+            for e in list(no_type_errors.values())[0]:
                 e.schema_path.extendleft(reversed(error.schema_path))
                 e.path.extendleft(reversed(error.path))
                 yield e
@@ -299,10 +362,9 @@ def select_child_errors(validator, errors):
 
 
 def validate_properties_w_defaults(validator, properties, instance, schema):
-
     if not validator.is_type(instance, 'object'):
         return
-    for key, subschema in properties.iteritems():
+    for key, subschema in properties.items():
         if 'default' in subschema:
             instance.setdefault(key, subschema['default'])
     for error in jsonschema.Draft4Validator.VALIDATORS["properties"](validator, properties, instance, schema):
@@ -321,9 +383,15 @@ def validate_oneOf(validator, oneOf, instance, schema):
         yield e
 
 
+def validate_deprecated(validator, message, instance, schema):
+    """Not really a validator, just warns if deprecated section of config is being used."""
+    log.warning(message)
+
+
 validators = {
     'anyOf': validate_anyOf,
-    'oneOf': validate_oneOf
+    'oneOf': validate_oneOf,
+    'deprecated': validate_deprecated
 }
 
 SchemaValidator = jsonschema.validators.extend(jsonschema.Draft4Validator, validators)
