@@ -1,7 +1,8 @@
 """
 Provides small event framework
 """
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 import logging
 
@@ -35,6 +36,9 @@ class Event(object):
 
     __repr__ = __str__
 
+    def __hash__(self):
+        return hash((self.name, self.func, self.priority))
+
 
 def event(name, priority=128):
     """Register event to function with a decorator"""
@@ -51,7 +55,7 @@ def get_events(name):
     :param String name: event name
     :return: List of :class:`Event` for *name* ordered by priority
     """
-    if not name in _events:
+    if name not in _events:
         raise KeyError('No such event %s' % name)
     _events[name].sort(reverse=True)
     return _events[name]
@@ -97,10 +101,9 @@ def fire_event(name, *args, **kwargs):
     :param args: List of arguments passed to handler function
     :param kwargs: Key Value arguments passed to handler function
     """
-    if not name in _events:
-        return
-    for event in get_events(name):
-        result = event(*args, **kwargs)
-        if result is not None:
-            args = (result,) + args[1:]
+    if name in _events:
+        for event in get_events(name):
+            result = event(*args, **kwargs)
+            if result is not None:
+                args = (result,) + args[1:]
     return args and args[0]
