@@ -1,38 +1,46 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import unicode_literals, division, absolute_import, print_function
 
-import atexit
-import codecs
-import copy
-import fnmatch
-import logging
-import os
-import shutil
-import signal
-import sys
-import threading
-from contextlib import contextmanager
-from datetime import datetime, timedelta
+# This needs to remain before the builtins import!
+native_int = int
 
-import pkg_resources
-import sqlalchemy
-import yaml
-from sqlalchemy.exc import OperationalError
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import sessionmaker
+from builtins import *  # noqa  pylint: disable=unused-import, redefined-builtin
+
+import atexit  # noqa
+import codecs  # noqa
+import copy  # noqa
+import fnmatch  # noqa
+import logging  # noqa
+import os  # noqa
+import shutil  # noqa
+import signal  # noqa
+import sys  # noqa
+import threading  # noqa
+import traceback  # noqa
+import hashlib  # noqa
+from contextlib import contextmanager  # noqa
+from datetime import datetime, timedelta  # noqa
+import io  # noqa
+
+import sqlalchemy  # noqa
+import yaml  # noqa
+from sqlalchemy.exc import OperationalError  # noqa
+from sqlalchemy.ext.declarative import declarative_base  # noqa
+from sqlalchemy.orm import sessionmaker  # noqa
 
 # These need to be declared before we start importing from other flexget modules, since they might import them
-from flexget.utils.sqlalchemy_utils import ContextSession
+from flexget.utils.sqlalchemy_utils import ContextSession  # noqa
+
 Base = declarative_base()
 Session = sessionmaker(class_=ContextSession)
 
-from flexget import config_schema, db_schema, logger, plugin
-from flexget.event import fire_event
-from flexget.ipc import IPCClient, IPCServer
-from flexget.options import CoreArgumentParser, get_parser, manager_parser, ParserError, unicode_argv
-from flexget.task import Task
-from flexget.task_queue import TaskQueue
-from flexget.utils.tools import pid_exists
-
+from flexget import config_schema, db_schema, logger, plugin  # noqa
+from flexget.event import fire_event  # noqa
+from flexget.ipc import IPCClient, IPCServer  # noqa
+from flexget.options import CoreArgumentParser, get_parser, manager_parser, ParserError, unicode_argv  # noqa
+from flexget.task import Task  # noqa
+from flexget.task_queue import TaskQueue  # noqa
+from flexget.utils.tools import pid_exists, get_current_flexget_version, io_encoding  # noqa
+from flexget.terminal import console  # noqa
 
 log = logging.getLogger('manager')
 
@@ -40,17 +48,7 @@ manager = None
 DB_CLEANUP_INTERVAL = timedelta(days=7)
 
 
-@sqlalchemy.event.listens_for(sqlalchemy.engine.Engine, 'connect')
-def set_sqlite_pragma(dbapi_connection, connection_record):
-    cursor = dbapi_connection.cursor()
-    # There were reported db problems with WAL mode on XFS filesystem, which is sticky and may have been turned
-    # on with certain FlexGet versions (e2c118e) #2749
-    cursor.execute('PRAGMA journal_mode=delete')
-    cursor.close()
-
-
 class Manager(object):
-
     """Manager class for FlexGet
 
     Fires events:
@@ -86,6 +84,10 @@ class Manager(object):
       If any plugins have declared a newer schema version than exists in the database, this event will be fired to
       allow plugins to upgrade their tables
 
+    * manager.shutdown_requested
+
+      When shutdown has been requested. Any plugins which might add to execution queue should stop when this is fired.
+
     * manager.shutdown
 
       When the manager is exiting
@@ -107,12 +109,17 @@ class Manager(object):
         :param args: CLI args
         """
         global manager
-        assert not manager, 'Only one instance of Manager should be created at a time!'
+        if not self.unit_test:
+            assert not manager, 'Only one instance of Manager should be created at a time!'
+        elif manager:
+            log.info('last manager was not torn down correctly')
 
         if args is None:
             # Decode all arguments to unicode before parsing
             args = unicode_argv()[1:]
         self.args = args
+        self.autoreload_config = False
+        self.config_file_hash = None
         self.config_base = None
         self.config_name = None
         self.config_path = None
@@ -134,13 +141,19 @@ class Manager(object):
             # TODO: This is a bit hacky, but we can't call parse on real arguments when --help is used because it will
             # cause a system exit before plugins are loaded and print incomplete help. This will get us a default
             # options object and we'll parse the real args later, or send them to daemon. #2807
-            self.options, extra = CoreArgumentParser().parse_known_args(['execute'])
+            self.options, _ = CoreArgumentParser().parse_known_args(['execute'])
         else:
             try:
-                self.options, extra = CoreArgumentParser().parse_known_args(args)
+                self.options, _ = CoreArgumentParser().parse_known_args(args)
             except ParserError:
-                # If a non-built-in command was used, we need to parse with a parser that doesn't define the subparsers
-                self.options, extra = manager_parser.parse_known_args(args)
+                try:
+                    # If a non-built-in command was used, we need to parse with a parser that
+                    # doesn't define the subparsers
+                    self.options, _ = manager_parser.parse_known_args(args)
+                except ParserError as e:
+                    manager_parser.print_help()
+                    print('\nError: %s' % e.message)
+                    sys.exit(1)
         try:
             self.find_config(create=False)
         except:
@@ -151,22 +164,20 @@ class Manager(object):
             # If an absolute path is not specified, use the config directory.
             if not os.path.isabs(log_file):
                 log_file = os.path.join(self.config_base, log_file)
+
             logger.start(log_file, self.options.loglevel.upper(), to_console=not self.options.cron)
 
         manager = self
 
-        log.debug('sys.defaultencoding: %s' % sys.getdefaultencoding())
-        log.debug('sys.getfilesystemencoding: %s' % sys.getfilesystemencoding())
+        log.debug('sys.defaultencoding: %s', sys.getdefaultencoding())
+        log.debug('sys.getfilesystemencoding: %s', sys.getfilesystemencoding())
+        log.debug('flexget detected io encoding: %s', io_encoding)
         log.debug('os.path.supports_unicode_filenames: %s' % os.path.supports_unicode_filenames)
         if codecs.lookup(sys.getfilesystemencoding()).name == 'ascii' and not os.path.supports_unicode_filenames:
             log.warning('Your locale declares ascii as the filesystem encoding. Any plugins reading filenames from '
                         'disk will not work properly for filenames containing non-ascii characters. Make sure your '
                         'locale env variables are set up correctly for the environment which is launching FlexGet.')
 
-    def __del__(self):
-        global manager
-        manager = None
-
     def initialize(self):
         """
         Load plugins, database, and config. Also initializes (but does not start) the task queue and ipc server.
@@ -178,6 +189,8 @@ class Manager(object):
         plugin.load_plugins(extra_dirs=[os.path.join(self.config_base, 'plugins')])
 
         # Reparse CLI options now that plugins are loaded
+        if not self.args:
+            self.args = ['--help']
         self.options = get_parser().parse_args(self.args)
 
         self.task_queue = TaskQueue()
@@ -209,7 +222,7 @@ class Manager(object):
         """A list of tasks in the config"""
         if not self.config:
             return []
-        return self.config.get('tasks', {}).keys()
+        return list(self.config.get('tasks', {}).keys())
 
     @property
     def has_lock(self):
@@ -224,7 +237,7 @@ class Manager(object):
             written to it.
         :param priority: If there are other executions waiting to be run, they will be run in priority order,
             lowest first.
-        :returns: a list of :class:`threading.Event` instances which will be
+         :returns: a list of :class:`threading.Event` instances which will be
             set when each respective task has finished running
         """
         if options is None:
@@ -234,6 +247,15 @@ class Manager(object):
             options_namespace.__dict__.update(options)
             options = options_namespace
         task_names = self.tasks
+        # Only reload config if daemon
+        config_hash = self.hash_config()
+        if self.is_daemon and self.autoreload_config and self.config_file_hash != config_hash:
+            log.info('Config change detected. Reloading.')
+            try:
+                self.load_config(output_to_console=False, config_file_hash=config_hash)
+                log.info('Config successfully reloaded!')
+            except Exception as e:
+                log.error('Reloading config failed: %s', e)
         # Handle --tasks
         if options.tasks:
             # Consider * the same as not specifying tasks at all (makes sure manual plugin still works)
@@ -243,7 +265,7 @@ class Manager(object):
                 # Create list of tasks to run, preserving order
                 task_names = []
                 for arg in options.tasks:
-                    matches = [t for t in self.tasks if fnmatch.fnmatchcase(unicode(t).lower(), arg.lower())]
+                    matches = [t for t in self.tasks if fnmatch.fnmatchcase(str(t).lower(), arg.lower())]
                     if not matches:
                         msg = '`%s` does not match any tasks' % arg
                         log.error(msg)
@@ -260,7 +282,7 @@ class Manager(object):
         for task_name in task_names:
             task = Task(self, task_name, options=options, output=output, loglevel=loglevel, priority=priority)
             self.task_queue.put(task)
-            finished_events.append(task.finished_event)
+            finished_events.append((task.id, task.name, task.finished_event))
         return finished_events
 
     def start(self):
@@ -277,8 +299,9 @@ class Manager(object):
         # If another process is started, send the execution to the running process
         ipc_info = self.check_ipc_info()
         if ipc_info:
+            console('There is a FlexGet process already running for this config, sending execution there.')
+            log.debug('Sending command to running FlexGet process: %s' % self.args)
             try:
-                log.info('There is a FlexGet process already running for this config, sending execution there.')
                 client = IPCClient(ipc_info['port'], ipc_info['password'])
             except ValueError as e:
                 log.error(e)
@@ -310,7 +333,6 @@ class Manager(object):
 
         * :meth:`.execute_command`
         * :meth:`.daemon_command`
-        * :meth:`.webui_command`
         * CLI plugin callback function
 
         The manager should have a lock and be initialized before calling this method.
@@ -322,13 +344,11 @@ class Manager(object):
         command = options.cli_command
         command_options = getattr(options, command)
         # First check for built-in commands
-        if command in ['execute', 'daemon', 'webui']:
+        if command in ['execute', 'daemon']:
             if command == 'execute':
                 self.execute_command(command_options)
             elif command == 'daemon':
                 self.daemon_command(command_options)
-            elif command == 'webui':
-                self.webui_command(command_options)
         else:
             # Otherwise dispatch the command to the callback function
             options.cli_command_callback(self, command_options)
@@ -348,14 +368,19 @@ class Manager(object):
         :param options: argparse options
         """
         fire_event('manager.execute.started', self, options)
-        if self.task_queue.is_alive():
+        if self.task_queue.is_alive() or self.is_daemon:
+            if not self.task_queue.is_alive():
+                log.error('Task queue has died unexpectedly. Restarting it. Please open an issue on Github and include'
+                          ' any previous error logs.')
+                self.task_queue = TaskQueue()
+                self.task_queue.start()
             if len(self.task_queue):
                 log.verbose('There is a task already running, execution queued.')
             finished_events = self.execute(options, output=logger.get_capture_stream(),
                                            loglevel=logger.get_capture_loglevel())
             if not options.cron:
                 # Wait until execution of all tasks has finished
-                for event in finished_events:
+                for _, _, event in finished_events:
                     event.wait()
         else:
             self.task_queue.start()
@@ -376,12 +401,19 @@ class Manager(object):
 
         :param options: argparse options
         """
+
+        # Import API so it can register to daemon.started event
         if options.action == 'start':
             if self.is_daemon:
                 log.error('Daemon already running for this config.')
                 return
+            elif self.task_queue.is_alive():
+                log.error('Non-daemon execution of FlexGet is running. Cannot start daemon until it is finished.')
+                return
             if options.daemonize:
                 self.daemonize()
+            if options.autoreload_config:
+                self.autoreload_config = True
             try:
                 signal.signal(signal.SIGTERM, self._handle_sigterm)
             except ValueError as e:
@@ -394,15 +426,17 @@ class Manager(object):
             self.ipc_server.start()
             self.task_queue.wait()
             fire_event('manager.daemon.completed', self)
-        elif options.action in ['stop', 'reload', 'status']:
+        elif options.action in ['stop', 'reload-config', 'status']:
             if not self.is_daemon:
                 log.error('There does not appear to be a daemon running.')
                 return
             if options.action == 'status':
                 log.info('Daemon running. (PID: %s)' % os.getpid())
             elif options.action == 'stop':
+                tasks = 'all queued tasks (if any) have' if options.wait else 'currently running task (if any) has'
+                log.info('Daemon shutdown requested. Shutdown will commence when %s finished executing.' % tasks)
                 self.shutdown(options.wait)
-            elif options.action == 'reload':
+            elif options.action == 'reload-config':
                 log.info('Reloading config from disk.')
                 try:
                     self.load_config()
@@ -411,32 +445,6 @@ class Manager(object):
                 else:
                     log.info('Config successfully reloaded from disk.')
 
-    def webui_command(self, options):
-        """
-        Handles the 'webui' CLI command.
-
-        :param options: argparse options
-        """
-        if self.is_daemon:
-            log.error('Webui or daemon is already running.')
-            return
-        # TODO: make webui an enablable plugin in regular daemon mode
-        try:
-            pkg_resources.require('flexget[webui]')
-        except pkg_resources.DistributionNotFound as e:
-            log.error('Dependency not met. %s' % e)
-            log.error('Webui dependencies not installed. You can use `pip install flexget[webui]` to install them.')
-            self.shutdown()
-            return
-        if options.daemonize:
-            self.daemonize()
-        self.is_daemon = True
-        from flexget.ui import webui
-        self.task_queue.start()
-        self.ipc_server.start()
-        webui.start(self)
-        self.task_queue.wait()
-
     def _handle_sigterm(self, signum, frame):
         log.info('Got SIGTERM. Shutting down.')
         self.shutdown(finish_queue=False)
@@ -448,6 +456,7 @@ class Manager(object):
             # Override the default string handling function
             # to always return unicode objects
             return self.construct_scalar(node)
+
         yaml.Loader.add_constructor(u'tag:yaml.org,2002:str', construct_yaml_str)
         yaml.SafeLoader.add_constructor(u'tag:yaml.org,2002:str', construct_yaml_str)
 
@@ -455,13 +464,14 @@ class Manager(object):
         def unicode_representer(dumper, uni):
             node = yaml.ScalarNode(tag=u'tag:yaml.org,2002:str', value=uni)
             return node
-        yaml.add_representer(unicode, unicode_representer)
+
+        yaml.add_representer(str, unicode_representer)
 
         # Set up the dumper to increase the indent for lists
         def increase_indent_wrapper(func):
-
             def increase_indent(self, flow=False, indentless=False):
                 func(self, flow, False)
+
             return increase_indent
 
         yaml.Dumper.increase_indent = increase_indent_wrapper(yaml.Dumper.increase_indent)
@@ -474,7 +484,6 @@ class Manager(object):
         :param bool create: If a config file is not found, and create is True, one will be created in the home folder
         :raises: `IOError` when no config file could be found, and `create` is False.
         """
-        config = None
         home_path = os.path.join(os.path.expanduser('~'), '.flexget')
         options_config = os.path.expanduser(self.options.config)
 
@@ -486,13 +495,13 @@ class Manager(object):
         else:
             log.debug('Figuring out config load paths')
             try:
-                possible.append(os.getcwdu())
+                possible.append(os.getcwd())
             except OSError:
                 log.debug('current directory invalid, not searching for config there')
             # for virtualenv / dev sandbox
             if hasattr(sys, 'real_prefix'):
                 log.debug('Adding virtualenv path')
-                possible.append(sys.prefix.decode(sys.getfilesystemencoding()))
+                possible.append(sys.prefix)
             # normal lookup locations
             possible.append(home_path)
             if sys.platform.startswith('win'):
@@ -532,61 +541,76 @@ class Manager(object):
         self.lockfile = os.path.join(self.config_base, '.%s-lock' % self.config_name)
         self.db_filename = os.path.join(self.config_base, 'db-%s.sqlite' % self.config_name)
 
+    def hash_config(self):
+        if not self.config_path:
+            return
+        sha1_hash = hashlib.sha1()
+        with io.open(self.config_path, 'rb') as f:
+            while True:
+                data = f.read(65536)
+                if not data:
+                    break
+                sha1_hash.update(data)
+        return sha1_hash.hexdigest()
 
-    def load_config(self):
+    def load_config(self, output_to_console=True, config_file_hash=None):
         """
         Loads the config file from disk, validates and activates it.
 
         :raises: `ValueError` if there is a problem loading the config file
         """
         fire_event('manager.before_config_load', self)
-        with codecs.open(self.config_path, 'rb', 'utf-8') as f:
+        with io.open(self.config_path, 'r', encoding='utf-8') as f:
             try:
                 raw_config = f.read()
             except UnicodeDecodeError:
                 log.critical('Config file must be UTF-8 encoded.')
                 raise ValueError('Config file is not UTF-8 encoded')
         try:
+            self.config_file_hash = config_file_hash or self.hash_config()
             config = yaml.safe_load(raw_config) or {}
         except Exception as e:
             msg = str(e).replace('\n', ' ')
             msg = ' '.join(msg.split())
             log.critical(msg, exc_info=False)
-            print('')
-            print('-' * 79)
-            print(' Malformed configuration file (check messages above). Common reasons:')
-            print('-' * 79)
-            print('')
-            print(' o Indentation error')
-            print(' o Missing : from end of the line')
-            print(' o Non ASCII characters (use UTF8)')
-            print(' o If text contains any of :[]{}% characters it must be single-quoted ' \
-                  '(eg. value{1} should be \'value{1}\')\n')
+            if output_to_console:
+                print('')
+                print('-' * 79)
+                print(' Malformed configuration file (check messages above). Common reasons:')
+                print('-' * 79)
+                print('')
+                print(' o Indentation error')
+                print(' o Missing : from end of the line')
+                print(' o Non ASCII characters (use UTF8)')
+                print(' o If text contains any of :[]{}% characters it must be single-quoted '
+                      '(eg. value{1} should be \'value{1}\')\n')
 
-            # Not very good practice but we get several kind of exceptions here, I'm not even sure all of them
-            # At least: ReaderError, YmlScannerError (or something like that)
-            if hasattr(e, 'problem') and hasattr(e, 'context_mark') and hasattr(e, 'problem_mark'):
-                lines = 0
-                if e.problem is not None:
-                    print(' Reason: %s\n' % e.problem)
-                    if e.problem == 'mapping values are not allowed here':
-                        print(' ----> MOST LIKELY REASON: Missing : from end of the line!')
+                # Not very good practice but we get several kind of exceptions here, I'm not even sure all of them
+                # At least: ReaderError, YmlScannerError (or something like that)
+                if hasattr(e, 'problem') and hasattr(e, 'context_mark') and hasattr(e, 'problem_mark'):
+                    lines = 0
+                    if e.problem is not None:
+                        print(' Reason: %s\n' % e.problem)
+                        if e.problem == 'mapping values are not allowed here':
+                            print(' ----> MOST LIKELY REASON: Missing : from end of the line!')
+                            print('')
+                    if e.context_mark is not None:
+                        print(' Check configuration near line %s, column %s' % (
+                            e.context_mark.line, e.context_mark.column))
+                        lines += 1
+                    if e.problem_mark is not None:
+                        print(' Check configuration near line %s, column %s' % (
+                            e.problem_mark.line, e.problem_mark.column))
+                        lines += 1
+                    if lines:
                         print('')
-                if e.context_mark is not None:
-                    print(' Check configuration near line %s, column %s' % (e.context_mark.line, e.context_mark.column))
-                    lines += 1
-                if e.problem_mark is not None:
-                    print(' Check configuration near line %s, column %s' % (e.problem_mark.line, e.problem_mark.column))
-                    lines += 1
-                if lines:
-                    print('')
-                if lines == 1:
-                    print(' Fault is almost always in this or previous line\n')
-                if lines == 2:
-                    print(' Fault is almost always in one of these lines or previous ones\n')
+                    if lines == 1:
+                        print(' Fault is almost always in this or previous line\n')
+                    if lines == 2:
+                        print(' Fault is almost always in one of these lines or previous ones\n')
 
             # When --debug escalate to full stacktrace
-            if self.options.debug:
+            if self.options.debug or not output_to_console:
                 raise
             raise ValueError('Config file is not valid YAML')
 
@@ -602,6 +626,7 @@ class Manager(object):
 
         :raises: `ValueError` and rolls back to previous config if the provided config is not valid.
         """
+        new_user_config = config
         old_config = self.config
         try:
             self.config = self.validate_config(config)
@@ -612,24 +637,39 @@ class Manager(object):
             self.config = old_config
             raise
         log.debug('New config data loaded.')
+        self.user_config = copy.deepcopy(new_user_config)
         fire_event('manager.config_updated', self)
 
+    def backup_config(self):
+        backup_path = os.path.join(self.config_base,
+                                   '%s-%s.bak' % (self.config_name, datetime.now().strftime('%y%m%d%H%M%S')))
+
+        log.debug('backing up old config to %s before new save' % backup_path)
+        try:
+            shutil.copy(self.config_path, backup_path)
+        except (OSError, IOError) as e:
+            log.warning('Config backup creation failed: %s', str(e))
+            raise
+        return backup_path
+
     def save_config(self):
         """Dumps current config to yaml config file"""
+        # TODO: Only keep x number of backups..
+
         # Back up the user's current config before overwriting
-        backup_path = os.path.join(self.config_base,
-                                   '%s-%s.bak' % (self.config_name, datetime.now().strftime('%y%m%d%H%M%S')))
-        log.debug('backing up old config to %s before new save' % backup_path)
-        shutil.copy(self.config_path, backup_path)
+        try:
+            self.backup_config()
+        except (OSError, IOError):
+            return
         with open(self.config_path, 'w') as config_file:
-            config_file.write(yaml.dump(self.config, default_flow_style=False))
+            config_file.write(yaml.dump(self.user_config, default_flow_style=False))
 
     def config_changed(self):
         """Makes sure that all tasks will have the config_modified flag come out true on the next run.
         Useful when changing the db and all tasks need to be completely reprocessed."""
         from flexget.task import config_changed
-        for task in self.tasks:
-            config_changed(task)
+        config_changed()
+        fire_event('manager.config_updated', self)
 
     def validate_config(self, config=None):
         """
@@ -667,7 +707,7 @@ class Manager(object):
             self.database_uri = 'sqlite:///%s' % filename
 
         if self.db_filename and not os.path.exists(self.db_filename):
-            log.verbose('Creating new database %s ...' % self.db_filename)
+            log.verbose('Creating new database %s - DO NOT INTERUPT ...' % self.db_filename)
 
         # fire up the engine
         log.debug('Connecting to: %s' % self.database_uri)
@@ -675,12 +715,13 @@ class Manager(object):
             self.engine = sqlalchemy.create_engine(self.database_uri,
                                                    echo=self.options.debug_sql,
                                                    connect_args={'check_same_thread': False, 'timeout': 10})
-        except ImportError:
-            print('FATAL: Unable to use SQLite. Are you running Python 2.5 - 2.7 ?\n'
+        except ImportError as e:
+            print('FATAL: Unable to use SQLite. Are you running Python 2.7, 3.3 or newer ?\n'
                   'Python should normally have SQLite support built in.\n'
                   'If you\'re running correct version of Python then it is not equipped with SQLite.\n'
                   'You can try installing `pysqlite`. If you have compiled python yourself, '
-                  'recompile it with SQLite support.', file=sys.stderr)
+                  'recompile it with SQLite support.\n'
+                  'Error: %s' % e, file=sys.stderr)
             sys.exit(1)
         Session.configure(bind=self.engine)
         # create all tables, doesn't do anything to existing tables
@@ -701,18 +742,18 @@ class Manager(object):
         """
         if self.lockfile and os.path.exists(self.lockfile):
             result = {}
-            with open(self.lockfile) as f:
+            with io.open(self.lockfile, encoding='utf-8') as f:
                 lines = [l for l in f.readlines() if l]
             for line in lines:
                 try:
-                    key, value = line.split(b':', 1)
+                    key, value = line.split(':', 1)
                 except ValueError:
                     log.debug('Invalid line in lock file: %s' % line)
                     continue
                 result[key.strip().lower()] = value.strip()
             for key in result:
                 if result[key].isdigit():
-                    result[key] = int(result[key])
+                    result[key] = native_int(result[key])
             result.setdefault('pid', None)
             if not result['pid']:
                 log.error('Invalid lock file. Make sure FlexGet is not running, then delete it.')
@@ -749,7 +790,7 @@ class Manager(object):
             if not self._has_lock:
                 # Exit if there is an existing lock.
                 if self.check_lock():
-                    with open(self.lockfile) as f:
+                    with io.open(self.lockfile, encoding='utf-8') as f:
                         pid = f.read()
                     print('Another process (%s) is running, will exit.' % pid.split('\n')[0], file=sys.stderr)
                     print('If you\'re sure there is no other instance running, delete %s' % self.lockfile,
@@ -769,11 +810,11 @@ class Manager(object):
 
     def write_lock(self, ipc_info=None):
         assert self._has_lock
-        with open(self.lockfile, 'w') as f:
-            f.write(b'PID: %s\n' % os.getpid())
+        with io.open(self.lockfile, 'w', encoding='utf-8') as f:
+            f.write('PID: %s\n' % os.getpid())
             if ipc_info:
                 for key in sorted(ipc_info):
-                    f.write(b'%s: %s\n' % (key, ipc_info[key]))
+                    f.write('%s: %s\n' % (key, ipc_info[key]))
 
     def release_lock(self):
         if os.path.exists(self.lockfile):
@@ -825,12 +866,13 @@ class Manager(object):
         # redirect standard file descriptors
         sys.stdout.flush()
         sys.stderr.flush()
-        si = file('/dev/null', 'r')
-        so = file('/dev/null', 'a+')
-        se = file('/dev/null', 'a+', 0)
+        si = open(os.devnull, 'r')
+        so = open(os.devnull, 'ab+')
+        se = open(os.devnull, 'ab+', 0)
         os.dup2(si.fileno(), sys.stdin.fileno())
         os.dup2(so.fileno(), sys.stdout.fileno())
         os.dup2(se.fileno(), sys.stderr.fileno())
+
         # If we have a lock, update the lock file with our new pid
         if self._has_lock:
             self.write_lock()
@@ -850,12 +892,10 @@ class Manager(object):
         expired = self.persist.get('last_cleanup', datetime(1900, 1, 1)) < datetime.now() - DB_CLEANUP_INTERVAL
         if force or expired:
             log.info('Running database cleanup.')
-            session = Session()
-            try:
+            with Session() as session:
                 fire_event('manager.db_cleanup', self, session)
-                session.commit()
-            finally:
-                session.close()
+            # Try to VACUUM after cleanup
+            fire_event('manager.db_vacuum', self)
             # Just in case some plugin was overzealous in its cleaning, mark the config changed
             self.config_changed()
             self.persist['last_cleanup'] = datetime.now()
@@ -870,20 +910,41 @@ class Manager(object):
         """
         if not self.initialized:
             raise RuntimeError('Cannot shutdown manager that was never initialized.')
+        fire_event('manager.shutdown_requested', self)
         self.task_queue.shutdown(finish_queue)
 
     def _shutdown(self):
         """Runs when the manager is done processing everything."""
+        if self.ipc_server:
+            self.ipc_server.shutdown()
         fire_event('manager.shutdown', self)
         if not self.unit_test:  # don't scroll "nosetests" summary results when logging is enabled
             log.debug('Shutting down')
         self.engine.dispose()
         # remove temporary database used in test mode
         if self.options.test:
-            if not 'test' in self.db_filename:
+            if 'test' not in self.db_filename:
                 raise Exception('trying to delete non test database?')
             if self._has_lock:
                 os.remove(self.db_filename)
                 log.info('Removed test database')
-        if not self.unit_test:  # don't scroll "nosetests" summary results when logging is enabled
-            log.debug('Shutdown completed')
+        global manager
+        manager = None
+
+    def crash_report(self):
+        """
+        This should be called when handling an unexpected exception. Will create a new log file containing the last 50
+        debug messages as well as the crash traceback.
+        """
+        if not self.unit_test:
+            filename = os.path.join(self.config_base, datetime.now().strftime('crash_report.%Y.%m.%d.%H%M%S%f.log'))
+            with codecs.open(filename, 'w', encoding='utf-8') as outfile:
+                outfile.writelines(logger.debug_buffer)
+                traceback.print_exc(file=outfile)
+            log.critical('An unexpected crash has occurred. Writing crash report to %s. '
+                         'Please verify you are running the latest version of flexget by using "flexget -V" '
+                         'from CLI or by using version_checker plugin'
+                         ' at http://flexget.com/wiki/Plugins/version_checker. You are currently using'
+                         ' version %s', filename, get_current_flexget_version())
+        log.debug('Traceback:', exc_info=True)
+        return traceback.format_exc()
