@@ -1,21 +1,20 @@
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
+import sys
 import copy
 import random
-import socket
 import string
-import sys
-from argparse import ArgumentParser as ArgParser
-from argparse import (_VersionAction, Action, ArgumentError, ArgumentTypeError, Namespace, PARSER, REMAINDER, SUPPRESS,
+from argparse import ArgumentParser as ArgParser, _UNRECOGNIZED_ARGS_ATTR
+from argparse import (_VersionAction, Action, ArgumentError, Namespace, PARSER, REMAINDER, SUPPRESS,
                       _SubParsersAction)
 
-import pkg_resources
-
 import flexget
+
 from flexget.entry import Entry
 from flexget.event import fire_event
-from flexget.logger import console
 from flexget.utils import requests
+from flexget.utils.tools import get_latest_flexget_version_number, get_current_flexget_version
 
 _UNSET = object()
 
@@ -24,7 +23,12 @@ core_parser = None
 
 def unicode_argv():
     """Like sys.argv, but decodes all arguments."""
-    return [arg.decode(sys.getfilesystemencoding()) for arg in sys.argv]
+    args = []
+    for arg in sys.argv:
+        if isinstance(arg, bytes):
+            arg = arg.decode(sys.getfilesystemencoding())
+        args.append(arg)
+    return args
 
 
 def get_parser(command=None):
@@ -53,30 +57,34 @@ def register_command(command, callback, **kwargs):
 
 def required_length(nmin, nmax):
     """Generates a custom Action to validate an arbitrary range of arguments."""
+
     class RequiredLength(Action):
         def __call__(self, parser, args, values, option_string=None):
             if not nmin <= len(values) <= nmax:
                 raise ArgumentError(self, 'requires between %s and %s arguments' % (nmin, nmax))
             setattr(args, self.dest, values)
+
     return RequiredLength
 
 
 class VersionAction(_VersionAction):
     """Action to print the current version. Also checks latest release revision."""
+
     def __call__(self, parser, namespace, values, option_string=None):
+        from flexget.terminal import console
+        current = get_current_flexget_version()
+        latest = get_latest_flexget_version_number()
+
         # Print the version number
-        console('%s' % self.version)
+        console('%s' % get_current_flexget_version())
         # Check for latest version from server
-        try:
-            page = requests.get('http://download.flexget.com/latestversion')
-        except requests.RequestException:
-            console('Error getting latest version number from download.flexget.com')
-        else:
-            ver = page.text.strip()
-            if self.version == ver:
+        if latest:
+            if current == latest:
                 console('You are on the latest release.')
             else:
-                console('Latest release: %s' % ver)
+                console('Latest release: %s' % latest)
+        else:
+            console('Error getting latest version number from https://pypi.python.org/pypi/FlexGet')
         parser.exit()
 
 
@@ -108,17 +116,19 @@ class InjectAction(Action):
         if values:
             kwargs['url'] = values.pop(0)
         else:
-            kwargs['url'] = 'http://localhost/inject/%s' % ''.join(random.sample(string.letters + string.digits, 30))
+            kwargs['url'] = 'http://localhost/inject/%s' % ''.join(random.sample(string.ascii_letters + string.digits, 30))
         if 'force' in [v.lower() for v in values]:
             kwargs['immortal'] = True
         entry = Entry(**kwargs)
         if 'accept' in [v.lower() for v in values]:
             entry.accept(reason='accepted by --inject')
-        setattr(namespace, self.dest, [entry])
+        existing = getattr(namespace, self.dest, None) or []
+        setattr(namespace, self.dest, existing + [entry])
 
 
 class ParseExtrasAction(Action):
     """This action will take extra arguments, and parser them with a different parser."""
+
     def __init__(self, option_strings, parser, help=None, metavar=None, dest=None, required=False):
         if metavar is None:
             metavar = '<%s arguments>' % parser.prog
@@ -161,7 +171,7 @@ class ScopedNamespace(Namespace):
         return object.__setattr__(self, key, value)
 
     def __iter__(self):
-        return (i for i in self.__dict__.iteritems() if i[0] != '__parent__')
+        return (i for i in self.__dict__.items() if i[0] != '__parent__')
 
     def __copy__(self):
         new = self.__class__()
@@ -198,6 +208,9 @@ class NestedSubparserAction(_SubParsersAction):
                 setattr(namespace, self.dest, parser_name)
                 delattr(subnamespace, self.dest)
             setattr(namespace, parser_name, subnamespace)
+            # Propagate unrecognized arguments back to parent namespace
+            vars(namespace).setdefault(_UNRECOGNIZED_ARGS_ATTR, [])
+            getattr(namespace, _UNRECOGNIZED_ARGS_ATTR).extend(getattr(subnamespace, _UNRECOGNIZED_ARGS_ATTR))
         else:
             super(NestedSubparserAction, self).__call__(parser, namespace, values, option_string)
 
@@ -258,7 +271,7 @@ class ArgumentParser(ArgParser):
                     self.set_post_defaults(**parent.post_defaults)
 
     def add_argument(self, *args, **kwargs):
-        if isinstance(kwargs.get('nargs'), basestring) and '-' in kwargs['nargs']:
+        if isinstance(kwargs.get('nargs'), str) and '-' in kwargs['nargs']:
             # Handle a custom range of arguments
             min, max = kwargs['nargs'].split('-')
             min, max = int(min), int(max)
@@ -289,7 +302,7 @@ class ArgumentParser(ArgParser):
     def error(self, msg):
         raise ParserError(msg, self)
 
-    def parse_args(self, args=None, namespace=None, raise_errors=False, file=None):
+    def parse_args(self, args=None, namespace=None, raise_errors=False, file=None):  # pylint: disable=W0221
         """
         :param raise_errors: If this is true, errors will be raised as `ParserError`s instead of calling sys.exit
         """
@@ -364,7 +377,7 @@ class ArgumentParser(ArgParser):
         cgitb.enable(format="text")
 
 
-# This will hold just the arguments directly for Manager. Webui needs this clean, to build its parser.
+# This will hold just the arguments directly for Manager.
 manager_parser = ArgumentParser(add_help=False)
 manager_parser.add_argument('-V', '--version', action=VersionAction, version=flexget.__version__,
                             help='Print FlexGet version and exit.')
@@ -402,6 +415,7 @@ class CoreArgumentParser(ArgumentParser):
     Warning: Only gets plugin arguments if instantiated after plugins have been loaded.
 
     """
+
     def __init__(self, **kwargs):
         kwargs.setdefault('parents', [manager_parser])
         kwargs.setdefault('prog', 'flexget')
@@ -434,46 +448,13 @@ class CoreArgumentParser(ArgumentParser):
         daemon_parser.add_subparsers(title='actions', metavar='<action>', dest='action')
         start_parser = daemon_parser.add_subparser('start', help='start the daemon')
         start_parser.add_argument('-d', '--daemonize', action='store_true', help=daemonize_help)
+        start_parser.add_argument('--autoreload-config', action='store_true',
+                                  help='automatically reload the config from disk if the daemon detects any changes')
         stop_parser = daemon_parser.add_subparser('stop', help='shutdown the running daemon')
         stop_parser.add_argument('--wait', action='store_true',
                                  help='wait for all queued tasks to finish before stopping daemon')
         daemon_parser.add_subparser('status', help='check if a daemon is running')
-        daemon_parser.add_subparser('reload', help='causes a running daemon to reload the config from disk')
-
-        # The parser for the webui
-        # Hide the webui command if deps aren't available
-        webui_kwargs = {}
-        try:
-            pkg_resources.require('flexget[webui]')
-            webui_kwargs['help'] = 'run continuously, with a web interface to configure and interact with the daemon'
-        except pkg_resources.DistributionNotFound:
-            pass
-        webui_parser = self.add_subparser('webui', **webui_kwargs)
-
-        def ip_type(value):
-            try:
-                socket.inet_aton(value)
-            except socket.error:
-                raise ArgumentTypeError('must be a valid ip address to bind to')
-            return value
-
-        webui_parser.add_argument('--bind', type=ip_type, default='0.0.0.0', metavar='IP',
-                                  help='IP address to bind to when serving the web interface [default: %(default)s]')
-        webui_parser.add_argument('--port', type=int, default=5050,
-                                  help='run FlexGet webui on port [default: %(default)s]')
-        webui_parser.add_argument('-d', '--daemonize', action='store_true', help=daemonize_help)
-
-        # TODO: move these to authentication plugin?
-        webui_parser.add_argument('--no-auth', action='store_true',
-                                  help='runs without authentication required (dangerous)')
-        webui_parser.add_argument('--no-local-auth', action='store_true',
-                                  help='runs without any authentication required when accessed from localhost')
-        webui_parser.add_argument('--username', help='username needed to login [default: flexget]')
-        webui_parser.add_argument('--password', help='password needed to login [default: flexget]')
-
-        # enable flask autoreloading (development)
-        webui_parser.add_argument('--autoreload', action='store_true', help=SUPPRESS)
-        webui_parser.set_defaults(loglevel='info')
+        daemon_parser.add_subparser('reload-config', help='causes a running daemon to reload the config from disk')
 
     def add_subparsers(self, **kwargs):
         # The subparsers should not be CoreArgumentParsers
@@ -488,4 +469,6 @@ class CoreArgumentParser(ArgumentParser):
             if hasattr(result, 'execute'):
                 exec_options.__dict__.update(result.execute.__dict__)
             result.execute = exec_options
+        # Set the 'allow_manual' flag to True for any usage of the CLI
+        setattr(result, 'allow_manual', True)
         return result
