@@ -1,17 +1,18 @@
-""" Plugin Loading & Management.
-"""
-
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.moves.urllib.error import HTTPError, URLError
+from future.utils import python_2_unicode_compatible
 
 import logging
 import os
 import re
-import sys
 import time
 import warnings
-from itertools import ifilter
+import pkg_resources
+from functools import total_ordering
+from http.client import BadStatusLine
 
-from path import path
+from path import Path
 from requests import RequestException
 
 from flexget import plugins as plugins_pkg
@@ -21,11 +22,8 @@ from flexget.event import fire_event, remove_event_handlers
 
 log = logging.getLogger('plugin')
 
-__all__ = ['PluginWarning', 'PluginError', 'register_plugin', 'register_parser_option', 'register_task_phase',
-           'get_plugin_by_name', 'get_plugins_by_group', 'get_plugin_keywords', 'get_plugins_by_phase',
-           'get_phases_by_plugin', 'internet', 'priority']
-
 
+@python_2_unicode_compatible
 class DependencyError(Exception):
     """Plugin depends on other plugin, but it cannot be loaded.
 
@@ -60,11 +58,10 @@ class DependencyError(Exception):
 
     def __str__(self):
         return '<DependencyError(issued_by=%r,missing=%r,message=%r,silent=%r)>' % \
-            (self.issued_by, self.missing, self.message, self.silent)
+               (self.issued_by, self.missing, self.message, self.silent)
 
 
 class RegisterException(Exception):
-
     def __init__(self, value):
         super(RegisterException, self).__init__()
         self.value = value
@@ -73,8 +70,8 @@ class RegisterException(Exception):
         return repr(self.value)
 
 
+@python_2_unicode_compatible
 class PluginWarning(Warning):
-
     def __init__(self, value, logger=log, **kwargs):
         super(PluginWarning, self).__init__()
         self.value = value
@@ -82,28 +79,22 @@ class PluginWarning(Warning):
         self.kwargs = kwargs
 
     def __str__(self):
-        return unicode(self).encode('utf-8')
-
-    def __unicode__(self):
         return self.value
 
 
+@python_2_unicode_compatible
 class PluginError(Exception):
-
     def __init__(self, value, logger=log, **kwargs):
         super(PluginError, self).__init__()
         # Value is expected to be a string
-        if not isinstance(value, basestring):
-            value = unicode(value)
+        if not isinstance(value, str):
+            value = str(value)
         self.value = value
         self.log = logger
         self.kwargs = kwargs
 
     def __str__(self):
-        return unicode(self).encode('utf-8')
-
-    def __unicode__(self):
-        return unicode(self.value)
+        return self.value
 
 
 # TODO: move to utils or somewhere more appropriate
@@ -123,16 +114,14 @@ class internet(object):
     def __call__(self, func):
 
         def wrapped_func(*args, **kwargs):
-            from httplib import BadStatusLine
-            import urllib2
             try:
                 return func(*args, **kwargs)
             except RequestException as e:
                 log.debug('decorator caught RequestException. handled traceback:', exc_info=True)
                 raise PluginError('RequestException: %s' % e)
-            except urllib2.HTTPError as e:
+            except HTTPError as e:
                 raise PluginError('HTTPError %s' % e.code, self.log)
-            except urllib2.URLError as e:
+            except URLError as e:
                 log.debug('decorator caught urlerror. handled traceback:', exc_info=True)
                 raise PluginError('URLError %s' % e.reason, self.log)
             except BadStatusLine:
@@ -148,6 +137,7 @@ class internet(object):
                 elif hasattr(e, 'code'):
                     raise PluginError('The server couldn\'t fulfill the request. Error code: %s' % e.code, self.log)
                 raise PluginError('IOError when connecting to server: %s' % e, self.log)
+
         return wrapped_func
 
 
@@ -157,15 +147,15 @@ def priority(value):
     def decorator(target):
         target.priority = value
         return target
+
     return decorator
 
+
 DEFAULT_PRIORITY = 128
 
-plugin_contexts = ['task', 'root']
-
 # task phases, in order of their execution; note that this can be extended by
 # registering new phases at runtime
-task_phases = ['start', 'input', 'metainfo', 'filter', 'download', 'modify', 'output', 'learn', 'exit']
+task_phases = ['prepare', 'start', 'input', 'metainfo', 'filter', 'download', 'modify', 'output', 'learn', 'exit']
 
 # map phase names to method names
 phase_methods = {
@@ -186,7 +176,11 @@ _new_phase_queue = {}
 
 
 def register_task_phase(name, before=None, after=None):
-    """Adds a new task phase to the available phases."""
+    """
+    Adds a new task phase to the available phases.
+
+    :param suppress_abort: If True, errors during this phase will be suppressed, and not affect task result.
+    """
     if before and after:
         raise RegisterException('You can only give either before or after for a phase.')
     if not before and not after:
@@ -195,9 +189,9 @@ def register_task_phase(name, before=None, after=None):
         raise RegisterException('Phase %s already exists.' % name)
 
     def add_phase(phase_name, before, after):
-        if not before is None and not before in task_phases:
+        if before is not None and before not in task_phases:
             return False
-        if not after is None and not after in task_phases:
+        if after is not None and after not in task_phases:
             return False
         # add method name to phase -> method lookup table
         phase_methods[phase_name] = 'on_task_' + phase_name
@@ -212,11 +206,12 @@ def register_task_phase(name, before=None, after=None):
     if not add_phase(name, before, after):
         _new_phase_queue[name] = [before, after]
 
-    for phase_name, args in _new_phase_queue.items():
+    for phase_name, args in list(_new_phase_queue.items()):
         if add_phase(phase_name, *args):
             del _new_phase_queue[phase_name]
 
 
+@total_ordering
 class PluginInfo(dict):
     """
     Allows accessing key/value pairs of this dictionary subclass via
@@ -225,33 +220,33 @@ class PluginInfo(dict):
     # Counts duplicate registrations
     dupe_counter = 0
 
-    def __init__(self, plugin_class, name=None, groups=None, builtin=False, debug=False, api_ver=1,
-                 contexts=None, category=None):
+    def __init__(self, plugin_class, name=None, interfaces=None, builtin=False, debug=False, api_ver=1, category=None,
+                 groups=None):
         """
         Register a plugin.
 
         :param plugin_class: The plugin factory.
         :param string name: Name of the plugin (if not given, default to factory class name in underscore form).
-        :param list groups: Groups this plugin belongs to.
+        :param list interfaces: Interfaces this plugin implements.
         :param bool builtin: Auto-activated?
         :param bool debug: True if plugin is for debugging purposes.
         :param int api_ver: Signature of callback hooks (1=task; 2=task,config).
-        :param list contexts: List of where this plugin is configurable. Can be 'task', 'root', or None
         :param string category: The type of plugin. Can be one of the task phases.
             Defaults to the package name containing the plugin.
+        :param groups: DEPRECATED
         """
         dict.__init__(self)
 
-        if groups is None:
-            groups = []
+        if interfaces is None:
+            interfaces = ['task']
+        if groups is not None:
+            warnings.warn('The `groups` argument for plugin registration is deprecated. `interfaces` should be used '
+                          'instead. Plugin %s' % name, DeprecationWarning, stacklevel=2)
+            interfaces.extend(groups)
         if name is None:
             # Convention is to take camel-case class name and rewrite it to an underscore form,
             # e.g. 'PluginName' to 'plugin_name'
             name = re.sub('[A-Z]+', lambda i: '_' + i.group(0).lower(), plugin_class.__name__).lstrip('_')
-        if contexts is None:
-            contexts = ['task']
-        elif isinstance(contexts, basestring):
-            contexts = [contexts]
         if category is None and plugin_class.__module__.startswith('flexget.plugins'):
             # By default look at the containing package of the plugin.
             category = plugin_class.__module__.split('.')[-2]
@@ -263,10 +258,9 @@ class PluginInfo(dict):
         # Set basic info attributes
         self.api_ver = api_ver
         self.name = name
-        self.groups = groups
+        self.interfaces = interfaces
         self.builtin = builtin
         self.debug = debug
-        self.contexts = contexts
         self.category = category
         self.phase_handlers = {}
 
@@ -275,8 +269,8 @@ class PluginInfo(dict):
 
         if self.name in plugins:
             PluginInfo.dupe_counter += 1
-            log.critical('Error while registering plugin %s. A plugin with the same name is already registered' %
-                         self.name)
+            log.critical('Error while registering plugin %s. '
+                         'A plugin with the same name is already registered', self.name)
         else:
             plugins[self.name] = self
 
@@ -313,7 +307,7 @@ class PluginInfo(dict):
 
     def build_phase_handlers(self):
         """(Re)build phase_handlers in this plugin"""
-        for phase, method_name in phase_methods.iteritems():
+        for phase, method_name in phase_methods.items():
             if phase in self.phase_handlers:
                 continue
             if hasattr(self.instance, method_name):
@@ -341,6 +335,15 @@ class PluginInfo(dict):
     def __str__(self):
         return '<PluginInfo(name=%s)>' % self.name
 
+    def _is_valid_operand(self, other):
+        return hasattr(other, 'name')
+
+    def __eq__(self, other):
+        return self.name == other.name
+
+    def __lt__(self, other):
+        return self.name < other.name
+
     __repr__ = __str__
 
 
@@ -366,21 +369,28 @@ def _get_standard_plugins_path():
     return paths
 
 
+def _check_phase_queue():
+    if _new_phase_queue:
+        for phase, args in _new_phase_queue.items():
+            log.error('Plugin %s requested new phase %s, but it could not be created at requested '
+                      'point (before, after). Plugin is not working properly.', args[0], phase)
+
+
 def _load_plugins_from_dirs(dirs):
     """
     :param list dirs: Directories from where plugins are loaded from
     """
 
-    log.debug('Trying to load plugins from: %s' % dirs)
-    dirs = [path(d) for d in dirs if os.path.isdir(d)]
+    log.debug('Trying to load plugins from: %s', dirs)
+    dirs = [Path(d) for d in dirs if os.path.isdir(d)]
     # add all dirs to plugins_pkg load path so that imports work properly from any of the plugin dirs
-    plugins_pkg.__path__ = map(_strip_trailing_sep, dirs)
+    plugins_pkg.__path__ = list(map(_strip_trailing_sep, dirs))
     for plugins_dir in dirs:
         for plugin_path in plugins_dir.walkfiles('*.py'):
             if plugin_path.name == '__init__.py':
                 continue
             # Split the relative path from the plugins dir to current file's parent dir to find subpackage names
-            plugin_subpackages = filter(None, plugin_path.relpath(plugins_dir).parent.splitall())
+            plugin_subpackages = [_f for _f in plugin_path.relpath(plugins_dir).parent.splitall() if _f]
             module_name = '.'.join([plugins_pkg.__name__] + plugin_subpackages + [plugin_path.namebase])
             try:
                 __import__(module_name)
@@ -388,28 +398,46 @@ def _load_plugins_from_dirs(dirs):
                 if e.has_message():
                     msg = e.message
                 else:
-                    msg = 'Plugin `%s` requires `%s` to load.' % (e.issued_by or module_name, e.missing or 'N/A')
+                    msg = 'Plugin `%s` requires `%s` to load.', e.issued_by or module_name, e.missing or 'N/A'
                 if not e.silent:
                     log.warning(msg)
                 else:
                     log.debug(msg)
-            except ImportError as e:
-                log.critical('Plugin `%s` failed to import dependencies' % module_name)
-                log.exception(e)
+            except ImportError:
+                log.critical('Plugin `%s` failed to import dependencies', module_name, exc_info=True)
             except ValueError as e:
                 # Debugging #2755
                 log.error('ValueError attempting to import `%s` (from %s): %s', module_name, plugin_path, e)
-            except Exception as e:
-                log.critical('Exception while loading plugin %s' % module_name)
-                log.exception(e)
+            except Exception:
+                log.critical('Exception while loading plugin %s', module_name, exc_info=True)
                 raise
             else:
-                log.trace('Loaded module %s from %s' % (module_name, plugin_path))
+                log.trace('Loaded module %s from %s', module_name, plugin_path)
+    _check_phase_queue()
 
-    if _new_phase_queue:
-        for phase, args in _new_phase_queue.iteritems():
-            log.error('Plugin %s requested new phase %s, but it could not be created at requested '
-                      'point (before, after). Plugin is not working properly.' % (args[0], phase))
+
+def _load_plugins_from_packages():
+    """Load plugins installed via PIP"""
+    for entrypoint in pkg_resources.iter_entry_points('FlexGet.plugins'):
+        try:
+            plugin_module = entrypoint.load()
+        except DependencyError as e:
+            if e.has_message():
+                msg = e.message
+            else:
+                msg = 'Plugin `%s` requires `%s` to load.', e.issued_by or entrypoint.module_name, e.missing or 'N/A'
+            if not e.silent:
+                log.warning(msg)
+            else:
+                log.debug(msg)
+        except ImportError:
+            log.critical('Plugin `%s` failed to import dependencies', entrypoint.module_name, exc_info=True)
+        except Exception:
+            log.critical('Exception while loading plugin %s', entrypoint.module_name, exc_info=True)
+            raise
+        else:
+            log.trace('Loaded packaged module %s from %s', entrypoint.module_name, plugin_module.__file__)
+    _check_phase_queue()
 
 
 def load_plugins(extra_dirs=None):
@@ -428,39 +456,38 @@ def load_plugins(extra_dirs=None):
     start_time = time.time()
     # Import all the plugins
     _load_plugins_from_dirs(extra_dirs)
+    _load_plugins_from_packages()
     # Register them
     fire_event('plugin.register')
     # Plugins should only be registered once, remove their handlers after
     remove_event_handlers('plugin.register')
     # After they have all been registered, instantiate them
-    for plugin in plugins.values():
+    for plugin in list(plugins.values()):
         plugin.initialize()
     took = time.time() - start_time
     plugins_loaded = True
-    log.debug('Plugins took %.2f seconds to load' % took)
+    log.debug('Plugins took %.2f seconds to load. %s plugins in registry.', took, len(plugins.keys()))
 
 
-def get_plugins(phase=None, group=None, context=None, category=None, name=None, min_api=None):
+def get_plugins(phase=None, interface=None, category=None, name=None, min_api=None):
     """
     Query other plugins characteristics.
 
     :param string phase: Require phase
-    :param string group: Plugin must belong to this group.
-    :param string context: Where plugin is configured, eg. (root, task)
+    :param string interface: Plugin must implement this interface.
     :param string category: Type of plugin, phase names.
     :param string name: Name of the plugin.
     :param int min_api: Minimum api version.
     :return: List of PluginInfo instances.
     :rtype: list
     """
+
     def matches(plugin):
         if phase is not None and phase not in phase_methods:
             raise ValueError('Unknown phase %s' % phase)
-        if phase and not phase in plugin.phase_handlers:
+        if phase and phase not in plugin.phase_handlers:
             return False
-        if group and not group in plugin.groups:
-            return False
-        if context and not context in plugin.contexts:
+        if interface and interface not in plugin.interfaces:
             return False
         if category and not category == plugin.category:
             return False
@@ -469,7 +496,8 @@ def get_plugins(phase=None, group=None, context=None, category=None, name=None,
         if min_api is not None and plugin.api_ver < min_api:
             return False
         return True
-    return ifilter(matches, plugins.itervalues())
+
+    return filter(matches, iter(plugins.values()))
 
 
 def plugin_schemas(**kwargs):
@@ -484,42 +512,18 @@ def plugin_schemas(**kwargs):
 config_schema.register_schema('/schema/plugins', plugin_schemas)
 
 
-def get_plugins_by_phase(phase):
-    """
-    .. deprecated:: 1.0.3328
-       Use :func:`get_plugins` instead
-
-    Return an iterator over all plugins that hook :phase:
-    """
-    warnings.warn('Deprecated API', DeprecationWarning, stacklevel=2)
-    if not phase in phase_methods:
-        raise Exception('Unknown phase %s' % phase)
-    return get_plugins(phase=phase)
-
-
 def get_phases_by_plugin(name):
     """Return all phases plugin :name: hooks"""
     return list(get_plugin_by_name(name).phase_handlers)
 
 
-def get_plugins_by_group(group):
-    """
-    .. deprecated:: 1.0.3328
-       Use :func:`get_plugins` instead
-
-    Return an iterator over all plugins with in specified group.
-    """
-    warnings.warn('Deprecated API', DeprecationWarning, stacklevel=2)
-    return get_plugins(group=group)
-
-
 def get_plugin_keywords():
     """Return iterator over all plugin keywords."""
-    return plugins.iterkeys()
+    return iter(plugins.keys())
 
 
 def get_plugin_by_name(name, issued_by='???'):
     """Get plugin by name, preferred way since this structure may be changed at some point."""
-    if not name in plugins:
+    if name not in plugins:
         raise DependencyError(issued_by=issued_by, missing=name, message='Unknown plugin %s' % name)
     return plugins[name]
