@@ -1,9 +1,11 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import options
 from flexget.event import event
-from flexget.logger import console
+from flexget.terminal import console
 from flexget.manager import Session
 
 log = logging.getLogger('perftests')
@@ -35,7 +37,7 @@ def imdb_query(session):
 
     log.info('Getting imdb_urls ...')
     # query so that we avoid loading whole object (maybe cached?)
-    for id, url in session.execute(select([Movie.id, Movie.url])):
+    for _, url in session.execute(select([Movie.id, Movie.url])):
         imdb_urls.append(url)
     log.info('Got %i urls from database' % len(imdb_urls))
     if not imdb_urls:
@@ -52,19 +54,19 @@ def imdb_query(session):
     for index, url in enumerate(imdb_urls):
         bar.update(index)
 
-        #movie = session.query(Movie).filter(Movie.url == url).first()
-        #movie = session.query(Movie).options(subqueryload(Movie.genres)).filter(Movie.url == url).one()
+        # movie = session.query(Movie).filter(Movie.url == url).first()
+        # movie = session.query(Movie).options(subqueryload(Movie.genres)).filter(Movie.url == url).one()
 
-        movie = session.query(Movie).\
+        movie = session.query(Movie). \
             options(joinedload_all(Movie.genres, Movie.languages,
-            Movie.actors, Movie.directors)).\
+                                   Movie.actors, Movie.directors)). \
             filter(Movie.url == url).first()
 
         # access it's members so they're loaded
-        var = [x.name for x in movie.genres]
-        var = [x.name for x in movie.directors]
-        var = [x.name for x in movie.actors]
-        var = [x.name for x in movie.languages]
+        [x.name for x in movie.genres]
+        [x.name for x in movie.directors]
+        [x.name for x in movie.actors]
+        [x.name for x in movie.languages]
 
     log_query_count('test')
     took = time.time() - start_time
