@@ -1,3 +1,6 @@
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import argparse
 import logging
 import os
@@ -7,19 +10,17 @@ import sys
 import flexget
 from flexget import options
 from flexget.event import event
-from flexget.logger import console
+from flexget.terminal import console
 
 log = logging.getLogger('win32_service')
 
-
 try:
     import servicemanager
     import win32event
     import win32service
     import win32serviceutil
 
-
-    class AppServerSvc (win32serviceutil.ServiceFramework):
+    class AppServerSvc(win32serviceutil.ServiceFramework):
         _svc_name_ = 'FlexGet'
         _svc_display_name_ = 'FlexGet Daemon'
         _svc_description_ = 'Runs FlexGet tasks according to defined schedules'
@@ -72,7 +73,7 @@ def register_parser_arguments():
     if not sys.platform.startswith('win'):
         return
     # Still not fully working. Hidden for now.
-    parser = options.register_command('service', do_cli,  #help='set up or control a windows service for the daemon',
+    parser = options.register_command('service', do_cli,  # help='set up or control a windows service for the daemon',
                                       add_help=False)
     parser.add_argument('--help', '-h', action='store_true')
     parser.add_argument('args', nargs=argparse.REMAINDER)
