@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import hashlib
 import logging
 
@@ -9,9 +11,9 @@ from apscheduler.jobstores.sqlalchemy import SQLAlchemyJobStore
 from apscheduler.schedulers.background import BackgroundScheduler
 from apscheduler.triggers.cron import CronTrigger
 
-from flexget.config_schema import register_config_key, format_checker
+from flexget.config_schema import register_config_key, format_checker, register_schema
 from flexget.event import event
-from flexget.manager import Base
+from flexget.manager import Base, manager
 from flexget.utils import json
 
 log = logging.getLogger('scheduler')
@@ -29,9 +31,12 @@ def is_cron_schedule(instance):
         raise ValueError('Invalid key for schedule.')
 
 
+DEFAULT_SCHEDULES = [{'tasks': ['*'], 'interval': {'hours': 1}}]
+
 UNITS = ['minutes', 'hours', 'days', 'weeks']
 interval_schema = {
     'type': 'object',
+    'title': 'Simple Interval',
     'properties': {
         'minutes': {'type': 'number'},
         'hours': {'type': 'number'},
@@ -39,13 +44,14 @@ interval_schema = {
         'weeks': {'type': 'number'}
     },
     # Only allow one unit to be specified
-    'oneOf': [{'required': [unit]} for unit in UNITS],
-    'error_oneOf': 'Interval must be specified as one of %s' % ', '.join(UNITS),
+    'maxProperties': 1,
+    'error_maxProperties': 'Interval must be specified as one of %s' % ', '.join(UNITS),
     'additionalProperties': False
 }
 
 cron_schema = {
     'type': 'object',
+    'title': 'Advanced Cron Interval',
     'properties': {
         'year': {'type': ['integer', 'string']},
         'month': {'type': ['integer', 'string']},
@@ -55,7 +61,23 @@ cron_schema = {
         'hour': {'type': ['integer', 'string']},
         'minute': {'type': ['integer', 'string']}
     },
-    'format': 'cron_schedule',
+    'additionalProperties': False
+}
+
+schedule_schema = {
+    'type': 'object',
+    'title': 'Schedule',
+    'description': 'A schedule which runs specified tasks periodically.',
+    'properties': {
+        'tasks': {'type': ['array', 'string'], 'items': {'type': 'string'}},
+        'interval': interval_schema,
+        'schedule': cron_schema
+    },
+    'required': ['tasks'],
+    'minProperties': 2,
+    'maxProperties': 2,
+    'error_minProperties': 'Either `cron` or `interval` must be defined.',
+    'error_maxProperties': 'Either `cron` or `interval` must be defined.',
     'additionalProperties': False
 }
 
@@ -63,37 +85,34 @@ main_schema = {
     'oneOf': [
         {
             'type': 'array',
-            'items': {
-                'properties': {
-                    'tasks': {'type': ['array', 'string'], 'items': {'type': 'string'}},
-                    'interval': interval_schema,
-                    'schedule': cron_schema
-                },
-                'required': ['tasks'],
-                'oneOf': [{'required': ['schedule']}, {'required': ['interval']}],
-                'error_oneOf': 'Either `cron` or `interval` must be defined.',
-                'additionalProperties': False
-            }
+            'title': 'Enable',
+            'items': schedule_schema
         },
-        {'type': 'boolean', 'enum': [False]}
+        {
+            'type': 'boolean',
+            'title': 'Disable',
+            'description': 'Disable task schedules',
+        }
     ]
 }
 
-
 scheduler = None
+scheduler_job_map = {}
 
 
 def job_id(conf):
     """Create a unique id for a schedule item in config."""
-    return hashlib.sha1(json.dumps(conf, sort_keys=True)).hexdigest()
+    return hashlib.sha1(json.dumps(conf, sort_keys=True).encode('utf-8')).hexdigest()
 
 
 def run_job(tasks):
     """Add the execution to the queue and waits until it is finished"""
-    from flexget.manager import manager
-    finished_events = manager.execute(options={'tasks': tasks, 'cron': True}, priority=5)
-    for event in finished_events:
-        event.wait()
+    log.debug('executing tasks: %s', tasks)
+    finished_events = manager.execute(options={'tasks': tasks, 'cron': True, 'allow_manual': False}, priority=5)
+    for _, task_name, event_ in finished_events:
+        log.debug('task finished executing: %s', task_name)
+        event_.wait()
+    log.debug('all tasks in schedule finished executing')
 
 
 @event('manager.daemon.started')
@@ -118,7 +137,7 @@ def setup_scheduler(manager):
     if not timezone:
         # The default sqlalchemy jobstore does not work when there isn't a name for the local timezone.
         # Just fall back to utc in this case
-        # FlexGet #2741, upstream ticket https://bitbucket.org/agronholm/apscheduler/issue/59
+        # FlexGet #2741, upstream ticket https://github.com/agronholm/apscheduler/issues/59
         log.info('Local timezone name could not be determined. Scheduler will display times in UTC for any log'
                  'messages. To resolve this set up /etc/timezone with correct time zone name.')
         timezone = pytz.utc
@@ -131,19 +150,29 @@ def setup_jobs(manager):
     """Set up the jobs for apscheduler to run."""
     if not manager.is_daemon:
         return
+
+    global scheduler_job_map
+    scheduler_job_map = {}
+
     if 'schedules' not in manager.config:
         log.info('No schedules defined in config. Defaulting to run all tasks on a 1 hour interval.')
-    config = manager.config.get('schedules', [{'tasks': ['*'], 'interval': {'hours': 1}}])
-    if not config:  # Schedules are disabled with `schedules: no`
+    config = manager.config.get('schedules', True)
+    if config is True:
+        config = DEFAULT_SCHEDULES
+    elif not config:  # Schedules are disabled with `schedules: no`
         if scheduler.running:
             log.info('Shutting down scheduler')
             scheduler.shutdown()
         return
+    if not scheduler.running:
+        log.info('Starting scheduler')
+        scheduler.start(paused=True)
     existing_job_ids = [job.id for job in scheduler.get_jobs()]
     configured_job_ids = []
     for job_config in config:
         jid = job_id(job_config)
         configured_job_ids.append(jid)
+        scheduler_job_map[id(job_config)] = jid
         if jid in existing_job_ids:
             continue
         if 'interval' in job_config:
@@ -159,17 +188,22 @@ def setup_jobs(manager):
     for jid in existing_job_ids:
         if jid not in configured_job_ids:
             scheduler.remove_job(jid)
-    if not scheduler.running:
-        log.info('Starting scheduler')
-        scheduler.start()
+    scheduler.resume()
 
 
-@event('manager.daemon.completed')
+@event('manager.shutdown_requested')
+def shutdown_requested(manager):
+    if scheduler and scheduler.running:
+        scheduler.shutdown(wait=True)
+
+
+@event('manager.shutdown')
 def stop_scheduler(manager):
-    if scheduler.running:
+    if scheduler and scheduler.running:
         scheduler.shutdown(wait=False)
 
 
 @event('config.register')
 def register_config():
     register_config_key('schedules', main_schema)
+    register_schema('/schema/config/schedule', schedule_schema)
