@@ -1,6 +1,8 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import logging
-import posixpath
 from fnmatch import fnmatch
 
 from flexget import plugin
@@ -86,7 +88,7 @@ class FilterContentFilter(object):
                 for f in entry['torrent'].get_filelist():
                     if not best or f['size'] > best:
                         best = f['size']
-                if (100*float(best)/float(entry['torrent'].size)) < 90:
+                if (100 * float(best) / float(entry['torrent'].size)) < 90:
                     log.info('Entry %s does not have a main file, rejecting' % (entry['title']))
                     entry.reject('does not have a main file', remember=True)
                     return True
@@ -94,16 +96,18 @@ class FilterContentFilter(object):
     @plugin.priority(150)
     def on_task_modify(self, task, config):
         if task.options.test or task.options.learn:
-            log.info('Plugin is partially disabled with --test and --learn because content filename information may not be available')
-            #return
+            log.info('Plugin is partially disabled with --test and --learn '
+                     'because content filename information may not be available')
+            # return
 
         config = self.prepare_config(config)
         for entry in task.accepted:
             if self.process_entry(task, entry, config):
-                task.rerun()
-            elif not 'content_files' in entry and config.get('strict'):
+                task.rerun(plugin='content_filter')
+            elif 'content_files' not in entry and config.get('strict'):
                 entry.reject('no content files parsed for entry', remember=True)
-                task.rerun()
+                task.rerun(plugin='content_filter')
+
 
 @event('plugin.register')
 def register_plugin():
