@@ -1,8 +1,11 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import logging
 import platform
 
-from path import path
+from path import Path
 
 from flexget import plugin
 from flexget.event import event
@@ -12,7 +15,6 @@ log = logging.getLogger('exists')
 
 
 class FilterExists(object):
-
     """
         Reject entries that already exist in given path.
 
@@ -38,7 +40,7 @@ class FilterExists(object):
         config = self.prepare_config(config)
         filenames = {}
         for folder in config:
-            folder = path(folder).expanduser()
+            folder = Path(folder).expanduser()
             if not folder.exists():
                 raise plugin.PluginWarning('Path %s does not exist' % folder, log)
             for p in folder.walk(errors='ignore'):
@@ -49,13 +51,14 @@ class FilterExists(object):
                 filenames[key] = p
         for entry in task.accepted:
             # priority is: filename, location (filename only), title
-            name = path(entry.get('filename', entry.get('location', entry['title']))).name
+            name = Path(entry.get('filename', entry.get('location', entry['title']))).name
             if platform.system() == 'Windows':
                 name = name.lower()
             if name in filenames:
                 log.debug('Found %s in %s' % (name, filenames[name]))
                 entry.reject('exists in %s' % filenames[name])
 
+
 @event('plugin.register')
 def register_plugin():
     plugin.register(FilterExists, 'exists', api_ver=2)
