@@ -1,8 +1,10 @@
 from __future__ import unicode_literals, division, absolute_import
-import copy
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import logging
 
-from path import path
+from path import Path
 
 from flexget import plugin
 from flexget.event import event
@@ -77,7 +79,7 @@ class FilterExistsSeries(object):
             # make new parser from parser in entry
             series_parser = accepted_series[series][0]['series_parser']
             for folder in paths:
-                folder = path(folder).expanduser()
+                folder = Path(folder).expanduser()
                 if not folder.isdir():
                     log.warning('Directory %s does not exist', folder)
                     continue
@@ -85,7 +87,8 @@ class FilterExistsSeries(object):
                 for filename in folder.walk(errors='ignore'):
                     # run parser on filename data
                     try:
-                        disk_parser = get_plugin_by_name('parsing').instance.parse_series(data=filename.name, name=series_parser.name)
+                        disk_parser = get_plugin_by_name('parsing').instance.parse_series(data=filename.name,
+                                                                                          name=series_parser.name)
                     except ParseWarning as pw:
                         disk_parser = pw.parsed
                         log_once(pw.value, logger=log)
@@ -117,6 +120,7 @@ class FilterExistsSeries(object):
                                 log.trace('new one is better proper, allowing')
                                 continue
 
+
 @event('plugin.register')
 def register_plugin():
-    plugin.register(FilterExistsSeries, 'exists_series', groups=['exists'], api_ver=2)
+    plugin.register(FilterExistsSeries, 'exists_series', interfaces=['task'], api_ver=2)
