@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -34,7 +36,7 @@ class FilterPrivateTorrents(object):
         private_torrents = config
 
         for entry in task.accepted:
-            if not 'torrent' in entry:
+            if 'torrent' not in entry:
                 log.debug('`%s` is not a torrent' % entry['title'])
                 continue
             private = entry['torrent'].private
