@@ -1,13 +1,16 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
+from past.builtins import basestring
+
 import logging
 from datetime import datetime, timedelta
 
 from sqlalchemy import Column, Integer, String, Unicode, DateTime, ForeignKey, and_, Index
 from sqlalchemy.orm import relation
 
-from flexget import db_schema, options, plugin
+from flexget import db_schema, plugin
 from flexget.event import event
-from flexget.logger import console
 from flexget.manager import Session
 from flexget.utils.sqlalchemy_utils import table_columns, table_add_column
 from flexget.utils.tools import parse_timedelta
@@ -39,7 +42,6 @@ def upgrade(ver, session):
 
 
 class RememberTask(Base):
-
     __tablename__ = 'remember_rejected_feeds'
 
     id = Column(Integer, primary_key=True)
@@ -49,7 +51,6 @@ class RememberTask(Base):
 
 
 class RememberEntry(Base):
-
     __tablename__ = 'remember_rejected_entry'
 
     id = Column(Integer, primary_key=True)
@@ -62,6 +63,7 @@ class RememberEntry(Base):
 
     task_id = Column('feed_id', Integer, ForeignKey('remember_rejected_feeds.id'), nullable=False)
 
+
 Index('remember_feed_title_url', RememberEntry.task_id, RememberEntry.title, RememberEntry.url)
 
 
@@ -78,23 +80,23 @@ class FilterRememberRejected(object):
     @plugin.priority(0)
     def on_task_start(self, task, config):
         """Purge remembered entries if the config has changed."""
-        # See if the task has changed since last run
-        old_task = task.session.query(RememberTask).filter(RememberTask.name == task.name).first()
-        if not task.is_rerun and old_task and task.config_modified:
-            log.debug('Task config has changed since last run, purging remembered entries.')
-            task.session.delete(old_task)
-            old_task = None
-        if not old_task:
-            # Create this task in the db if not present
-            task.session.add(RememberTask(name=task.name))
-        elif not task.is_rerun:
-            # Delete expired items if this is not a rerun
-            deleted = task.session.query(RememberEntry).filter(RememberEntry.task_id == old_task.id).\
-                filter(RememberEntry.expires < datetime.now()).delete()
-            if deleted:
-                log.debug('%s entries have expired from remember_rejected table.' % deleted)
-                task.config_changed()
-        task.session.commit()
+        with Session() as session:
+            # See if the task has changed since last run
+            old_task = session.query(RememberTask).filter(RememberTask.name == task.name).first()
+            if not task.is_rerun and old_task and task.config_modified:
+                log.debug('Task config has changed since last run, purging remembered entries.')
+                session.delete(old_task)
+                old_task = None
+            if not old_task:
+                # Create this task in the db if not present
+                session.add(RememberTask(name=task.name))
+            elif not task.is_rerun:
+                # Delete expired items if this is not a rerun
+                deleted = session.query(RememberEntry).filter(RememberEntry.task_id == old_task.id). \
+                    filter(RememberEntry.expires < datetime.now()).delete()
+                if deleted:
+                    log.debug('%s entries have expired from remember_rejected table.' % deleted)
+                    task.config_changed()
 
     @plugin.priority(-255)
     def on_task_input(self, task, config):
@@ -104,19 +106,20 @@ class FilterRememberRejected(object):
     @plugin.priority(255)
     def on_task_filter(self, task, config):
         """Reject any remembered entries from previous runs"""
-        (task_id,) = task.session.query(RememberTask.id).filter(RememberTask.name == task.name).first()
-        reject_entries = task.session.query(RememberEntry).filter(RememberEntry.task_id == task_id)
-        if reject_entries.count():
-            # Reject all the remembered entries
-            for entry in task.entries:
-                if not entry.get('url'):
-                    # We don't record or reject any entries without url
-                    continue
-                reject_entry = reject_entries.filter(and_(RememberEntry.title == entry['title'],
-                                                          RememberEntry.url == entry['original_url'])).first()
-                if reject_entry:
-                    entry.reject('Rejected on behalf of %s plugin: %s' %
-                        (reject_entry.rejected_by, reject_entry.reason))
+        with Session() as session:
+            (task_id,) = session.query(RememberTask.id).filter(RememberTask.name == task.name).first()
+            reject_entries = session.query(RememberEntry).filter(RememberEntry.task_id == task_id)
+            if reject_entries.count():
+                # Reject all the remembered entries
+                for entry in task.entries:
+                    if not entry.get('url'):
+                        # We don't record or reject any entries without url
+                        continue
+                    reject_entry = reject_entries.filter(and_(RememberEntry.title == entry['title'],
+                                                              RememberEntry.url == entry['original_url'])).first()
+                    if reject_entry:
+                        entry.reject('Rejected on behalf of %s plugin: %s' %
+                                     (reject_entry.rejected_by, reject_entry.reason))
 
     def on_entry_reject(self, entry, remember=None, remember_time=None, **kwargs):
         # We only remember rejections that specify the remember keyword argument
@@ -136,50 +139,18 @@ class FilterRememberRejected(object):
 
     @plugin.priority(-255)
     def on_task_learn(self, task, config):
-        for entry in task.all_entries:
-            if not entry.get('remember_rejected'):
-                continue
-            expires = None
-            if isinstance(entry['remember_rejected'], timedelta):
-                expires = datetime.now() + entry['remember_rejected']
+        with Session() as session:
+            for entry in task.all_entries:
+                if not entry.get('remember_rejected'):
+                    continue
+                expires = None
+                if isinstance(entry['remember_rejected'], timedelta):
+                    expires = datetime.now() + entry['remember_rejected']
 
-            (remember_task_id,) = task.session.query(RememberTask.id).filter(RememberTask.name == task.name).first()
-            task.session.add(RememberEntry(title=entry['title'], url=entry['original_url'], task_id=remember_task_id,
-                                           rejected_by=entry.get('rejected_by'), reason=entry.get('reason'),
-                                           expires=expires))
-
-
-def do_cli(manager, options):
-    if options.rejected_action == 'list':
-        list_rejected()
-    elif options.rejected_action == 'clear':
-        clear_rejected(manager)
-
-
-def list_rejected():
-    session = Session()
-    try:
-        results = session.query(RememberEntry).all()
-        if not results:
-            console('No rejected entries recorded by remember_rejected')
-        else:
-            console('Rejections remembered by remember_rejected:')
-        for entry in results:
-            console('%s from %s by %s because %s' % (entry.title, entry.task.name, entry.rejected_by, entry.reason))
-    finally:
-        session.close()
-
-
-def clear_rejected(manager):
-    session = Session()
-    try:
-        results = session.query(RememberEntry).delete()
-        console('Cleared %i items.' % results)
-        session.commit()
-        if results:
-            manager.config_changed()
-    finally:
-        session.close()
+                (remember_task_id,) = session.query(RememberTask.id).filter(RememberTask.name == task.name).first()
+                session.add(RememberEntry(title=entry['title'], url=entry['original_url'], task_id=remember_task_id,
+                                          rejected_by=entry.get('rejected_by'), reason=entry.get('reason'),
+                                          expires=expires))
 
 
 @event('manager.db_cleanup')
@@ -195,9 +166,12 @@ def register_plugin():
     plugin.register(FilterRememberRejected, 'remember_rejected', builtin=True, api_ver=2)
 
 
-@event('options.register')
-def register_parser_arguments():
-    parser = options.register_command('rejected', do_cli, help='list or clear remembered rejections')
-    subparsers = parser.add_subparsers(dest='rejected_action', metavar='<action>')
-    subparsers.add_parser('list', help='list all the entries that have been rejected')
-    subparsers.add_parser('clear', help='clear all rejected entries from database, so they can be retried')
+def get_rejected(session, count=None, start=None, stop=None, sort_by=None, descending=None):
+    query = session.query(RememberEntry)
+    if count:
+        return query.count()
+    if descending:
+        query = query.order_by(getattr(RememberEntry, sort_by).desc())
+    else:
+        query = query.order_by(getattr(RememberEntry, sort_by))
+    return query.slice(start, stop).all()
