@@ -1,4 +1,7 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import logging
 
 from flexget import plugin
@@ -25,9 +28,12 @@ class FilterRequireField(object):
             config = [config]
         for entry in task.entries:
             for field in config:
-                if not entry.get(field):
+                if field not in entry:
                     entry.reject('Required field %s is not present' % field)
                     break
+                if entry[field] is None:
+                    entry.reject('Required field %s is `None`' % field)
+                    break
 
 
 @event('plugin.register')
