@@ -1,18 +1,20 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 from datetime import datetime, timedelta
 
 from sqlalchemy import Column, Integer, String, Unicode, DateTime
 from sqlalchemy.schema import Index
 
-from flexget import db_schema, options, plugin
+from flexget import db_schema, plugin
 from flexget.event import event
-from flexget.logger import console
 from flexget.manager import Session
+from flexget.utils.sqlalchemy_utils import table_add_column
 from flexget.utils.tools import parse_timedelta
-from flexget.utils.sqlalchemy_utils import table_add_column, table_schema
 
 SCHEMA_VER = 3
+FAIL_LIMIT = 100
 
 log = logging.getLogger('failed')
 Base = db_schema.versioned_base('failed', SCHEMA_VER)
@@ -20,16 +22,8 @@ Base = db_schema.versioned_base('failed', SCHEMA_VER)
 
 @db_schema.upgrade('failed')
 def upgrade(ver, session):
-    if ver is None:
-        # add count column
-        table_add_column('failed', 'count', Integer, session, default=1)
-        ver = 0
-    if ver == 0:
-        # define an index
-        log.info('Adding database index ...')
-        failed = table_schema('failed', session)
-        Index('failed_title_url', failed.c.title, failed.c.url, failed.c.count).create()
-        ver = 1
+    if ver is None or ver < 1:
+        raise db_schema.UpgradeImpossible
     if ver == 1:
         table_add_column('failed', 'reason', Unicode, session)
         ver = 2
@@ -59,11 +53,37 @@ class FailedEntry(Base):
     def __str__(self):
         return '<Failed(title=%s)>' % self.title
 
+    def to_dict(self):
+        return {
+            'id': self.id,
+            'title': self.title,
+            'url': self.url,
+            'added_at': self.tof,
+            'reason': self.reason,
+            'count': self.count,
+            'retry_time': self.retry_time
+        }
+
+
 # create indexes, used when creating tables
 columns = Base.metadata.tables['failed'].c
 Index('failed_title_url', columns.title, columns.url, columns.count)
 
 
+@event('manager.db_cleanup')
+def db_cleanup(manager, session):
+    # Delete everything older than 30 days
+    session.query(FailedEntry).filter(FailedEntry.tof < datetime.now() - timedelta(days=30)).delete()
+    # Of the remaining, always keep latest 25. Drop any after that if fail was more than a week ago.
+    keep_num = 25
+    keep_ids = [fe.id for fe in session.query(FailedEntry).order_by(FailedEntry.tof.desc())[:keep_num]]
+    if len(keep_ids) == keep_num:
+        query = session.query(FailedEntry)
+        query = query.filter(FailedEntry.id.notin_(keep_ids))
+        query = query.filter(FailedEntry.tof < datetime.now() - timedelta(days=7))
+        query.delete(synchronize_session=False)
+
+
 class PluginFailed(object):
     """
     Records entry failures and stores them for trying again after a certain interval.
@@ -79,7 +99,10 @@ class PluginFailed(object):
                 "type": "object",
                 "properties": {
                     "retry_time": {"type": "string", "format": "interval", "default": "1 hour"},
-                    "max_retries": {"type": "integer", "minimum": 0, "default": 3},
+                    "max_retries": {"type": "integer",
+                                    "minimum": 0,
+                                    "maximum": FAIL_LIMIT,
+                                    "default": 3},
                     "retry_time_multiplier": {
                         # Allow turning off the retry multiplier with 'no' as well as 1
                         "oneOf": [{"type": "number", "minimum": 0}, {"type": "boolean"}],
@@ -116,6 +139,10 @@ class PluginFailed(object):
         # Timedeltas do not allow floating point multiplication. Convert to seconds and then back to avoid this.
         base_retry_secs = base_retry_time.days * 86400 + base_retry_time.seconds
         retry_secs = base_retry_secs * (config['retry_time_multiplier'] ** fail_count)
+        # prevent OverflowError: date value out of range, cap to 30 days
+        max = 60 * 60 * 24 * 30
+        if retry_secs > max:
+            retry_secs = max
         return timedelta(seconds=retry_secs)
 
     @plugin.priority(-255)
@@ -128,27 +155,28 @@ class PluginFailed(object):
 
     def add_failed(self, entry, reason=None, config=None, **kwargs):
         """Adds entry to internal failed list, displayed with --failed"""
-        reason = reason or 'Unknown'
+        # Make sure reason is a string, in case it is set to an exception instance
+        reason = str(reason) or 'Unknown'
         with Session() as session:
             # query item's existence
-            item = session.query(FailedEntry).filter(FailedEntry.title == entry['title']).\
+            item = session.query(FailedEntry).filter(FailedEntry.title == entry['title']). \
                 filter(FailedEntry.url == entry['original_url']).first()
             if not item:
                 item = FailedEntry(entry['title'], entry['original_url'], reason)
                 item.count = 0
+            if item.count > FAIL_LIMIT:
+                log.error('entry with title \'%s\' has failed over %s times', entry['title'], FAIL_LIMIT)
+                return
             retry_time = self.retry_time(item.count, config)
             item.retry_time = datetime.now() + retry_time
             item.count += 1
             item.tof = datetime.now()
             item.reason = reason
             session.merge(item)
-            log.debug('Marking %s in failed list. Has failed %s times.' % (item.title, item.count))
+            log.debug('Marking %s in failed list. Has failed %s times.', item.title, item.count, )
             if self.backlog and item.count <= config['max_retries']:
                 self.backlog.instance.add_backlog(entry.task, entry, amount=retry_time, session=session)
-            entry.task.rerun()
-            # limit item number to 25
-            for row in session.query(FailedEntry).order_by(FailedEntry.tof.desc())[25:]:
-                session.delete(row)
+            entry.task.rerun(plugin='retry_failed')
 
     @plugin.priority(255)
     def on_task_filter(self, task, config):
@@ -157,7 +185,7 @@ class PluginFailed(object):
         config = self.prepare_config(config)
         max_count = config['max_retries']
         for entry in task.entries:
-            item = task.session.query(FailedEntry).filter(FailedEntry.title == entry['title']).\
+            item = task.session.query(FailedEntry).filter(FailedEntry.title == entry['title']). \
                 filter(FailedEntry.url == entry['original_url']).first()
             if item:
                 if item.count > max_count:
@@ -168,45 +196,17 @@ class PluginFailed(object):
                                  item.reason)
 
 
-def do_cli(manager, options):
-    if options.failed_action == 'list':
-        list_failed()
-    elif options.failed_action == 'clear':
-        clear_failed(manager)
-
-
-def list_failed():
-    session = Session()
-    try:
-        results = session.query(FailedEntry).all()
-        if not results:
-            console('No failed entries recorded')
-        for entry in results:
-            console('%16s - %s - %s times - %s' %
-                    (entry.tof.strftime('%Y-%m-%d %H:%M'), entry.title, entry.count, entry.reason))
-    finally:
-        session.close()
-
-
-def clear_failed(manager):
-    session = Session()
-    try:
-        results = session.query(FailedEntry).delete()
-        console('Cleared %i items.' % results)
-        session.commit()
-        if results:
-            manager.config_changed()
-    finally:
-        session.close()
-
-
 @event('plugin.register')
 def register_plugin():
     plugin.register(PluginFailed, 'retry_failed', builtin=True, api_ver=2)
 
-@event('options.register')
-def register_parser_arguments():
-    parser = options.register_command('failed', do_cli, help='list or clear remembered failures')
-    subparsers = parser.add_subparsers(dest='failed_action', metavar='<action>')
-    subparsers.add_parser('list', help='list all the entries that have had failures')
-    subparsers.add_parser('clear', help='clear all failures from database, so they can be retried')
+
+def get_failures(session, count=None, start=None, stop=None, sort_by=None, descending=None):
+    query = session.query(FailedEntry)
+    if count:
+        return query.count()
+    if descending:
+        query = query.order_by(getattr(FailedEntry, sort_by).desc())
+    else:
+        query = query.order_by(getattr(FailedEntry, sort_by))
+    return query.slice(start, stop).all()
