@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
 
 from flexget import plugin
 from flexget.event import event
@@ -7,18 +9,20 @@ from flexget.plugins.filter.seen import FilterSeen
 
 class FilterSeenInfoHash(FilterSeen):
     """Prevents the same torrent from being downloaded twice by remembering the infohash of all downloaded torrents."""
+    schema = {
+        'oneOf': [
+            {'type': 'boolean'},
+            {'type': 'string', 'enum': ['global', 'local']}
+        ]
+    }
 
     def __init__(self):
         # remember and filter by these fields
         self.fields = ['torrent_info_hash']
         self.keyword = 'seen_info_hash'
 
-    def validator(self):
-        from flexget import validator
-        return validator.factory('boolean')
-
     @plugin.priority(180)
-    def on_task_filter(self, task, config):
+    def on_task_filter(self, task, config):  # pylint: disable=W0221
         # Return if we are disabled.
         if config is False:
             return
