@@ -1,4 +1,5 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 from flexget import plugin
 from flexget.event import event
@@ -41,8 +42,6 @@ class FilterSeriesPremiere(FilterSeriesBase):
         if not config:
             # Don't run when we are disabled
             return
-        if task.is_rerun:
-            return
         # Generate the group settings for series plugin
         group_settings = {}
         allow_seasonless = False
@@ -60,7 +59,7 @@ class FilterSeriesPremiere(FilterSeriesBase):
         guessed_series = {}
         for entry in task.entries:
             if guess_entry(entry, allow_seasonless=allow_seasonless, config=group_settings):
-                if entry['series_season'] == 1 and entry['series_episode'] in desired_eps:
+                if not entry['season_pack'] and entry['series_season'] == 1 and entry['series_episode'] in desired_eps:
                     normalized_name = normalize_series_name(entry['series_name'])
                     db_series = task.session.query(Series).filter(Series.name == normalized_name).first()
                     if db_series and db_series.in_tasks:
@@ -68,17 +67,17 @@ class FilterSeriesPremiere(FilterSeriesBase):
                     guessed_series.setdefault(normalized_name, entry['series_name'])
         # Reject any further episodes in those series
         for entry in task.entries:
-            for series in guessed_series.itervalues():
-                if entry.get('series_name') == series and not (
-                        entry.get('series_season') == 1
-                        and entry.get('series_episode') in desired_eps):
-                    entry.reject('Non premiere episode in a premiere series')
-        # Since we are running after task start phase, make sure not to merge into the config multiple times on reruns
-        if not task.is_rerun:
-            # Combine settings and series into series plugin config format
-            allseries = {'settings': {'series_premiere': group_settings}, 'series_premiere': guessed_series.values()}
-            # Merge the our config in to the main series config
-            self.merge_config(task, allseries)
+            for series in guessed_series.values():
+                if entry.get('series_name') == series and \
+                        (entry.get('season_pack') or not (
+                        entry.get('series_season') == 1 and entry.get('series_episode') in desired_eps)):
+                    entry.reject('Non premiere episode or season pack in a premiere series')
+
+        # Combine settings and series into series plugin config format
+        allseries = {'settings': {'series_premiere': group_settings},
+                     'series_premiere': list(guessed_series.values())}
+        # Merge the our config in to the main series config
+        self.merge_config(task, allseries)
 
 
 @event('plugin.register')
