@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -96,20 +98,15 @@ class FilterTvdb(object):
             'reject_network': {'type': 'array', 'items': {'type': 'string'}},
             'reject_genres': {'type': 'array', 'items': {'type': 'string'}},
             'reject_status': {'type': 'array', 'items': {'type': 'string'}},
-            'reject_languages': {'type': 'array', 'items': {'type': 'string'}},
-            'accept_languages': {'type': 'array', 'items': {'type': 'string'}},
             'accept_actors': {'type': 'array', 'items': {'type': 'string'}},
             'reject_actors': {'type': 'array', 'items': {'type': 'string'}},
-            'accept_writers': {'type': 'array', 'items': {'type': 'string'}},
-            'reject_writers': {'type': 'array', 'items': {'type': 'string'}},
             'accept_directors': {'type': 'array', 'items': {'type': 'string'}},
             'reject_directors': {'type': 'array', 'items': {'type': 'string'}},
-
         },
         'additionalProperties': False
     }
 
-    def is_in_set(self, config, configkey, entryitem,):
+    def is_in_set(self, config, configkey, entryitem, ):
         '''
         this takes the config object, config key (to a list), and entry
         item so it can return True if the object matches,
@@ -153,10 +150,12 @@ class FilterTvdb(object):
                     reasons.append('tvdb_ep_rating (%s < %s)' % (entry['tvdb_ep_rating'], config['min_episode_rating']))
             if 'min_episode_air_year' in config:
                 if entry['tvdb_ep_air_date'].strftime("%Y") < config['min_episode_air_year']:
-                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"), config['min_episode_air_year']))
+                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"),
+                                                                   config['min_episode_air_year']))
             if 'max_episode_air_year' in config:
                 if entry['tvdb_ep_air_date'].strftime("%Y") > config['max_episode_air_year']:
-                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"), config['max_episode_air_year']))
+                    reasons.append('tvdb_ep_air_date (%s < %s)' % (entry['tvdb_ep_air_date'].strftime("%Y"),
+                                                                   config['max_episode_air_year']))
 
             if self.is_in_set(config, 'reject_content_rating', entry['tvdb_content_rating']):
                 reasons.append('reject_content_rating')
@@ -176,12 +175,6 @@ class FilterTvdb(object):
             if self.is_in_set(config, 'reject_status', entry['tvdb_status']):
                 reasons.append('reject_status')
 
-            if self.is_in_set(config, 'reject_languages', entry['tvdb_language']):
-                reasons.append('reject_languages')
-
-            if not self.is_in_set(config, 'accept_languages', entry['tvdb_language']):
-                reasons.append('accept_languages')
-
             # Accept if actors contains an accepted actor, but don't reject otherwise
             if self.is_in_set(config, 'accept_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']):
                 force_accept = True
@@ -189,13 +182,6 @@ class FilterTvdb(object):
             if self.is_in_set(config, 'reject_actors', entry['tvdb_actors'] + entry['tvdb_ep_guest_stars']):
                 reasons.append('reject_genres')
 
-            # Accept if writer is an accepted writer, but don't reject otherwise
-            if self.is_in_set(config, 'accept_writers', entry['tvdb_ep_writer']):
-                force_accept = True
-
-            if self.is_in_set(config, 'reject_writers', entry['tvdb_ep_writer']):
-                reasons.append('reject_writers')
-
             # Accept if director is an accepted director, but don't reject otherwise
             if self.is_in_set(config, 'accept_directors', entry['tvdb_ep_director']):
                 force_accept = True
@@ -205,7 +191,7 @@ class FilterTvdb(object):
 
             if reasons and not force_accept:
                 msg = 'Skipping %s because of rule(s) %s' % \
-                    (entry.get('series_name_thetvdb', None) or entry['title'], ', '.join(reasons))
+                      (entry.get('series_name_thetvdb', None) or entry['title'], ', '.join(reasons))
                 if task.options.debug:
                     log.debug(msg)
                 else:
