@@ -1,14 +1,15 @@
 from __future__ import unicode_literals, division, absolute_import
-from flexget.utils.log import log_once
-
-__author__ = 'paranoidi'
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 import logging
 import sys
 
+from flexget.utils.log import log_once
 from flexget.event import event
 from flexget.utils.simple_persistence import SimplePersistence
 
+__author__ = 'paranoidi'
+
 log = logging.getLogger('cron_env')
 
 
@@ -19,7 +20,7 @@ def check_env(manager, options):
     if options.cron:
         if 'terminal_encoding' in persistence:
             terminal_encoding = persistence['terminal_encoding']
-            if terminal_encoding != encoding:
+            if terminal_encoding.lower() != encoding.lower():
                 log.warning('Your cron environment has different filesystem encoding '
                             '(%s) compared to your terminal environment (%s).' %
                             (encoding, terminal_encoding))
