@@ -1,8 +1,9 @@
 from __future__ import unicode_literals, division, absolute_import
-from argparse import SUPPRESS
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import os
-
+from argparse import SUPPRESS
 
 from flexget import options
 from flexget.event import event
@@ -11,15 +12,15 @@ log = logging.getLogger('log_start')
 
 
 @event('manager.startup')
-def log_start(manager):
+def log_on_start(manager):
     if manager.options.log_start:
-            log.info('FlexGet started (PID: %s)' % os.getpid())
+        log.info('FlexGet started (PID: %s)' % os.getpid())
 
 
 @event('manager.shutdown')
-def log_start(manager):
+def log_on_shutdown(manager):
     if manager.options.log_start:
-            log.info('FlexGet stopped (PID: %s)' % os.getpid())
+        log.info('FlexGet stopped (PID: %s)' % os.getpid())
 
 
 @event('options.register')
