@@ -1,20 +1,22 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import pickle
 from datetime import datetime
 
-from sqlalchemy import Column, Integer, String, DateTime, PickleType, Index
+from sqlalchemy import Column, Integer, String, Unicode, DateTime, Index, select
 
 from flexget import db_schema, plugin
-from flexget.entry import Entry
 from flexget.event import event
+from flexget.utils import json
 from flexget.manager import Session
-from flexget.utils.database import safe_pickle_synonym, with_session
-from flexget.utils.sqlalchemy_utils import table_schema
+from flexget.utils.database import entry_synonym, with_session
 from flexget.utils.tools import parse_timedelta
+from flexget.utils.sqlalchemy_utils import table_schema, table_add_column
 
 log = logging.getLogger('backlog')
-Base = db_schema.versioned_base('backlog', 1)
+Base = db_schema.versioned_base('backlog', 2)
 
 
 @db_schema.upgrade('backlog')
@@ -35,26 +37,65 @@ def upgrade(ver, session):
         log.info('Creating index on backlog table.')
         Index('ix_backlog_feed_expire', backlog_table.c.feed, backlog_table.c.expire).create(bind=session.bind)
         ver = 1
+    if ver == 1:
+        table = table_schema('backlog', session)
+        table_add_column(table, 'json', Unicode, session)
+        # Make sure we get the new schema with the added column
+        table = table_schema('backlog', session)
+        for row in session.execute(select([table.c.id, table.c.entry])):
+            try:
+                p = pickle.loads(row['entry'])
+                session.execute(table.update().where(table.c.id == row['id']).values(
+                    json=json.dumps(p, encode_datetime=True)))
+            except KeyError as e:
+                log.error('Unable error upgrading backlog pickle object due to %s' % str(e))
+
+        ver = 2
     return ver
 
 
 class BacklogEntry(Base):
-
     __tablename__ = 'backlog'
 
     id = Column(Integer, primary_key=True)
     task = Column('feed', String)
     title = Column(String)
     expire = Column(DateTime)
-    _entry = Column('entry', PickleType)
-    entry = safe_pickle_synonym('_entry')
+    _json = Column('json', Unicode)
+    entry = entry_synonym('_json')
 
     def __repr__(self):
         return '<BacklogEntry(title=%s)>' % (self.title)
 
+
 Index('ix_backlog_feed_expire', BacklogEntry.task, BacklogEntry.expire)
 
 
+@with_session
+def get_entries(task=None, session=None):
+    query = session.query(BacklogEntry)
+    if task:
+        query = query.filter(BacklogEntry.task == task)
+    return query.all()
+
+
+@with_session
+def clear_entries(task=None, all=False, session=None):
+    """
+    Clear entries from backlog.
+
+    :param task: If given, only entries from specified task will be cleared.
+    :param all: If True, all entries will be cleared, otherwise only expired entries will be cleared.
+    :returns: Number of entries cleared from backlog.
+    """
+    query = session.query(BacklogEntry)
+    if task:
+        query = query.filter(BacklogEntry.task == task)
+    if not all:
+        query = query.filter(BacklogEntry.expire < datetime.now())
+    return query.delete()
+
+
 class InputBacklog(object):
     """
     Keeps task history for given amount of time.
@@ -99,7 +140,7 @@ class InputBacklog(object):
                 log.warning('No input snapshot available for `%s`, using current state' % entry['title'])
             snapshot = entry
         expire_time = datetime.now() + parse_timedelta(amount)
-        backlog_entry = session.query(BacklogEntry).filter(BacklogEntry.title == entry['title']).\
+        backlog_entry = session.query(BacklogEntry).filter(BacklogEntry.title == entry['title']). \
             filter(BacklogEntry.task == task.name).first()
         if backlog_entry:
             # If there is already a backlog entry for this, update the expiry time if necessary.
@@ -125,9 +166,8 @@ class InputBacklog(object):
     def get_injections(self, task, session=None):
         """Insert missing entries from backlog."""
         entries = []
-        task_backlog = session.query(BacklogEntry).filter(BacklogEntry.task == task.name)
-        for backlog_entry in task_backlog.all():
-            entry = Entry(backlog_entry.entry)
+        for backlog_entry in get_entries(task=task.name, session=session):
+            entry = backlog_entry.entry
 
             # this is already in the task
             if task.find_entry(title=entry['title'], url=entry['url']):
@@ -138,9 +178,8 @@ class InputBacklog(object):
             log.verbose('Added %s entries from backlog' % len(entries))
 
         # purge expired
-        for backlog_entry in task_backlog.filter(datetime.now() > BacklogEntry.expire).all():
-            log.debug('Purging %s' % backlog_entry.title)
-            session.delete(backlog_entry)
+        purged = clear_entries(task=task.name, all=False, session=session)
+        log.debug('%s entries purged from backlog' % purged)
 
         return entries
 
