@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import random
 import string
@@ -45,9 +47,9 @@ class GenSeries(object):
                     for quality in config['qualities']:
                         entry = Entry()
                         entry['title'] = 'series %d name - S%02dE%02d - %s' % \
-                            (num, season + 1, episode + 1, quality)
+                                         (num, season + 1, episode + 1, quality)
                         entry['url'] = 'http://localhost/mock/%s' % \
-                                       ''.join([random.choice(string.letters + string.digits) for x in range(1, 30)])
+                                       ''.join([random.choice(string.ascii_letters + string.digits) for x in range(1, 30)])
                         self.entries.append(entry)
         log.info('Generated %d entries' % len(self.entries))
 
