@@ -1,10 +1,11 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
 from flexget.event import event
 from flexget.entry import Entry
-from flexget import validator
 
 log = logging.getLogger(__name__.rsplit('.')[-1])
 
@@ -21,9 +22,11 @@ class Generate(object):
             entry = Entry()
             import string
             import random
-            entry['url'] = 'http://localhost/generate/%s/%s' % (i, ''.join([random.choice(string.letters + string.digits) for x in range(1, 30)]))
-            entry['title'] = ''.join([random.choice(string.letters + string.digits) for x in range(1, 30)])
-            entry['description'] = ''.join([random.choice(string.letters + string.digits) for x in range(1, 1000)])
+            entry['url'] = 'http://localhost/generate/%s/%s' % (
+                i,
+                ''.join([random.choice(string.ascii_letters + string.digits) for x in range(1, 30)]))
+            entry['title'] = ''.join([random.choice(string.ascii_letters + string.digits) for x in range(1, 30)])
+            entry['description'] = ''.join([random.choice(string.ascii_letters + string.digits) for x in range(1, 1000)])
             entries.append(entry)
         return entries
 
