@@ -1,9 +1,15 @@
 from __future__ import unicode_literals, division, absolute_import
-import urlparse
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+from future.moves.urllib import parse
+
+
 import logging
-import urllib
+import posixpath
 import zlib
 import re
+import io
+
 from jinja2 import Template
 
 from flexget import plugin
@@ -54,7 +60,7 @@ class InputHtml(object):
 
         def get_auth_from_url():
             """Moves basic authentication from url to username and password fields"""
-            parts = list(urlparse.urlsplit(config['url']))
+            parts = list(parse.urlsplit(config['url']))
             split = parts[1].split('@')
             if len(split) > 1:
                 auth = split[0].split(':')
@@ -63,7 +69,7 @@ class InputHtml(object):
                 else:
                     log.warning('Invalid basic authentication in url: %s' % config['url'])
                 parts[1] = split[1]
-                config['url'] = urlparse.urlunsplit(parts)
+                config['url'] = parse.urlunsplit(parts)
 
         if isinstance(config, basestring):
             config = {'url': config}
@@ -119,7 +125,7 @@ class InputHtml(object):
                 current += step
             return entries
         else:
-            return self._request_url(task, config, base_url, auth)
+            return self._request_url(task, config, base_url, auth, dump_name=config.get('dump'))
 
     def _request_url(self, task, config, url, auth, dump_name=None):
         log.verbose('Requesting: %s' % url)
@@ -131,7 +137,7 @@ class InputHtml(object):
         if dump_name:
             log.verbose('Dumping: %s' % dump_name)
             data = soup.prettify()
-            with open(dump_name, 'w') as f:
+            with io.open(dump_name, 'w', encoding='utf-8') as f:
                 f.write(data)
 
         return self.create_entries(url, soup, config)
@@ -147,9 +153,15 @@ class InputHtml(object):
         return title or None
 
     def _title_from_url(self, url):
-        parts = urllib.splitquery(url[url.rfind('/') + 1:])
-        title = urllib.unquote_plus(parts[0])
-        return title
+        parts = parse.urlsplit(url)
+        name = ''
+        if parts.scheme == 'magnet':
+            match = re.search('(?:&dn(?:\.\d)?=)(.+?)(?:&)', parts.query)
+            if match:
+                name = match.group(1)
+        else:
+            name = posixpath.basename(parts.path)
+        return parse.unquote_plus(name)
 
     def create_entries(self, page_url, soup, config):
 
@@ -172,15 +184,15 @@ class InputHtml(object):
                 continue
 
             url = link['href']
-            log_link = url
-            log_link = log_link.replace('\n', '')
-            log_link = log_link.replace('\r', '')
-
             # fix broken urls
             if url.startswith('//'):
                 url = 'http:' + url
             elif not url.startswith('http://') or not url.startswith('https://'):
-                url = urlparse.urljoin(page_url, url)
+                url = parse.urljoin(page_url, url)
+
+            log_link = url
+            log_link = log_link.replace('\n', '')
+            log_link = log_link.replace('\r', '')
 
             # get only links matching regexp
             regexps = config.get('links_re', None)
@@ -190,6 +202,7 @@ class InputHtml(object):
                     if re.search(regexp, url):
                         accept = True
                 if not accept:
+                    log.debug('url does not match any "links_re": %s' % url)
                     continue
 
             title_from = config.get('title_from', 'auto')
@@ -262,6 +275,9 @@ class InputHtml(object):
             entry['url'] = url
             entry['title'] = title
 
+            if 'username' in config and 'password' in config:
+                entry['download_auth'] = (config['username'], config['password'])
+
             queue.append(entry)
 
         # add from queue to task
