@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -20,7 +22,17 @@ class PluginInputs(object):
 
     schema = {
         'type': 'array',
-        'items': {'allOf': [{'$ref': '/schema/plugins?phase=input'}, {'maxProperties': 1, 'minProperties': 1}]}
+        'items': {
+            'allOf': [
+                {'$ref': '/schema/plugins?phase=input'},
+                {
+                    'maxProperties': 1,
+                    'error_maxProperties': 'Plugin options within inputs plugin must be indented 2 more spaces than '
+                                           'the first letter of the plugin name.',
+                    'minProperties': 1
+                }
+            ]
+        }
     }
 
     def on_task_input(self, task, config):
@@ -28,7 +40,7 @@ class PluginInputs(object):
         entry_titles = set()
         entry_urls = set()
         for item in config:
-            for input_name, input_config in item.iteritems():
+            for input_name, input_config in item.items():
                 input = plugin.get_plugin_by_name(input_name)
                 if input.api_ver == 1:
                     raise plugin.PluginError('Plugin %s does not support API v2' % input_name)
