@@ -1,5 +1,7 @@
 """Plugin for mocking task data."""
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -39,8 +41,8 @@ class Mock(object):
         for line in config:
             entry = Entry(line)
             # no url specified, add random one based on title (ie. test)
-            if not 'url' in entry:
-                entry['url'] = 'http://localhost/mock/%s' % hash(entry['title'])
+            if 'url' not in entry:
+                entry['url'] = 'mock://localhost/mock/%s' % hash(entry['title'])
             entries.append(entry)
         return entries
 
