@@ -1,18 +1,18 @@
 from __future__ import unicode_literals, division, absolute_import
-import logging
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
-from bs4 import BeautifulSoup
+import logging
 
 from flexget import plugin
 from flexget.entry import Entry
 from flexget.event import event
 from flexget.utils import requests
+from flexget.utils.soup import get_soup
 
 log = logging.getLogger('pogcal')
 
 
 class InputPogDesign(object):
-
     schema = {
         'type': 'object',
         'properties': {
@@ -30,29 +30,39 @@ class InputPogDesign(object):
         session = requests.Session()
         data = {'username': config['username'], 'password': config['password'], 'sub_login': 'Account Login'}
         try:
-            r = session.post('http://www.pogdesign.co.uk/cat/', data=data)
-            if 'U / P Invalid' in r.text:
+            r = session.post('https://www.pogdesign.co.uk/cat/login', data=data)
+            if 'Login to Your Account' in r.text:
                 raise plugin.PluginError('Invalid username/password for pogdesign.')
-            page = session.get('http://www.pogdesign.co.uk/cat/showselect.php')
+            page = session.get('https://www.pogdesign.co.uk/cat/show-select')
         except requests.RequestException as e:
             raise plugin.PluginError('Error retrieving source: %s' % e)
-        soup = BeautifulSoup(page.text)
+        soup = get_soup(page.text)
         entries = []
-        for row in soup.find_all('label', {'class': 'label_check'}):
-            if row.find(attrs={'checked': 'checked'}):
-                t = row.find('strong').text
-                if t.endswith('[The]'):
-                    t = 'The ' + t[:-6]
+        for row in soup.find_all('li', {'class': 'selectgrp checked'}):
 
-                # Make certain names friendlier
-                if t in self.name_map:
-                    t = self.name_map[t]
+            # Get name
+            t = row.find('strong')
+
+            # Remove <span> tags
+            spantags = t.find_all('span')
+            for s in spantags:
+                s.extract()
+            t = row.text
+            # remove newlines and whitepsace
+            t = t.replace('\n', '').strip()
+
+            if t.endswith('[The]'):
+                t = 'The ' + t[:-6]
+
+            # Make certain names friendlier
+            if t in self.name_map:
+                t = self.name_map[t]
+
+            e = Entry()
+            e['title'] = t
+            e['url'] = 'https://www.pogdesign.co.uk/{0}'.format(row.find_next('a')['href'].lstrip('/'))
+            entries.append(e)
 
-                e = Entry()
-                e['title'] = t
-                url = row.find_next('a', {'class': 'slink'})
-                e['url'] = 'http://www.pogdesign.co.uk' + url['href']
-                entries.append(e)
         return entries
 
 
