@@ -1,10 +1,13 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.utils import tobytes
+from future.moves.urllib.parse import urlparse, urlsplit
+
 import os
 import logging
-import urlparse
 import xml.sax
 import posixpath
-import httplib
+import http.client
 from datetime import datetime
 
 import dateutil.parser
@@ -23,6 +26,7 @@ from flexget.utils.pathscrub import pathscrub
 log = logging.getLogger('rss')
 feedparser.registerDateHandler(lambda date_string: dateutil.parser.parse(date_string).timetuple())
 
+
 def fp_field_name(name):
     """Translates literal field name to the sanitized one feedparser will use."""
     return name.replace(':', '_').lower()
@@ -120,24 +124,27 @@ class InputRSS(object):
 
     def build_config(self, config):
         """Set default values to config"""
-        if isinstance(config, basestring):
+        if isinstance(config, str):
             config = {'url': config}
+        else:
+            # Make a copy so that original config is not modified
+            config = dict(config)
         # set the default link value to 'auto'
         config.setdefault('link', 'auto')
         # Convert any field names from the config to format feedparser will use for 'link', 'title' and 'other_fields'
         if config['link'] != 'auto':
             if not isinstance(config['link'], list):
                 config['link'] = [config['link']]
-            config['link'] = map(fp_field_name, config['link'])
+            config['link'] = list(map(fp_field_name, config['link']))
         config.setdefault('title', 'title')
         config['title'] = fp_field_name(config['title'])
         if config.get('other_fields'):
             other_fields = []
             for item in config['other_fields']:
-                if isinstance(item, basestring):
+                if isinstance(item, str):
                     key, val = item, item
                 else:
-                    key, val = item.items()[0]
+                    key, val = list(item.items())[0]
                 other_fields.append({fp_field_name(key): val.lower()})
             config['other_fields'] = other_fields
         # set default value for group_links as deactivated
@@ -152,24 +159,27 @@ class InputRSS(object):
         if data is None:
             log.critical('Received empty page - no content')
             return
+        else:
+            data = tobytes(data)
+
         ext = 'xml'
-        if '<html>' in data.lower():
+        if b'<html>' in data.lower():
             log.critical('Received content is HTML page, not an RSS feed')
             ext = 'html'
-        if 'login' in data.lower() or 'username' in data.lower():
+        if b'login' in data.lower() or b'username' in data.lower():
             log.critical('Received content looks a bit like login page')
-        if 'error' in data.lower():
+        if b'error' in data.lower():
             log.critical('Received content looks a bit like error page')
         received = os.path.join(task.manager.config_base, 'received')
         if not os.path.isdir(received):
             os.mkdir(received)
         filename = task.name
-        sourcename = urlparse.urlparse(url).netloc
+        sourcename = urlparse(url).netloc
         if sourcename:
             filename += '-' + sourcename
         filename = pathscrub(filename, filename=True)
         filepath = os.path.join(received, '%s.%s' % (filename, ext))
-        with open(filepath, 'w') as f:
+        with open(filepath, 'wb') as f:
             f.write(data)
         log.critical('I have saved the invalid content to %s for you to view', filepath)
 
@@ -180,13 +190,13 @@ class InputRSS(object):
         if 'length' in enclosure:
             try:
                 entry['size'] = int(enclosure['length'])
-            except:
+            except ValueError:
                 entry['size'] = 0
         if 'type' in enclosure:
             entry['type'] = enclosure['type']
         # TODO: better and perhaps join/in download plugin?
         # Parse filename from enclosure url
-        basename = posixpath.basename(urlparse.urlsplit(entry['url']).path)
+        basename = posixpath.basename(urlsplit(entry['url']).path)
         # If enclosure has size OR there are multiple enclosures use filename from url
         if (entry.get('size') or multiple and basename) and filename:
             entry['filename'] = basename
@@ -206,7 +216,7 @@ class InputRSS(object):
         # last run and if caching wasn't disabled with --no-cache argument.
         all_entries = (config['all_entries'] or task.config_modified or
                        task.options.nocache or task.options.retry)
-        headers = {}
+        headers = task.requests.headers
         if not all_entries:
             etag = task.simple_persistence.get('%s_etag' % url_hash, None)
             if etag:
@@ -214,7 +224,7 @@ class InputRSS(object):
                 headers['If-None-Match'] = etag
             modified = task.simple_persistence.get('%s_modified' % url_hash, None)
             if modified:
-                if not isinstance(modified, basestring):
+                if not isinstance(modified, str):
                     log.debug('Invalid date was stored for last modified time.')
                 else:
                     headers['If-Modified-Since'] = modified
@@ -232,7 +242,7 @@ class InputRSS(object):
                 content = response.content
             except RequestException as e:
                 raise plugin.PluginError('Unable to download the RSS for task %s (%s): %s' %
-                                  (task.name, config['url'], e))
+                                         (task.name, config['url'], e))
             if config.get('ascii'):
                 # convert content to ascii (cleanup), can also help with parsing problems on malformed feeds
                 content = response.text.encode('ascii', 'ignore')
@@ -274,7 +284,7 @@ class InputRSS(object):
 
         if not content:
             log.error('No data recieved for rss feed.')
-            return
+            return []
         try:
             rss = feedparser.parse(content)
         except LookupError as e:
@@ -304,14 +314,15 @@ class InputRSS(object):
                     self.process_invalid_content(task, content, config['url'])
                     if task.options.debug:
                         log.error('bozo error parsing rss: %s' % ex)
-                    raise plugin.PluginError('Received invalid RSS content from task %s (%s)' % (task.name, config['url']))
-                elif isinstance(ex, httplib.BadStatusLine) or isinstance(ex, IOError):
+                    raise plugin.PluginError('Received invalid RSS content from task %s (%s)' % (task.name,
+                                                                                                 config['url']))
+                elif isinstance(ex, http.client.BadStatusLine) or isinstance(ex, IOError):
                     raise ex  # let the @internet decorator handle
                 else:
                     # all other bozo errors
                     self.process_invalid_content(task, content, config['url'])
                     raise plugin.PluginError('Unhandled bozo_exception. Type: %s (task: %s)' %
-                                      (ex.__class__.__name__, task.name), log)
+                                             (ex.__class__.__name__, task.name), log)
 
         log.debug('encoding %s', rss.encoding)
 
@@ -327,6 +338,15 @@ class InputRSS(object):
         # new entries to be created
         entries = []
 
+        # Dict with fields to grab mapping from rss field name to FlexGet field name
+        fields = {'guid': 'guid',
+                  'author': 'author',
+                  'description': 'description',
+                  'infohash': 'torrent_info_hash'}
+        # extend the dict of fields to grab with other_fields list in config
+        for field_map in config.get('other_fields', []):
+            fields.update(field_map)
+
         # field name for url can be configured by setting link.
         # default value is auto but for example guid is used in some feeds
         ignored = 0
@@ -353,28 +373,30 @@ class InputRSS(object):
             # remove annoying zero width spaces
             entry.title = entry.title.replace(u'\u200B', u'')
 
-            # Dict with fields to grab mapping from rss field name to FlexGet field name
-            fields = {'guid': 'guid',
-                      'author': 'author',
-                      'description': 'description',
-                      'infohash': 'torrent_info_hash'}
-            # extend the dict of fields to grab with other_fields list in config
-            for field_map in config.get('other_fields', []):
-                fields.update(field_map)
-
             # helper
             # TODO: confusing? refactor into class member ...
 
             def add_entry(ea):
                 ea['title'] = entry.title
 
-                for rss_field, flexget_field in fields.iteritems():
+                # fields dict may be modified during this loop, so loop over a copy (fields.items())
+                for rss_field, flexget_field in list(fields.items()):
                     if rss_field in entry:
-                        if not isinstance(getattr(entry, rss_field), basestring):
+                        if rss_field == 'content':
+                            content_str = ''
+                            for content in entry[rss_field]:
+                                try:
+                                    content_str += decode_html(content.value)
+                                except UnicodeDecodeError:
+                                    log.warning('Failed to decode entry `%s` field `%s`', ea['title'], rss_field)
+                            ea[flexget_field] = content_str
+                            log.debug('Field `%s` set to `%s` for `%s`', rss_field, ea[rss_field], ea['title'])
+                            continue
+                        if not isinstance(getattr(entry, rss_field), str):
                             # Error if this field is not a string
                             log.error('Cannot grab non text field `%s` from rss.', rss_field)
                             # Remove field from list of fields to avoid repeated error
-                            config['other_fields'].remove(rss_field)
+                            del fields[rss_field]
                             continue
                         if not getattr(entry, rss_field):
                             log.debug('Not grabbing blank field %s from rss for %s.', rss_field, ea['title'])
@@ -401,7 +423,7 @@ class InputRSS(object):
                 # There is more than 1 enclosure, create an Entry for each of them
                 log.debug('adding %i entries from enclosures', len(enclosures))
                 for enclosure in enclosures:
-                    if not 'href' in enclosure:
+                    if 'href' not in enclosure:
                         log.debug('RSS-entry `%s` enclosure does not have URL', entry.title)
                         continue
                     # There is a valid url for this enclosure, create an Entry for it
@@ -453,7 +475,8 @@ class InputRSS(object):
         if rss.entries:
             log.debug('Saving location in rss feed.')
             try:
-                task.simple_persistence['%s_last_entry' % url_hash] = rss.entries[0].title + rss.entries[0].get('guid', '')
+                task.simple_persistence['%s_last_entry' % url_hash] = (rss.entries[0].title +
+                                                                       rss.entries[0].get('guid', ''))
             except AttributeError:
                 log.debug('rss feed location saving skipped: no title information in first entry')
 
