@@ -1,4 +1,7 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
+import io
 import os
 import re
 import logging
@@ -24,7 +27,6 @@ class TailPosition(Base):
 
 
 class InputTail(object):
-
     """
     Parse any text for entries using regular expression.
 
@@ -74,7 +76,7 @@ class InputTail(object):
     }
 
     def format_entry(self, entry, d):
-        for k, v in d.iteritems():
+        for k, v in d.items():
             entry[k] = v % entry
 
     def on_task_input(self, task, config):
@@ -83,7 +85,7 @@ class InputTail(object):
         task.no_entries_ok = True
 
         filename = os.path.expanduser(config['file'])
-        encoding = config.get('encoding', None)
+        encoding = config.get('encoding', 'utf-8')
         with Session() as session:
             db_pos = (session.query(TailPosition).
                       filter(TailPosition.task == task.name).filter(TailPosition.filename == filename).first())
@@ -92,7 +94,7 @@ class InputTail(object):
             else:
                 last_pos = 0
 
-            with open(filename, 'r') as file:
+            with io.open(filename, 'r', encoding=encoding, errors='replace') as file:
                 if task.options.tail_reset == filename or task.options.tail_reset == task.name:
                     if last_pos == 0:
                         log.info('Task %s tail position is already zero' % task.name)
@@ -118,19 +120,12 @@ class InputTail(object):
 
                 # now parse text
 
-                while True:
-                    line = file.readline()
-                    if encoding:
-                        try:
-                            line = line.decode(encoding)
-                        except UnicodeError:
-                            raise plugin.PluginError('Failed to decode file using %s. Check encoding.' % encoding)
-
+                for line in file:
                     if not line:
                         break
 
-                    for field, regexp in entry_config.iteritems():
-                        #log.debug('search field: %s regexp: %s' % (field, regexp))
+                    for field, regexp in entry_config.items():
+                        # log.debug('search field: %s regexp: %s' % (field, regexp))
                         match = re.search(regexp, line)
                         if match:
                             # check if used field detected, in such case start with new entry
@@ -141,7 +136,8 @@ class InputTail(object):
                                     self.format_entry(entry, format_config)
                                     entries.append(entry)
                                 else:
-                                    log.info('Invalid data, entry field %s is already found once. Ignoring entry.' % field)
+                                    log.info(
+                                        'Invalid data, entry field %s is already found once. Ignoring entry.' % field)
                                 # start new entry
                                 entry = Entry()
                                 used = {}
