@@ -1,8 +1,12 @@
 """Plugin for text file or URL feeds via regex."""
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import re
 import logging
 
+import path
+
 from flexget import plugin
 from flexget.entry import Entry
 from flexget.event import event
@@ -12,7 +16,6 @@ log = logging.getLogger('text')
 
 
 class Text(object):
-
     """
     Parse any text for entries using regular expression.
 
@@ -45,6 +48,7 @@ class Text(object):
                     {'type': 'string', 'format': 'file'}
                 ]
             },
+            'encoding': {'type': 'string'},
             'entry': {
                 'type': 'object',
                 'properties': {
@@ -64,7 +68,7 @@ class Text(object):
     }
 
     def format_entry(self, entry, d):
-        for k, v in d.iteritems():
+        for k, v in d.items():
             entry[k] = v % entry
 
     @cached('text')
@@ -72,9 +76,9 @@ class Text(object):
     def on_task_input(self, task, config):
         url = config['url']
         if '://' in url:
-            lines = task.requests.get(url).iter_lines()
+            lines = task.requests.get(url).text.split('\n')
         else:
-            lines = open(url, 'rb').readlines()
+            lines = path.Path(url).lines(encoding=config.get('encoding', 'utf-8'))
 
         entry_config = config.get('entry')
         format_config = config.get('format', {})
@@ -86,8 +90,8 @@ class Text(object):
 
         # now parse text
         for line in lines:
-            for field, regexp in entry_config.iteritems():
-                #log.debug('search field: %s regexp: %s' % (field, regexp))
+            for field, regexp in entry_config.items():
+                # log.debug('search field: %s regexp: %s' % (field, regexp))
                 match = re.search(regexp, line)
                 if match:
                     # check if used field detected, in such case start with new entry
