@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import re
 import math
@@ -48,8 +50,8 @@ class MetainfoContentSize(object):
                 log.trace('setting content size to %s' % amount)
                 entry['content_size'] = int(amount)
                 continue
-            # If this entry has a local file, (it was added by listdir) grab the size.
-            elif 'location' in entry:
+            # If this entry has a local file, (it was added by filesystem plugin) grab the size.
+            elif entry.get('location'):
                 # If it is a .torrent or .nzb, don't bother getting the size as it will not be the content's size
                 if entry['location'].endswith('.torrent') or entry['location'].endswith('.nzb'):
                     continue
