@@ -1,45 +1,52 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 from datetime import datetime, timedelta
 
-from sqlalchemy import Table, Column, Integer, Float, String, Unicode, Boolean, DateTime, delete
+from sqlalchemy import Table, Column, Integer, Float, String, Unicode, Boolean, DateTime
 from sqlalchemy.schema import ForeignKey, Index
-from sqlalchemy.orm import relation, joinedload
+from sqlalchemy.orm import relation
 
 from flexget import db_schema, plugin
+from flexget.db_schema import UpgradeImpossible
 from flexget.event import event
 from flexget.entry import Entry
-from flexget.manager import Session
 from flexget.utils.log import log_once
 from flexget.utils.imdb import ImdbSearch, ImdbParser, extract_id, make_url
-from flexget.utils.sqlalchemy_utils import table_add_column
 from flexget.utils.database import with_session
-from flexget.utils.sqlalchemy_utils import table_columns, get_index_by_name, table_schema
 
-SCHEMA_VER = 4
+SCHEMA_VER = 8
 
 Base = db_schema.versioned_base('imdb_lookup', SCHEMA_VER)
 
-
 # association tables
 genres_table = Table('imdb_movie_genres', Base.metadata,
-    Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
-    Column('genre_id', Integer, ForeignKey('imdb_genres.id')),
-    Index('ix_imdb_movie_genres', 'movie_id', 'genre_id'))
+                     Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
+                     Column('genre_id', Integer, ForeignKey('imdb_genres.id')),
+                     Index('ix_imdb_movie_genres', 'movie_id', 'genre_id'))
+Base.register_table(genres_table)
 
 actors_table = Table('imdb_movie_actors', Base.metadata,
-    Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
-    Column('actor_id', Integer, ForeignKey('imdb_actors.id')),
-    Index('ix_imdb_movie_actors', 'movie_id', 'actor_id'))
+                     Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
+                     Column('actor_id', Integer, ForeignKey('imdb_actors.id')),
+                     Index('ix_imdb_movie_actors', 'movie_id', 'actor_id'))
+Base.register_table(actors_table)
 
 directors_table = Table('imdb_movie_directors', Base.metadata,
-    Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
-    Column('director_id', Integer, ForeignKey('imdb_directors.id')),
-    Index('ix_imdb_movie_directors', 'movie_id', 'director_id'))
+                        Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
+                        Column('director_id', Integer, ForeignKey('imdb_directors.id')),
+                        Index('ix_imdb_movie_directors', 'movie_id', 'director_id'))
+Base.register_table(directors_table)
+
+writers_table = Table('imdb_movie_writers', Base.metadata,
+                      Column('movie_id', Integer, ForeignKey('imdb_movies.id')),
+                      Column('writer_id', Integer, ForeignKey('imdb_writers.id')),
+                      Index('ix_imdb_movie_writers', 'movie_id', 'writer_id'))
+Base.register_table(writers_table)
 
 
 class Movie(Base):
-
     __tablename__ = 'imdb_movies'
 
     id = Column(Integer, primary_key=True)
@@ -51,6 +58,7 @@ class Movie(Base):
     genres = relation('Genre', secondary=genres_table, backref='movies')
     actors = relation('Actor', secondary=actors_table, backref='movies')
     directors = relation('Director', secondary=directors_table, backref='movies')
+    writers = relation('Writer', secondary=writers_table, backref='movies')
     languages = relation('MovieLanguage', order_by='MovieLanguage.prominence')
 
     score = Column(Float)
@@ -89,7 +97,6 @@ class Movie(Base):
 
 
 class MovieLanguage(Base):
-
     __tablename__ = 'imdb_movie_languages'
 
     movie_id = Column(Integer, ForeignKey('imdb_movies.id'), primary_key=True)
@@ -104,7 +111,6 @@ class MovieLanguage(Base):
 
 
 class Language(Base):
-
     __tablename__ = 'imdb_languages'
 
     id = Column(Integer, primary_key=True)
@@ -115,7 +121,6 @@ class Language(Base):
 
 
 class Genre(Base):
-
     __tablename__ = 'imdb_genres'
 
     id = Column(Integer, primary_key=True)
@@ -126,7 +131,6 @@ class Genre(Base):
 
 
 class Actor(Base):
-
     __tablename__ = 'imdb_actors'
 
     id = Column(Integer, primary_key=True)
@@ -139,7 +143,6 @@ class Actor(Base):
 
 
 class Director(Base):
-
     __tablename__ = 'imdb_directors'
 
     id = Column(Integer, primary_key=True)
@@ -151,8 +154,19 @@ class Director(Base):
         self.name = name
 
 
-class SearchResult(Base):
+class Writer(Base):
+    __tablename__ = 'imdb_writers'
 
+    id = Column(Integer, primary_key=True)
+    imdb_id = Column(String)
+    name = Column(Unicode)
+
+    def __init__(self, imdb_id, name=None):
+        self.imdb_id = imdb_id
+        self.name = name
+
+
+class SearchResult(Base):
     __tablename__ = 'imdb_search'
 
     id = Column(Integer, primary_key=True)
@@ -173,51 +187,18 @@ class SearchResult(Base):
     def __repr__(self):
         return '<SearchResult(title=%s,url=%s,fails=%s)>' % (self.title, self.url, self.fails)
 
+
 log = logging.getLogger('imdb_lookup')
 
 
 @db_schema.upgrade('imdb_lookup')
 def upgrade(ver, session):
-    if ver is None:
-        columns = table_columns('imdb_movies', session)
-        if not 'photo' in columns:
-            log.info('Adding photo column to imdb_movies table.')
-            table_add_column('imdb_movies', 'photo', String, session)
-        if not 'updated' in columns:
-            log.info('Adding updated column to imdb_movies table.')
-            table_add_column('imdb_movies', 'updated', DateTime, session)
-        if not 'mpaa_rating' in columns:
-            log.info('Adding mpaa_rating column to imdb_movies table.')
-            table_add_column('imdb_movies', 'mpaa_rating', String, session)
-        ver = 0
-    if ver == 0:
-        # create indexes retrospectively (~r2563)
-        log.info('Adding imdb indexes delivering up to 20x speed increase \o/ ...')
-        indexes = [get_index_by_name(actors_table, 'ix_imdb_movie_actors'),
-                   get_index_by_name(genres_table, 'ix_imdb_movie_genres'),
-                   get_index_by_name(directors_table, 'ix_imdb_movie_directors')]
-        for index in indexes:
-            if index is None:
-                log.critical('Index adding failure!')
-                continue
-            log.info('Creating index %s ...' % index.name)
-            index.create(bind=session.connection())
-        ver = 1
-    if ver == 1:
-        # http://flexget.com/ticket/1399
-        log.info('Adding prominence column to imdb_movie_languages table.')
-        table_add_column('imdb_movie_languages', 'prominence', Integer, session)
-        ver = 2
-    if ver == 2:
-        log.info('Adding search result timestamp and clearing all previous results.')
-        table_add_column('imdb_search', 'queried', DateTime, session)
-        search_table = table_schema('imdb_search', session)
-        session.execute(delete(search_table, search_table.c.fails))
-        ver = 3
-    if ver == 3:
-        log.info('Adding original title column, cached data will not have this information')
-        table_add_column('imdb_movies', 'original_title', Unicode, session)
-        ver = 4
+    # v5 We may have cached bad data due to imdb changes, just wipe everything. GitHub #697
+    # v6 The association tables were not cleared on the last upgrade, clear again. GitHub #714
+    # v7 Another layout change cached bad data. GitHub #729
+    # v8 Added writers to the DB Schema
+    if ver is None or ver <= 7:
+        raise UpgradeImpossible('Resetting imdb_lookup caches because bad data may have been cached.')
     return ver
 
 
@@ -246,6 +227,7 @@ class ImdbLookup(object):
         'imdb_languages': lambda movie: [lang.language.name for lang in movie.languages],
         'imdb_actors': lambda movie: dict((actor.imdb_id, actor.name) for actor in movie.actors),
         'imdb_directors': lambda movie: dict((director.imdb_id, director.name) for director in movie.directors),
+        'imdb_writers': lambda movie: dict((writer.imdb_id, writer.name) for writer in movie.writers),
         'imdb_mpaa_rating': 'mpaa_rating',
         # Generic fields filled by all movie lookup plugins:
         'movie_name': 'title',
@@ -261,20 +243,17 @@ class ImdbLookup(object):
             self.register_lazy_fields(entry)
 
     def register_lazy_fields(self, entry):
-        entry.register_lazy_fields(self.field_map, self.lazy_loader)
+        entry.register_lazy_func(self.lazy_loader, self.field_map)
 
-    def lazy_loader(self, entry, field):
+    def lazy_loader(self, entry):
         """Does the lookup for this entry and populates the entry fields."""
         try:
             self.lookup(entry)
         except plugin.PluginError as e:
-            log_once(unicode(e.value).capitalize(), logger=log)
-            # Set all of our fields to None if the lookup failed
-            entry.unregister_lazy_fields(self.field_map, self.lazy_loader)
-        return entry[field]
+            log_once(str(e.value).capitalize(), logger=log)
 
     @with_session
-    def imdb_id_lookup(self, movie_title=None, raw_title=None, session=None):
+    def imdb_id_lookup(self, movie_title=None, movie_year=None, raw_title=None, session=None):
         """
         Perform faster lookup providing just imdb_id.
         Falls back to using basic lookup if data cannot be found from cache.
@@ -291,7 +270,10 @@ class ImdbLookup(object):
         """
         if movie_title:
             log.debug('imdb_id_lookup: trying with title: %s' % movie_title)
-            movie = session.query(Movie).filter(Movie.title == movie_title).first()
+            query = session.query(Movie).filter(Movie.title == movie_title)
+            if movie_year is not None:
+                query = query.filter(Movie.year == movie_year)
+            movie = query.first()
             if movie:
                 log.debug('--> success! got %s returning %s' % (movie, movie.imdb_id))
                 return movie.imdb_id
@@ -343,7 +325,7 @@ class ImdbLookup(object):
                 entry['imdb_url'] = make_url(imdb_id)
             else:
                 log.debug('imdb url %s is invalid, removing it' % entry['imdb_url'])
-                del(entry['imdb_url'])
+                del (entry['imdb_url'])
 
         # no imdb_url, check if there is cached result for it or if the
         # search is known to fail
@@ -361,8 +343,6 @@ class ImdbLookup(object):
                         entry['imdb_id'] = result.imdb_id
                         entry['imdb_url'] = result.url
 
-        movie = None
-
         # no imdb url, but information required, try searching
         if not entry.get('imdb_url', eval_lazy=False) and search_allowed:
             log.verbose('Searching from imdb `%s`' % entry['title'])
@@ -424,7 +404,8 @@ class ImdbLookup(object):
                 log.exception(e)
             raise plugin.PluginError('Invalid parameter: %s' % entry['imdb_url'], log)
 
-        for att in ['title', 'score', 'votes', 'year', 'genres', 'languages', 'actors', 'directors', 'mpaa_rating']:
+        for att in ['title', 'score', 'votes', 'year', 'genres', 'languages', 'actors', 'directors', 'writers',
+                    'mpaa_rating']:
             log.trace('movie.%s: %s' % (att, getattr(movie, att)))
 
         # Update the entry fields
@@ -461,21 +442,32 @@ class ImdbLookup(object):
             if not language:
                 language = Language(name)
             movie.languages.append(MovieLanguage(language, prominence=index))
-        for imdb_id, name in parser.actors.iteritems():
+        for imdb_id, name in parser.actors.items():
             actor = session.query(Actor).filter(Actor.imdb_id == imdb_id).first()
             if not actor:
                 actor = Actor(imdb_id, name)
             movie.actors.append(actor)  # pylint:disable=E1101
-        for imdb_id, name in parser.directors.iteritems():
+        for imdb_id, name in parser.directors.items():
             director = session.query(Director).filter(Director.imdb_id == imdb_id).first()
             if not director:
                 director = Director(imdb_id, name)
             movie.directors.append(director)  # pylint:disable=E1101
+        for imdb_id, name in parser.writers.items():
+            writer = session.query(Writer).filter(Writer.imdb_id == imdb_id).first()
+            if not writer:
+                writer = Writer(imdb_id, name)
+            movie.writers.append(writer)  # pylint:disable=E1101
             # so that we can track how long since we've updated the info later
         movie.updated = datetime.now()
         session.add(movie)
         return movie
 
+    @property
+    def movie_identifier(self):
+        """Returns the plugin main identifier type"""
+        return 'imdb_id'
+
+
 @event('plugin.register')
 def register_plugin():
-    plugin.register(ImdbLookup, 'imdb_lookup', api_ver=2)
+    plugin.register(ImdbLookup, 'imdb_lookup', api_ver=2, interfaces=['task', 'movie_metainfo'])
