@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import re
 import logging
 
@@ -26,11 +28,11 @@ class MetainfoImdbUrl(object):
             # Don't override already populated imdb_ids
             if entry.get('imdb_id', eval_lazy=False):
                 continue
-            if not 'description' in entry:
+            if 'description' not in entry:
                 continue
             urls = re.findall(r'\bimdb.com/title/tt\d+\b', entry['description'])
             # Find unique imdb ids
-            imdb_ids = filter(None, set(extract_id(url) for url in urls))
+            imdb_ids = [_f for _f in set(extract_id(url) for url in urls) if _f]
             if not imdb_ids:
                 continue
 
@@ -42,6 +44,7 @@ class MetainfoImdbUrl(object):
             entry['imdb_url'] = make_url(entry['imdb_id'])
             log.debug('Found imdb url in description %s' % entry['imdb_url'])
 
+
 @event('plugin.register')
 def register_plugin():
     plugin.register(MetainfoImdbUrl, 'scan_imdb', builtin=True, api_ver=2)
