@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -23,11 +25,7 @@ class MetainfoQuality(object):
         if config is False:
             return
         for entry in task.entries:
-            entry.register_lazy_fields(['quality'], self.lazy_loader)
-
-    def lazy_loader(self, entry, field):
-        self.get_quality(entry)
-        return entry.get(field)
+            entry.register_lazy_func(self.get_quality, ['quality'])
 
     def get_quality(self, entry):
         if entry.get('quality', eval_lazy=False):
