@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -20,7 +22,7 @@ class MetainfoTask(object):
             return
 
         for entry in task.entries:
-            entry.setdefault('task', task.name)
+            entry['task'] = task.name
 
 
 @event('plugin.register')
