@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -8,7 +10,6 @@ log = logging.getLogger('extension')
 
 
 class ModifyExtension(object):
-
     """
         Allows specifying file extension explicitly when all other built-in detection mechanisms fail.
 
@@ -20,7 +21,7 @@ class ModifyExtension(object):
     schema = {'type': ['string', 'number']}
 
     def on_task_modify(self, task, config):
-        ext = unicode(config)
+        ext = str(config)
         if ext.startswith('.'):
             ext = ext[1:]
 
@@ -29,6 +30,7 @@ class ModifyExtension(object):
             entry['filename'] = '%s.%s' % (entry.get('filename', entry['title']), ext)
             log.debug('filename is now `%s`' % entry['filename'])
 
+
 @event('plugin.register')
 def register_plugin():
     plugin.register(ModifyExtension, 'extension', api_ver=2)
