@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import re
 
@@ -36,9 +38,9 @@ class Manipulate(object):
         bundle = root.accept('list').accept('dict')
         # prevent invalid indentation level
         bundle.reject_keys(['from', 'extract', 'replace', 'phase'],
-            'Option \'$key\' has invalid indentation level. It needs 2 more spaces.')
+                           'Option \'$key\' has invalid indentation level. It needs 2 more spaces.')
         edit = bundle.accept_any_key('dict')
-        edit.accept('choice', key='phase').accept_choices(['metainfo', 'filter'], ignore_case=True)
+        edit.accept('choice', key='phase').accept_choices(['metainfo', 'filter', 'modify'], ignore_case=True)
         edit.accept('text', key='from')
         edit.accept('regexp', key='extract')
         edit.accept('text', key='separator')
@@ -53,9 +55,9 @@ class Manipulate(object):
         Separates the config into a dict with a list of jobs per phase.
         Allows us to skip phases without any jobs in them.
         """
-        self.phase_jobs = {'filter': [], 'metainfo': []}
+        self.phase_jobs = {'filter': [], 'metainfo': [], 'modify': []}
         for item in config:
-            for item_config in item.itervalues():
+            for item_config in item.values():
                 # Get the phase specified for this item, or use default of metainfo
                 phase = item_config.get('phase', 'metainfo')
                 self.phase_jobs[phase].append(item)
@@ -76,6 +78,14 @@ class Manipulate(object):
         modified = sum(self.process(entry, self.phase_jobs['filter']) for entry in task.entries + task.rejected)
         log.verbose('Modified %d entries.' % modified)
 
+    @plugin.priority(255)
+    def on_task_modify(self, task, config):
+        if not self.phase_jobs['modify']:
+            # return if no jobs for this phase
+            return
+        modified = sum(self.process(entry, self.phase_jobs['modify']) for entry in task.entries + task.rejected)
+        log.verbose('Modified %d entries.' % modified)
+
     def process(self, entry, jobs):
         """Process given jobs from config for an entry.
 
@@ -86,7 +96,7 @@ class Manipulate(object):
 
         modified = False
         for item in jobs:
-            for field, config in item.iteritems():
+            for field, config in item.items():
                 from_field = field
                 if 'from' in config:
                     from_field = config['from']
@@ -125,6 +135,7 @@ class Manipulate(object):
                 entry[field] = field_value
         return modified
 
+
 @event('plugin.register')
 def register_plugin():
     plugin.register(Manipulate, 'manipulate', api_ver=2)
