@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import os
 
@@ -21,18 +23,18 @@ class TorrentFilename(object):
         # Only scan through accepted entries, as the file must have been downloaded in order to parse anything
         for entry in task.accepted:
             # skip if entry does not have file assigned
-            if not 'file' in entry:
-                log.trace('%s doesn\'t have a file associated' % entry['title'])
+            if 'file' not in entry:
+                log.trace('%s doesn\'t have a file associated', entry['title'])
                 continue
             if not os.path.exists(entry['file']):
-                entry.fail('File %s does not exists' % entry['file'])
+                log.debug('File %s does not exist', entry['file'])
                 continue
             if os.path.getsize(entry['file']) == 0:
-                entry.fail('File %s is 0 bytes in size' % entry['file'])
+                log.debug('File %s is 0 bytes in size', entry['file'])
                 continue
             if not is_torrent_file(entry['file']):
                 continue
-            log.debug('%s seems to be a torrent' % entry['title'])
+            log.debug('%s seems to be a torrent', entry['title'])
 
             # create torrent object from torrent
             try:
@@ -51,10 +53,11 @@ class TorrentFilename(object):
                 try:
                     torrent = Torrent(data)
                 except SyntaxError as e:
-                    entry.fail('%s - broken or invalid torrent file received' % e.message)
+                    entry.fail('%s - broken or invalid torrent file received' % e.args[0])
                     self.purge(entry)
                     continue
 
+                log.trace('working on torrent %s', torrent)
                 entry['torrent'] = torrent
                 entry['torrent_info_hash'] = torrent.info_hash
                 # if we do not have good filename (by download plugin)
@@ -106,7 +109,7 @@ class TorrentFilename(object):
         if os.path.exists(entry['file']):
             log.debug('removing temp file %s from %s' % (entry['file'], entry['title']))
             os.remove(entry['file'])
-        del(entry['file'])
+        del (entry['file'])
 
 
 @event('plugin.register')
