@@ -1,9 +1,8 @@
-""" Torrent Scrubber Plugin.
-"""
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 import logging
 
-from flexget import plugin, validator
+from flexget import plugin
 from flexget.event import event
 from flexget.plugins.modify.torrent import TorrentFilename
 from flexget.utils import bittorrent
@@ -96,14 +95,14 @@ class TorrentScrub(object):
             if modified:
                 entry["torrent"].content = metainfo
                 entry["torrent"].modified = True
-                log.info((("Key %s was" if len(modified) == 1 else "Keys %s were")
-                          + " scrubbed from torrent '%s'!") % (", ".join(sorted(modified)), entry['title']))
+                log.info((("Key %s was" if len(modified) == 1 else "Keys %s were") +
+                          " scrubbed from torrent '%s'!") % (", ".join(sorted(modified)), entry['title']))
                 new_infohash = entry["torrent"].info_hash
                 if infohash != new_infohash:
-                    log.warn("Info hash changed from #%s to #%s in '%s'" %
-                             (infohash, new_infohash, entry['filename']))
+                    log.warning("Info hash changed from #%s to #%s in '%s'" %
+                                (infohash, new_infohash, entry['filename']))
 
 
 @event('plugin.register')
 def register_plugin():
-    plugin.register(TorrentScrub, groups=["torrent"], api_ver=2)
+    plugin.register(TorrentScrub, interfaces=['task', 'torrent'], api_ver=2)
