@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import re
 
@@ -9,7 +11,6 @@ log = logging.getLogger('modify_torrents')
 
 
 class AddTrackers(object):
-
     """
         Adds tracker URL to torrent files.
 
@@ -28,7 +29,7 @@ class AddTrackers(object):
         for entry in task.entries:
             if 'torrent' in entry:
                 for url in config:
-                    if not url in entry['torrent'].trackers:
+                    if url not in entry['torrent'].trackers:
                         entry['torrent'].add_multitracker(url)
                         log.info('Added %s tracker to %s' % (url, entry['title']))
             if entry['url'].startswith('magnet:'):
@@ -36,7 +37,6 @@ class AddTrackers(object):
 
 
 class RemoveTrackers(object):
-
     """
         Removes trackers from torrent files using regexp matching.
 
@@ -69,7 +69,6 @@ class RemoveTrackers(object):
 
 
 class ModifyTrackers(object):
-
     """
     Modify tracker URL to torrent files.
 
@@ -102,7 +101,7 @@ class ModifyTrackers(object):
                 torrent = entry['torrent']
                 trackers = torrent.trackers
                 for item in config:
-                    for replace in item.itervalues():
+                    for replace in item.values():
                         for tracker in trackers:
                             if replace.get('from') in tracker:
                                 torrent.remove_multitracker(tracker)
