@@ -1,8 +1,11 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
 from flexget.event import event
+from flexget.utils.requests import TimedLimiter
 
 log = logging.getLogger('domain_delay')
 
@@ -19,9 +22,9 @@ class DomainDelay(object):
     schema = {'type': 'object', 'additionalProperties': {'type': 'string', 'format': 'interval'}}
 
     def on_task_start(self, task, config):
-        for domain, delay in config.iteritems():
+        for domain, delay in config.items():
             log.debug('Adding minimum interval of %s between requests to %s' % (delay, domain))
-            task.requests.set_domain_delay(domain, delay)
+            task.requests.add_domain_limiter(TimedLimiter(domain, delay))
 
 
 @event('plugin.register')
