@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import os
 
@@ -38,7 +40,7 @@ class PluginFreeSpace(object):
         ]
     }
 
-    def prepare_config(self, config):
+    def prepare_config(self, config, task):
         if isinstance(config, (float, int)):
             config = {'space': config}
         # Use config path if none is specified
@@ -48,7 +50,7 @@ class PluginFreeSpace(object):
 
     @plugin.priority(255)
     def on_task_download(self, task, config):
-        config = self.prepare_config(config)
+        config = self.prepare_config(config, task)
         # Only bother aborting if there were accepted entries this run.
         if task.accepted:
             if get_free_space(config['path']) < config['space']:
