@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -18,13 +20,11 @@ class ManualTask(object):
         if not config:
             return
         # If --task hasn't been specified disable this plugin
-        if not task.options.tasks or task.name not in task.options.tasks:
-            log.debug('Disabling task %s' % task.name)
+        if not task.options.tasks or task.name not in task.options.tasks or not task.options.allow_manual:
+            log.debug('Disabling task %s, task can only run in manual mode (via API/CLI)' % task.name)
             task.abort('manual task not specified in --tasks', silent=True)
 
 
 @event('plugin.register')
 def register_plugin():
     plugin.register(ManualTask, 'manual', api_ver=2)
-
-
