@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -14,18 +16,26 @@ class MaxReRuns(object):
     schema = {'type': 'integer'}
 
     def __init__(self):
-        self.default = Task.max_reruns
+        self.default = Task.RERUN_DEFAULT
+
+    def reset(self, task):
+        task.unlock_reruns()
+        task.max_reruns = self.default
+        log.debug('changing max task rerun variable back to: %s' % self.default)
 
     def on_task_start(self, task, config):
         self.default = task.max_reruns
+        log.debug('saving old max task rerun value: %s', self.default)
         task.max_reruns = int(config)
+        task.lock_reruns()
         log.debug('changing max task rerun variable to: %s' % config)
 
     def on_task_exit(self, task, config):
-        log.debug('restoring max task rerun variable to: %s' % self.default)
-        task.max_reruns = self.default
+        if task.rerun_count > task.max_reruns:
+            self.reset(task)
 
-    on_task_abort = on_task_exit
+    def on_task_abort(self, task, config):
+        self.reset(task)
 
 
 @event('plugin.register')
