@@ -1,6 +1,7 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import copy
-import functools
 import logging
 
 from flexget import plugin
@@ -14,7 +15,7 @@ class RunTask(object):
     schema = {
         'type': 'object',
         'properties': {
-            'task': {'type': 'string'},
+            'task': one_or_more({'type': 'string'}),
             'when': one_or_more({'type': 'string',
                                  'enum': ['accepted', 'rejected', 'failed', 'no_entries', 'aborted', 'always']})
         },
@@ -22,25 +23,40 @@ class RunTask(object):
         'additionalProperties': False
     }
 
+    def prepare_config(self, config):
+        tasks = config['task']
+        if not isinstance(tasks, list):
+            config['task'] = [tasks]
+
+        config.setdefault('when', ['always'])
+
+        when = config['when']
+        if when and not isinstance(when, list):
+            config['when'] = [when]
+
+        return config
+
     def on_task_exit(self, task, config):
-        config.setdefault('when', 'always')
+        config = self.prepare_config(config)
         conditions = [
             task.accepted and 'accepted' in config['when'],
             task.rejected and 'rejected' in config['when'],
+            task.failed and 'failed' in config['when'],
             not task.all_entries and 'no_entries' in config['when'],
             'always' in config['when']
         ]
         if any(conditions):
-            self.run_task(task, config['task'])
+            self.run_tasks(task, config['task'])
 
     def on_task_abort(self, task, config):
-        if 'aborted' in config:
-            self.run_task(task, config['task'])
+        config = self.prepare_config(config)
+        if 'aborted' in config['when']:
+            self.run_tasks(task, config['task'])
 
-    def run_task(self, current_task, run_task):
-        log.info('Scheduling %s task to run' % run_task)
+    def run_tasks(self, current_task, tasks):
+        log.info('Scheduling tasks %s to run', tasks)
         options = copy.copy(current_task.options)
-        options.tasks = [run_task]
+        options.tasks = tasks
         current_task.manager.execute(options=options)
 
 
