@@ -1,7 +1,10 @@
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.moves.urllib.parse import urlparse, unquote
+
 import logging
 import os
 import ftplib
-from urlparse import urlparse
 
 from flexget import plugin
 from flexget.event import event
@@ -18,8 +21,10 @@ class OutputFtp(object):
 
         config:
             ftp_download:
-              tls: False
-              ftp_tmp_path: /tmp
+              use-ssl: <True/False>
+              ftp_tmp_path: <path>
+              delete_origin: <True/False>
+              download_empty_dirs: <True/False>
 
         TODO:
           - Resume downloads
@@ -33,7 +38,8 @@ class OutputFtp(object):
         'properties': {
             'use-ssl': {'type': 'boolean', 'default': False},
             'ftp_tmp_path': {'type': 'string', 'format': 'path'},
-            'delete_origin': {'type': 'boolean', 'default' : False}
+            'delete_origin': {'type': 'boolean', 'default': False},
+            'download_empty_dirs': {'type': 'boolean', 'default': False}
         },
         'additionalProperties': False
     }
@@ -42,6 +48,7 @@ class OutputFtp(object):
         config.setdefault('use-ssl', False)
         config.setdefault('delete_origin', False)
         config.setdefault('ftp_tmp_path', os.path.join(task.manager.config_base, 'temp'))
+        config.setdefault('download_empty_dirs', False)
         return config
 
     def ftp_connect(self, config, ftp_url, current_path):
@@ -49,11 +56,16 @@ class OutputFtp(object):
             ftp = ftplib.FTP_TLS()
         else:
             ftp = ftplib.FTP()
+
+        # ftp.set_debuglevel(2)
         log.debug("Connecting to " + ftp_url.hostname)
         ftp.connect(ftp_url.hostname, ftp_url.port)
         ftp.login(ftp_url.username, ftp_url.password)
+        if config['use-ssl']:
+            ftp.prot_p()
         ftp.sendcmd('TYPE I')
         ftp.set_pasv(True)
+        log.debug("Changing directory to: " + current_path)
         ftp.cwd(current_path)
 
         return ftp
@@ -61,7 +73,7 @@ class OutputFtp(object):
     def check_connection(self, ftp, config, ftp_url, current_path):
         try:
             ftp.voidcmd("NOOP")
-        except:
+        except (IOError, ftplib.Error):
             ftp = self.ftp_connect(config, ftp_url, current_path)
         return ftp
 
@@ -69,11 +81,12 @@ class OutputFtp(object):
         config = self.prepare_config(config, task)
         for entry in task.accepted:
             ftp_url = urlparse(entry.get('url'))
+            ftp_url = ftp_url._replace(path=unquote(ftp_url.path))
             current_path = os.path.dirname(ftp_url.path)
             try:
                 ftp = self.ftp_connect(config, ftp_url, current_path)
-            except:
-                entry.failed("Unable to connect to server")
+            except ftplib.all_errors as e:
+                entry.fail("Unable to connect to server : %s" % (e))
                 break
 
             if not os.path.isdir(config['ftp_tmp_path']):
@@ -111,6 +124,10 @@ class OutputFtp(object):
             return ftp
 
         if not dirs:
+            if config['download_empty_dirs']:
+                os.mkdir(tmp_path)
+            else:
+                log.debug("Empty directory, skipping.")
             return ftp
 
         for file_name in (path for path in dirs if path not in ('.', '..')):
@@ -146,7 +163,7 @@ class OutputFtp(object):
         try:
             ftp.sendcmd("TYPE I")
             file_size = ftp.size(file_name)
-        except Exception as e:
+        except Exception:
             file_size = 1
 
         max_attempts = 5
@@ -163,7 +180,7 @@ class OutputFtp(object):
                     ftp.retrbinary('RETR %s' % file_name, local_file.write)
             except Exception as error:
                 if max_attempts != 0:
-                    log.debug("Retrying download after error %s" % error);
+                    log.debug("Retrying download after error %s" % error)
                 else:
                     log.error("Too many errors downloading %s. Aborting." % file_name)
                     break
