@@ -1,19 +1,19 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 from datetime import datetime
 
-from sqlalchemy import Column, String, Integer, DateTime, Unicode, desc
+from sqlalchemy import Column, String, Integer, DateTime, Unicode
 
-from flexget import options, plugin
+from flexget import plugin
 from flexget.event import event
-from flexget.logger import console
-from flexget.manager import Base, Session
+from flexget.manager import Base
 
 log = logging.getLogger('history')
 
 
 class History(Base):
-
     __tablename__ = 'history'
 
     id = Column(Integer, primary_key=True)
@@ -30,6 +30,17 @@ class History(Base):
     def __str__(self):
         return '<History(filename=%s,task=%s)>' % (self.filename, self.task)
 
+    def to_dict(self):
+        return {
+            'id': self.id,
+            'task': self.task,
+            'filename': self.filename,
+            'url': self.url,
+            'title': self.title,
+            'time': self.time.isoformat(),
+            'details': self.details,
+        }
+
 
 class PluginHistory(object):
     """Records all accepted entries for later lookup"""
@@ -54,36 +65,6 @@ class PluginHistory(object):
             task.session.add(item)
 
 
-def do_cli(manager, options):
-    session = Session()
-    try:
-        console('-- History: ' + '-' * 67)
-        query = session.query(History)
-        if options.search:
-            search_term = options.search.replace(' ', '%').replace('.', '%')
-            query = query.filter(History.title.like('%' + search_term + '%'))
-        query = query.order_by(desc(History.time)).limit(options.limit)
-        for item in reversed(query.all()):
-            console(' Task    : %s' % item.task)
-            console(' Title   : %s' % item.title)
-            console(' Url     : %s' % item.url)
-            if item.filename:
-                console(' Stored  : %s' % item.filename)
-            console(' Time    : %s' % item.time.strftime("%c"))
-            console(' Details : %s' % item.details)
-            console('-' * 79)
-    finally:
-        session.close()
-
-
-@event('options.register')
-def register_parser_arguments():
-    parser = options.register_command('history', do_cli, help='view the history of entries that FlexGet has accepted')
-    parser.add_argument('--limit', action='store', type=int, metavar='NUM', default=50,
-                        help='limit to %(metavar)s results')
-    parser.add_argument('--search', action='store', metavar='TERM', help='limit to results that contain %(metavar)s')
-
-
 @event('plugin.register')
 def register_plugin():
     plugin.register(PluginHistory, 'history', builtin=True, api_ver=2)
