@@ -1,3 +1,6 @@
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -5,6 +8,7 @@ from flexget.event import event
 
 log = logging.getLogger('mock_output')
 
+
 class MockOutput(object):
     """
     Debugging plugin which records a copy of all accepted entries into a list stored in `mock_output` attribute
