@@ -1,6 +1,9 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import base64
 import hashlib
+import io
 import logging
 import datetime
 import os
@@ -18,7 +21,7 @@ Base = db_schema.versioned_base('make_rss', 0)
 rss2gen = True
 try:
     import PyRSS2Gen
-except:
+except ImportError:
     rss2gen = False
 
 
@@ -26,7 +29,7 @@ except:
 def upgrade(ver, session):
     if ver is None:
         columns = table_columns('make_rss', session)
-        if not 'rsslink' in columns:
+        if 'rsslink' not in columns:
             log.info('Adding rsslink column to table make_rss.')
             table_add_column('make_rss', 'rsslink', String, session)
         ver = 0
@@ -34,7 +37,6 @@ def upgrade(ver, session):
 
 
 class RSSEntry(Base):
-
     __tablename__ = 'make_rss'
 
     id = Column(Integer, primary_key=True)
@@ -156,7 +158,7 @@ class OutputRSS(object):
         config.setdefault('encoding', 'iso-8859-1')
         config.setdefault('link', ['imdb_url', 'input_url'])
         config.setdefault('title', '{{title}} (from {{task}})')
-        config.setdefault('template', 'default')
+        config.setdefault('template', 'rss')
         # add url as last resort
         config['link'].append('url')
         return config
@@ -179,7 +181,7 @@ class OutputRSS(object):
             try:
                 rss.title = entry.render(config['title'])
             except RenderError as e:
-                log.error('Error rendering jinja title for `%s` falling back to entry title: %s' % (entry['title'], e))
+                log.error('Error rendering jinja title for `%s` falling back to entry title: %s', entry['title'], e)
                 rss.title = entry['title']
             for field in config['link']:
                 if field in entry:
@@ -187,18 +189,18 @@ class OutputRSS(object):
                     break
 
             try:
-                template = get_template(config['template'], 'rss')
+                template = get_template(config['template'], scope='task')
             except ValueError as e:
                 raise plugin.PluginError('Invalid template specified: %s' % e)
             try:
                 rss.description = render_from_entry(template, entry)
             except RenderError as e:
-                log.error('Error while rendering entry %s, falling back to plain title: %s' % (entry, e))
+                log.error('Error while rendering entry %s, falling back to plain title: %s', entry, e)
                 rss.description = entry['title'] + ' - (Render Error)'
             rss.file = config['file']
 
             # TODO: check if this exists and suggest disabling history if it does since it shouldn't happen normally ...
-            log.debug('Saving %s into rss database' % entry['title'])
+            log.debug('Saving %s into rss database', entry['title'])
             task.session.add(rss)
 
         if not rss2gen:
@@ -207,7 +209,7 @@ class OutputRSS(object):
         if task.options.learn:
             return
 
-        db_items = task.session.query(RSSEntry).filter(RSSEntry.file == config['file']).\
+        db_items = task.session.query(RSSEntry).filter(RSSEntry.file == config['file']). \
             order_by(RSSEntry.published.desc()).all()
 
         # make items
@@ -226,15 +228,15 @@ class OutputRSS(object):
                 hasher.update(db_item.title.encode('utf8'))
                 hasher.update(db_item.description.encode('utf8'))
                 hasher.update(db_item.link.encode('utf8'))
-                guid = base64.urlsafe_b64encode(hasher.digest())
-                guid = PyRSS2Gen.Guid(guid, isPermaLink = False)
+                guid = base64.urlsafe_b64encode(hasher.digest()).decode('ascii')
+                guid = PyRSS2Gen.Guid(guid, isPermaLink=False)
 
                 gen = {'title': db_item.title,
                        'description': db_item.description,
                        'link': db_item.link,
                        'pubDate': db_item.published,
                        'guid': guid}
-                log.trace('Adding %s into rss %s' % (gen['title'], config['file']))
+                log.trace('Adding %s into rss %s', gen['title'], config['file'])
                 rss_items.append(PyRSS2Gen.RSSItem(**gen))
             else:
                 # no longer needed
@@ -254,16 +256,16 @@ class OutputRSS(object):
 
         # write rss
         fn = os.path.expanduser(config['file'])
-        with open(fn, 'w') as file:
+        with io.open(fn, 'wb') as file:
             try:
-                log.verbose('Writing output rss to %s' % fn)
+                log.verbose('Writing output rss to %s', fn)
                 rss.write_xml(file, encoding=config['encoding'])
             except LookupError:
-                log.critical('Unknown encoding %s' % config['encoding'])
+                log.critical('Unknown encoding %s', config['encoding'])
                 return
             except IOError:
                 # TODO: plugins cannot raise PluginWarnings in terminate event ..
-                log.critical('Unable to write %s' % fn)
+                log.critical('Unable to write %s', fn)
                 return
 
 
