@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -11,15 +13,15 @@ log = logging.getLogger('set_series_begin')
 class SetSeriesBegin(object):
     """
     Set the first episode for series. Uses series_name and series_id.
-    
+
     Example::
 
       set_series_begin: yes
-    
+
     """
-    
+
     schema = {'type': 'boolean'}
-    
+
     def on_task_output(self, task, config):
         if not (config and task.accepted):
             return
@@ -33,8 +35,8 @@ class SetSeriesBegin(object):
                 try:
                     set_series_begin(fshow, entry['series_id'])
                 except ValueError as e:
-                    self.log.error('An error occurred trying to set begin for %s: %s' % (entry['series_name'], e))
-                self.log.info('First episode for "%s" set to %s' % (entry['series_name'], entry['series_id']))
+                    log.error('An error occurred trying to set begin for %s: %s', entry['series_name'], e)
+                log.info('First episode for "%s" set to %s', entry['series_name'], entry['series_id'])
 
 
 @event('plugin.register')
