@@ -1,49 +1,27 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.utils import native
 
 import datetime
 import logging
 import re
 import time
 
-import guessit
-from guessit.containers import PropertiesContainer, NoValidator
-from guessit.matcher import GuessFinder
-from guessit.plugins.transformers import Transformer, add_transformer
+from guessit.rules import rebulk_builder
+from guessit.api import GuessItApi, GuessitException
+from rebulk import Rebulk
+from rebulk.pattern import RePattern
 
 from flexget import plugin
 from flexget.event import event
 from flexget.utils import qualities
-from .parser_common import clean_value, old_assume_quality
+from .parser_common import old_assume_quality
 from .parser_common import ParsedEntry, ParsedVideoQuality, ParsedVideo, ParsedSerie, ParsedMovie
 
-
 log = logging.getLogger('parser_guessit')
-# Guessit debug log is a bit too verbose
-logging.getLogger('guessit').setLevel(logging.INFO)
 
-
-class GuessRegexpId(Transformer):
-    def __init__(self):
-        Transformer.__init__(self, 21)
-
-    def supported_properties(self):
-        return ['regexpId']
-
-    def guess_regexps_id(self, string, node=None, options=None):
-        container = PropertiesContainer(enhance=False, canonical_from_pattern=False)
-        for regexp in options.get("id_regexps"):
-            container.register_property('regexpId', regexp, confidence=1.0, validator=NoValidator())
-        found = container.find_properties(string, node, options)
-        return container.as_guess(found, string)
-
-    def should_process(self, mtree, options=None):
-        return options and options.get("id_regexps")
-
-    def process(self, mtree, options=None):
-        GuessFinder(self.guess_regexps_id, None, self.log, options).process_nodes(mtree.unidentified_leaves())
-
-
-add_transformer('guess_regexp_id = flexget.plugins.parsers.parser_guessit:GuessRegexpId')
+logging.getLogger('rebulk').setLevel(logging.WARNING)
+logging.getLogger('guessit').setLevel(logging.WARNING)
 
 
 class GuessitParsedEntry(ParsedEntry):
@@ -53,7 +31,7 @@ class GuessitParsedEntry(ParsedEntry):
 
     @property
     def parsed_group(self):
-        return self._guess_result.get('releaseGroup')
+        return self._guess_result.get('release_group')
 
     @property
     def parsed_type(self):
@@ -72,7 +50,7 @@ class GuessitParsedEntry(ParsedEntry):
             version = -1
         else:
             version = version - 1
-        proper_count = self._guess_result.get('properCount', 0)
+        proper_count = self._guess_result.get('proper_count', 0)
         fastsub = 'Fastsub' in self._guess_result.get('other', [])
         return version + proper_count - (5 if fastsub else 0)
 
@@ -87,7 +65,7 @@ class GuessitParsedVideoQuality(ParsedVideoQuality):
 
     @property
     def video_codec(self):
-        return self._guess_result.get('videoCodec')
+        return self._guess_result.get('video_codec')
 
     @property
     def source(self):
@@ -99,23 +77,23 @@ class GuessitParsedVideoQuality(ParsedVideoQuality):
 
     @property
     def audio_codec(self):
-        return self._guess_result.get('audioCodec')
+        return self._guess_result.get('audio_codec')
 
     @property
     def video_profile(self):
-        return self._guess_result.get('videoProfile')
+        return self._guess_result.get('video_profile')
 
     @property
     def screen_size(self):
-        return self._guess_result.get('screenSize')
+        return self._guess_result.get('screen_size')
 
     @property
     def audio_channels(self):
-        return self._guess_result.get('audioChannels')
+        return self._guess_result.get('audio_channels')
 
     @property
     def audio_profile(self):
-        return self._guess_result.get('audioProfile')
+        return self._guess_result.get('audio_profile')
 
     @property
     def old_resolution(self):
@@ -156,7 +134,7 @@ class GuessitParsedVideoQuality(ParsedVideoQuality):
         codec = self.old_codec
         audio = self.old_audio
 
-        old_quality = qualities.Quality(' '.join(filter(None, [resolution, source, codec, audio])))
+        old_quality = qualities.Quality(' '.join([_f for _f in [resolution, source, codec, audio] if _f]))
         old_quality = old_assume_quality(old_quality, assumed_quality)
 
         return old_quality
@@ -179,11 +157,11 @@ class GuessitParsedVideo(GuessitParsedEntry, ParsedVideo):
 
     @property
     def subtitle_languages(self):
-        return self._guess_result.get('subtitleLanguage')
+        return self._guess_result.get('subtitle_language')
 
     @property
     def languages(self):
-        return self._guess_result.get('Language')
+        return self._guess_result.get('language')
 
     @property
     def year(self):
@@ -198,6 +176,30 @@ class GuessitParsedMovie(GuessitParsedVideo, ParsedMovie):
     def title(self):
         return self._guess_result.get('title')
 
+    @property
+    def fields(self):
+        """
+        Return a dict of all parser fields
+        """
+        return {
+            'movie_parser': self,
+            'movie_name': self.name,
+            'movie_year': self.year,
+            'proper': self.proper,
+            'proper_count': self.proper_count,
+            'release_group': self.parsed_group,
+            'is_3d': self.is_3d,
+            'subtitle_languages': self.subtitle_languages,
+            'languages': self.languages,
+            'video_codec': self.quality2.video_codec,
+            'format': self.quality2.format,
+            'audio_codec': self.quality2.audio_codec,
+            'video_profile': self.quality2.video_profile,
+            'screen_size': self.quality2.screen_size,
+            'audio_channels': self.quality2.audio_channels,
+            'audio_profile': self.quality2.audio_profile
+        }
+
 
 class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
     part_re = re.compile('part\\s?(\\d+)', re.IGNORECASE)
@@ -207,7 +209,9 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
 
     @property
     def series(self):
-        return self._guess_result.get('series')
+        if self._guess_result.get('country') and hasattr(self._guess_result.get('country'), 'alpha2'):
+            return "%s (%s)" % (self._guess_result.get('title'), self._guess_result.get('country').alpha2)
+        return self._guess_result.get('title')
 
     @property
     def country(self):
@@ -219,7 +223,7 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
 
     @property
     def regexp_id(self):
-        regexp_id = self._guess_result.get('regexpId')
+        regexp_id = [match.value for match in self._guess_result.matches['regexpId']]
         if isinstance(regexp_id, list):
             return '-'.join(regexp_id)
         else:
@@ -227,19 +231,20 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
 
     @property
     def title(self):
-        return self._guess_result.get('title')
+        return self._guess_result.get('episode_title')
 
     @property
     def special(self):
-        return self.episode_details and len(self.episode_details) > 0 or (self.title and self.title.lower().strip() == 'special')
+        return (self.episode_details and len(self.episode_details) > 0 or
+                (self.title and self.title.lower().strip() == 'special'))
 
     @property
     def episode_details(self):
-        return self._guess_result.get('episodeDetails')
+        return self._guess_result.get('episode_details')
 
     @property
     def episode(self):
-        episode = self._guess_result.get('episodeNumber')
+        episode = self._guess_result.get('episode')
         if episode is None and 'part' in self._guess_result and not self.date:
             return self._guess_result.get('part')
         if episode is None and self.title:
@@ -250,7 +255,9 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
 
     @property
     def episodes(self):
-        return len(self._guess_result.get('episodeList', filter(lambda x: x is not None, [self.episode])))
+        if 'episode' not in self._guess_result.values_list:
+            return len(self._guess_result.values_list.get('part', []))
+        return len(self._guess_result.values_list['episode'])
 
     @property
     def date(self):
@@ -269,7 +276,7 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
         if season is None and self.episode and not self.allow_seasonless:
             if 'part' in self._guess_result:
                 return 1
-            episode_raw = self._guess_result.metadata('episodeNumber').raw
+            episode_raw = self._guess_result.matches['episode'][0].initiator.raw
             if episode_raw and any(c.isalpha() and c.lower() != 'v' for c in episode_raw):
                 return 1
         return season
@@ -278,11 +285,30 @@ class GuessitParsedSerie(GuessitParsedVideo, ParsedSerie):
     def valid_strict(self):
         return True
 
+    @property
+    def season_pack(self):
+        # Guessit does not support season pack
+        return False
+
+
+def _id_regexps_function(input_string, context):
+    ret = []
+    for regexp in context.get('id_regexps'):
+        for match in RePattern(regexp, children=True).matches(input_string, context):
+            ret.append(match.span)
+    return ret
+
+
+_id_regexps = Rebulk().functional(_id_regexps_function, name='regexpId',
+                                  disabled=lambda context: not context.get('id_regexps'))
+
+guessit_api = GuessItApi(rebulk_builder().rebulk(_id_regexps))
+
 
 class ParserGuessit(object):
     def _guessit_options(self, options):
-        settings = {'name_only': True, 'clean_function': clean_value, 'allowed_languages': ['en', 'fr'],
-                    'allowed_countries': ['us', 'uk', 'gb']}
+        settings = {'name_only': True, 'allowed_languages': ['en', 'fr'], 'allowed_countries': ['us', 'uk', 'gb']}
+        # 'clean_function': clean_value
         options['episode_prefer_number'] = not options.get('identified_by') == 'ep'
         if options.get('allow_groups'):
             options['expected_group'] = options['allow_groups']
@@ -298,8 +324,10 @@ class ParserGuessit(object):
         log.debug('Parsing movie: `%s` [options: %s]', data, kwargs)
         start = time.clock()
         guessit_options = self._guessit_options(kwargs)
-        guess_result = guessit.guess_file_info(data, options=guessit_options, type='movie')
-        parsed = GuessitParsedMovie(data, kwargs.pop('name', None), guess_result, **kwargs)
+        guessit_options['type'] = 'movie'
+        guess_result = guessit_api.guessit(data, options=guessit_options)
+        # NOTE: Guessit expects str on PY3 and unicode on PY2 hence the use of future.utils.native
+        parsed = GuessitParsedMovie(native(data), kwargs.pop('name', None), guess_result, **kwargs)
         end = time.clock()
         log.debug('Parsing result: %s (in %s ms)', parsed, (end - start) * 1000)
         return parsed
@@ -309,17 +337,24 @@ class ParserGuessit(object):
         log.debug('Parsing series: `%s` [options: %s]', data, kwargs)
         guessit_options = self._guessit_options(kwargs)
         if kwargs.get('name') and not guessit_options.get('strict_name'):
-            guessit_options['expected_series'] = [kwargs['name']]
+            expected_title = kwargs['name']
+            expected_title = expected_title.replace('\'', '(?:\'|\\\'|\\\\\'|-|)?')  # apostrophe support
+            guessit_options['expected_title'] = ['re:' + expected_title]
+        if kwargs.get('id_regexps'):
+            guessit_options['id_regexps'] = kwargs.get('id_regexps')
         start = time.clock()
         # If no series name is provided, we don't tell guessit what kind of match we are looking for
         # This prevents guessit from determining that too general of matches are series
         parse_type = 'episode' if kwargs.get('name') else None
-        guess_result = guessit.guess_file_info(data, options=guessit_options, type=parse_type)
-        if guess_result.get('type') != 'episode':
-            # TODO: All other failures return an invalid parser. This is just a hack to match. Maybe exception instead?
-            class InvalidParser(object):
-                valid = False
-            return InvalidParser()
+        if parse_type:
+            guessit_options['type'] = parse_type
+
+        # NOTE: Guessit expects str on PY3 and unicode on PY2 hence the use of future.utils.native
+        try:
+            guess_result = guessit_api.guessit(native(data), options=guessit_options)
+        except GuessitException:
+            log.warning('Parsing %s with guessit failed. Most likely a unicode error.', data)
+            guess_result = {}
         parsed = GuessitParsedSerie(data, kwargs.pop('name', None), guess_result, **kwargs)
         end = time.clock()
         log.debug('Parsing result: %s (in %s ms)', parsed, (end - start) * 1000)
@@ -328,4 +363,4 @@ class ParserGuessit(object):
 
 @event('plugin.register')
 def register_plugin():
-    plugin.register(ParserGuessit, 'parser_guessit', groups=['movie_parser', 'series_parser'], api_ver=2)
+    plugin.register(ParserGuessit, 'parser_guessit', interfaces=['movie_parser', 'series_parser'], api_ver=2)
