@@ -1,4 +1,5 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 import logging
 import time
@@ -10,12 +11,12 @@ from flexget.utils.titles.movie import MovieParser
 from flexget.utils.titles.series import SeriesParser
 from .parser_common import ParseWarning
 
-
 log = logging.getLogger('parser_internal')
 
 
 class ParserInternal(object):
     # movie_parser API
+
     @plugin.priority(1)
     def parse_movie(self, data, **kwargs):
         log.debug('Parsing movie: `%s` kwargs: %s', data, kwargs)
@@ -46,4 +47,4 @@ class ParserInternal(object):
 
 @event('plugin.register')
 def register_plugin():
-    plugin.register(ParserInternal, 'parser_internal', groups=['movie_parser', 'series_parser'], api_ver=2)
+    plugin.register(ParserInternal, 'parser_internal', interfaces=['movie_parser', 'series_parser'], api_ver=2)
