@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 
 from flexget import plugin
@@ -20,12 +22,14 @@ def init_parsers(manager):
     """Prepare our list of parsing plugins and default parsers."""
     for parser_type in PARSER_TYPES:
         parsers[parser_type] = {}
-        for p in plugin.get_plugins(group=parser_type + '_parser'):
+        for p in plugin.get_plugins(interface=parser_type + '_parser'):
             parsers[parser_type][p.name.replace('parser_', '')] = p.instance
         # Select default parsers based on priority
         func_name = 'parse_' + parser_type
-        default_parsers[parser_type] = max(parsers[parser_type].iteritems(),
-                                          key=lambda p: getattr(getattr(p[1], func_name), 'priority', 0))[0]
+        default_parsers[parser_type] = max(iter(parsers[parser_type].items()),
+                                           key=lambda p: getattr(getattr(p[1], func_name), 'priority', 0))[0]
+        log.debug('setting default %s parser to %s. (options: %s)' %
+                  (parser_type, default_parsers[parser_type], parsers[parser_type]))
 
 
 class PluginParsing(object):
@@ -36,7 +40,7 @@ class PluginParsing(object):
         # Create a schema allowing only our registered parsers to be used under the key of each parser type
         properties = {}
         for parser_type in PARSER_TYPES:
-            parser_names = [p.name.replace('parser_', '') for p in plugin.get_plugins(group=parser_type + '_parser')]
+            parser_names = [p.name.replace('parser_', '') for p in plugin.get_plugins(interface=parser_type + '_parser')]
             properties[parser_type] = {'type': 'string', 'enum': parser_names}
         s = {
             'type': 'object',
@@ -50,11 +54,11 @@ class PluginParsing(object):
         if config:
             selected_parsers.update(config)
 
-    def on_task_end(self, task, config):
+    def on_task_exit(self, task, config):
         # Restore default parsers for next task run
         selected_parsers.clear()
 
-    on_task_abort = on_task_end
+    on_task_abort = on_task_exit
 
     def parse_series(self, data, name=None, **kwargs):
         """
