@@ -1,4 +1,6 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import re
 import random
@@ -8,14 +10,29 @@ from flexget.event import event
 
 log = logging.getLogger('torrent_cache')
 
-MIRRORS = ['http://torrage.com/torrent/',
-           'https://torcache.net/torrent/',
-           'http://zoink.it/torrent/']
+MIRRORS = [
+    'https://thetorrent.org/',
+    'http://torrage.com/torrent/',
+    'http://zoink.it/torrent/',
+    'http://itorrents.org/torrent/',
+]
 
 
 class TorrentCache(object):
     """Adds urls to torrent cache sites to the urls list."""
 
+    def infohash_urls(self, info_hash):
+        """
+        Other plugins may use this to make downloadable URLs
+        from infohash.
+
+        :param str info_hash: Torrent infohash
+        :returns: shuffled urls from which infohash can be retrieved
+        """
+        urls = [host + info_hash.upper() + '.torrent' for host in MIRRORS]
+        random.shuffle(urls)
+        return urls
+
     @plugin.priority(120)
     def on_task_urlrewrite(self, task, config):
         for entry in task.accepted:
