@@ -1,11 +1,13 @@
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 import copy
-import hashlib
 import itertools
 import logging
 import threading
-from functools import wraps
+import random
+import string
+from functools import wraps, total_ordering
 
 from sqlalchemy import Column, Integer, String, Unicode
 
@@ -20,6 +22,8 @@ from flexget.plugin import (
 from flexget.utils import requests
 from flexget.utils.database import with_session
 from flexget.utils.simple_persistence import SimpleTaskPersistence
+from flexget.utils.tools import get_config_hash, MergeException, merge_dict_from_to
+from flexget.utils.template import render_from_task, FlexGetTemplate
 
 log = logging.getLogger('task')
 Base = db_schema.versioned_base('feed', 0)
@@ -44,6 +48,8 @@ def config_changed(task=None, session=None):
     Forces config_modified flag to come out true on next run of `task`. Used when the db changes, and all
     entries need to be reprocessed.
 
+    .. WARNING: DO NOT (FURTHER) USE FROM PLUGINS
+
     :param task: Name of the task. If `None`, will be set for all tasks.
     """
     log.debug('Marking config for %s as changed.' % (task or 'all tasks'))
@@ -54,7 +60,6 @@ def config_changed(task=None, session=None):
 
 
 def use_task_logging(func):
-
     @wraps(func)
     def wrapper(self, *args, **kw):
         # Set the task name in the logger and capture output
@@ -74,12 +79,12 @@ class EntryIterator(object):
 
     def __init__(self, entries, states):
         self.all_entries = entries
-        if isinstance(states, basestring):
+        if isinstance(states, str):
             states = [states]
         self.filter = lambda e: e._state in states
 
     def __iter__(self):
-        return itertools.ifilter(self.filter, self.all_entries)
+        return filter(self.filter, self.all_entries)
 
     def __bool__(self):
         return any(e for e in self)
@@ -94,6 +99,8 @@ class EntryIterator(object):
         return itertools.chain(other, self)
 
     def __getitem__(self, item):
+        if isinstance(item, slice):
+            return list(itertools.islice(self, item.start, item.stop))
         if not isinstance(item, int):
             raise ValueError('Index must be integer.')
         for index, entry in enumerate(self):
@@ -102,9 +109,6 @@ class EntryIterator(object):
         else:
             raise IndexError('%d is out of bounds' % item)
 
-    def __getslice__(self, a, b):
-        return list(itertools.islice(self, a, b))
-
     def reverse(self):
         self.all_entries.sort(reverse=True)
 
@@ -144,8 +148,8 @@ class TaskAbort(Exception):
         return 'TaskAbort(reason=%s, silent=%s)' % (self.reason, self.silent)
 
 
+@total_ordering
 class Task(object):
-
     """
     Represents one task in the configuration.
 
@@ -176,11 +180,14 @@ class Task(object):
 
     """
 
-    max_reruns = 5
     # Used to determine task order, when priority is the same
     _counter = itertools.count()
 
-    def __init__(self, manager, name, config=None, options=None, output=None, loglevel=None, priority=None):
+    RERUN_DEFAULT = 5
+    RERUN_MAX = 100
+
+    def __init__(self, manager, name, config=None, options=None, output=None, loglevel=None, priority=None,
+                 suppress_warnings=None):
         """
         :param Manager manager: Manager instance.
         :param string name: Name of the task.
@@ -190,9 +197,11 @@ class Task(object):
         :param loglevel: Custom loglevel, only log messages at this level will be sent to `output`
         :param priority: If multiple tasks are waiting to run, the task with the lowest priority will be run first.
             The default is 0, if the cron option is set though, the default is lowered to 10.
+        :param suppress_warnings: Allows suppressing log warning about missing plugin in key phases
 
         """
-        self.name = unicode(name)
+        self.name = str(name)
+        self.id = ''.join(random.choice(string.digits) for _ in range(6))
         self.manager = manager
         if config is None:
             config = manager.config['tasks'].get(name, {})
@@ -204,9 +213,13 @@ class Task(object):
             options_namespace = copy.copy(self.manager.options.execute)
             options_namespace.__dict__.update(options)
             options = options_namespace
+        # If execution hasn't specifically set the `allow_manual` flag, set it to False by default
+        if not hasattr(options, 'allow_manual'):
+            setattr(options, 'allow_manual', False)
         self.options = options
         self.output = output
         self.loglevel = loglevel
+        self.suppress_warnings = suppress_warnings or []
         if priority is None:
             self.priority = 10 if self.options.cron else 0
         else:
@@ -218,8 +231,10 @@ class Task(object):
         # simple persistence
         self.simple_persistence = SimpleTaskPersistence(self)
 
-        # not to be reset
+        # rerun related flags and values
         self._rerun_count = 0
+        self._max_reruns = Task.RERUN_DEFAULT
+        self._reruns_locked = False
 
         self.config_modified = None
 
@@ -244,6 +259,41 @@ class Task(object):
         self.current_phase = None
         self.current_plugin = None
 
+    @property
+    def max_reruns(self):
+        """How many times task can be rerunned before stopping"""
+        return self._max_reruns
+
+    @max_reruns.setter
+    def max_reruns(self, value):
+        """Set new maximum value for reruns unless property has been locked"""
+        if not self._reruns_locked:
+            self._max_reruns = value
+        else:
+            log.debug('max_reruns is locked, %s tried to modify it', self.current_plugin)
+
+    def lock_reruns(self):
+        """Prevent modification of max_reruns property"""
+        log.debug('Enabling rerun lock')
+        self._reruns_locked = True
+
+    def unlock_reruns(self):
+        """Allow modification of max_reruns property"""
+        log.debug('Releasing rerun lock')
+        self._reruns_locked = False
+
+    @property
+    def reruns_locked(self):
+        return self._reruns_locked
+
+    @property
+    def is_rerun(self):
+        return bool(self._rerun_count)
+
+    @property
+    def rerun_count(self):
+        return self._rerun_count
+
     @property
     def undecided(self):
         """
@@ -286,12 +336,11 @@ class Task(object):
         """
         return self._all_entries
 
-    @property
-    def is_rerun(self):
-        return self._rerun_count
+    def __lt__(self, other):
+        return (self.priority, self._count) < (other.priority, other._count)
 
-    def __cmp__(self, other):
-        return cmp((self.priority, self._count), (other.priority, other._count))
+    def __eq__(self, other):
+        return (self.priority, self._count) == (other.priority, other._count)
 
     def __str__(self):
         return '<Task(name=%s,aborted=%s)>' % (self.name, self.aborted)
@@ -299,9 +348,6 @@ class Task(object):
     def disable_phase(self, phase):
         """Disable ``phase`` from execution.
 
-        All disabled phases are re-enabled by :meth:`Task._reset()` after task
-        execution has been completed.
-
         :param string phase: Name of ``phase``
         :raises ValueError: *phase* could not be found.
         """
@@ -311,11 +357,12 @@ class Task(object):
             log.debug('Disabling %s phase' % phase)
             self.disabled_phases.append(phase)
 
-    def abort(self, reason='Unknown', silent=False):
+    def abort(self, reason='Unknown', silent=False, traceback=None):
         """Abort this task execution, no more plugins will be executed except the abort handling ones."""
         self.aborted = True
         self.abort_reason = reason
         self.silent_abort = silent
+        self.traceback = traceback
         if not self.silent_abort:
             log.warning('Aborting task (plugin: %s)' % self.current_plugin)
         else:
@@ -334,7 +381,7 @@ class Task(object):
         if not isinstance(cat, EntryIterator):
             raise TypeError('category must be a EntryIterator')
         for entry in cat:
-            for k, v in values.iteritems():
+            for k, v in values.items():
                 if not (k in entry and entry[k] == v):
                     break
             else:
@@ -352,7 +399,7 @@ class Task(object):
         if phase:
             plugins = sorted(get_plugins(phase=phase), key=lambda p: p.phase_handlers[phase], reverse=True)
         else:
-            plugins = all_plugins.itervalues()
+            plugins = iter(all_plugins.values())
         return (p for p in plugins if p.name in self.config or p.builtin)
 
     def __run_task_phase(self, phase):
@@ -375,11 +422,12 @@ class Task(object):
                     if not p.builtin:
                         break
                 else:
-                    if phase == 'filter':
-                        log.warning('Task does not have any filter plugins to accept entries. '
-                                    'You need at least one to accept the entries you  want.')
-                    else:
-                        log.warning('Task doesn\'t have any %s plugins, you should add (at least) one!' % phase)
+                    if phase not in self.suppress_warnings:
+                        if phase == 'filter':
+                            log.warning('Task does not have any filter plugins to accept entries. '
+                                        'You need at least one to accept the entries you  want.')
+                        else:
+                            log.warning('Task doesn\'t have any %s plugins, you should add (at least) one!' % phase)
 
         for plugin in self.plugins(phase):
             # Abort this phase if one of the plugins disables it
@@ -411,6 +459,9 @@ class Task(object):
                 finally:
                     fire_event('task.execute.after_plugin', self, plugin.name)
                 self.session = None
+        # check config hash for changes at the end of 'prepare' phase
+        if phase == 'prepare':
+            self.check_config_hash()
 
     def __run_plugin(self, plugin, phase, args=None, kwargs=None):
         """
@@ -463,15 +514,27 @@ class Task(object):
             self.abort(msg)
         except Exception as e:
             msg = 'BUG: Unhandled error in plugin %s: %s' % (keyword, e)
-            log.exception(msg)
-            self.abort(msg)
+            log.critical(msg)
+            traceback = self.manager.crash_report()
+            self.abort(msg, traceback=traceback)
 
-    def rerun(self):
-        """Immediately re-run the task after execute has completed,
-        task can be re-run up to :attr:`.max_reruns` times."""
-        msg = 'Plugin %s has requested task to be ran again after execution has completed.' % self.current_plugin
+    def rerun(self, plugin=None, reason=None):
+        """
+        Immediately re-run the task after execute has completed,
+        task can be re-run up to :attr:`.max_reruns` times.
+
+        :param str plugin: Plugin name
+        :param str reason: Why the rerun is done
+        """
+        msg = 'Plugin {0} has requested task to be ran again after execution has completed.'.format(
+            self.current_plugin if plugin is None else plugin)
+        if reason:
+            msg += ' Reason: {0}'.format(reason)
         # Only print the first request for a rerun to the info log
-        log.debug(msg) if self._rerun else log.info(msg)
+        if self._rerun:
+            log.debug(msg)
+        else:
+            log.info(msg)
         self._rerun = True
 
     def config_changed(self):
@@ -481,6 +544,33 @@ class Task(object):
         """
         self.config_modified = True
 
+    def merge_config(self, new_config):
+        try:
+            merge_dict_from_to(new_config, self.config)
+        except MergeException as e:
+            raise PluginError('Failed to merge configs for task %s: %s' % (self.name, e))
+
+    def check_config_hash(self):
+        """
+        Checks the task's config hash and updates the hash if necessary.
+        """
+        # Save current config hash and set config_modified flag
+        config_hash = get_config_hash(self.config)
+        if self.is_rerun:
+            # Restore the config to state right after start phase
+            if self.prepared_config:
+                self.config = copy.deepcopy(self.prepared_config)
+            else:
+                log.error('BUG: No prepared_config on rerun, please report.')
+        with Session() as session:
+            last_hash = session.query(TaskConfigHash).filter(TaskConfigHash.task == self.name).first()
+            if not last_hash:
+                session.add(TaskConfigHash(task=self.name, hash=config_hash))
+                self.config_changed()
+            elif last_hash.hash != config_hash:
+                last_hash.hash = config_hash
+                self.config_changed()
+
     def _execute(self):
         """Executes the task without rerunning."""
         if not self.enabled:
@@ -495,32 +585,11 @@ class Task(object):
             self.disable_phase('download')
             self.disable_phase('output')
         if self.options.disable_phases:
-            map(self.disable_phase, self.options.disable_phases)
+            list(map(self.disable_phase, self.options.disable_phases))
         if self.options.inject:
             # If entries are passed for this execution (eg. rerun), disable the input phase
             self.disable_phase('input')
-            self.all_entries.extend(self.options.inject)
-
-        # Save current config hash and set config_modidied flag
-        with Session() as session:
-            config_hash = hashlib.md5(str(sorted(self.config.items()))).hexdigest()
-            last_hash = session.query(TaskConfigHash).filter(TaskConfigHash.task == self.name).first()
-            if self.is_rerun:
-                # Restore the config to state right after start phase
-                if self.prepared_config:
-                    self.config = copy.deepcopy(self.prepared_config)
-                else:
-                    log.error('BUG: No prepared_config on rerun, please report.')
-                self.config_modified = False
-            elif not last_hash:
-                self.config_modified = True
-                last_hash = TaskConfigHash(task=self.name, hash=config_hash)
-                session.add(last_hash)
-            elif last_hash.hash != config_hash:
-                self.config_modified = True
-                last_hash.hash = config_hash
-            else:
-                self.config_modified = False
+            self.all_entries.extend(copy.deepcopy(self.options.inject))
 
         # run phases
         try:
@@ -532,9 +601,9 @@ class Task(object):
                             log.info('Plugin %s is not executed because %s phase is disabled (e.g. --test)' %
                                      (plugin.name, phase))
                     continue
-                if phase == 'start' and self.is_rerun:
-                    log.debug('skipping task_start during rerun')
-                elif phase == 'exit' and self._rerun:
+                if phase in ('start', 'prepare') and self.is_rerun:
+                    log.debug('skipping phase %s during rerun', phase)
+                elif phase == 'exit' and self._rerun and self._rerun_count < self.max_reruns:
                     log.debug('not running task_exit yet because task will rerun')
                 else:
                     # run all plugins with this phase
@@ -567,13 +636,14 @@ class Task(object):
         """
 
         try:
+            self.finished_event.clear()
             if self.options.cron:
                 self.manager.db_cleanup()
             fire_event('task.execute.started', self)
             while True:
                 self._execute()
                 # rerun task
-                if self._rerun and self._rerun_count < self.max_reruns:
+                if self._rerun and self._rerun_count < self.max_reruns and self._rerun_count < Task.RERUN_MAX:
                     log.info('Rerunning the task in case better resolution can be achieved.')
                     self._rerun_count += 1
                     # TODO: Potential optimization is to take snapshots (maybe make the ones backlog uses built in
@@ -590,7 +660,7 @@ class Task(object):
 
     @staticmethod
     def validate_config(config):
-        schema = plugin_schemas(context='task')
+        schema = plugin_schemas(interface='task')
         # Don't validate commented out plugins
         schema['patternProperties'] = {'^_': {}}
         return config_schema.process_config(config, schema)
@@ -606,12 +676,27 @@ class Task(object):
 
     copy = __copy__
 
+    def render(self, template):
+        """
+        Renders a template string based on fields in the entry.
+
+        :param template: A template string or FlexGetTemplate that uses jinja2 or python string replacement format.
+        :return: The result of the rendering.
+        :rtype: string
+        :raises RenderError: If there is a problem.
+        """
+        if not isinstance(template, (str, FlexGetTemplate)):
+            raise ValueError(
+                'Trying to render non string template or unrecognized template format, got %s' % repr(template))
+        log.trace('rendering: %s', template)
+        return render_from_task(template, self)
+
 
 @event('config.register')
 def register_config_key():
     task_config_schema = {
         'type': 'object',
-        'additionalProperties': plugin_schemas(context='task')
+        'additionalProperties': plugin_schemas(interface='task')
     }
 
     config_schema.register_config_key('tasks', task_config_schema, required=True)
