@@ -1,7 +1,9 @@
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
 
 import logging
-import Queue
+import queue
+import sys
 import threading
 import time
 
@@ -17,11 +19,14 @@ class TaskQueue(object):
     Task processing thread.
     Only executes one task at a time, if more are requested they are queued up and run in turn.
     """
+
     def __init__(self):
-        self.run_queue = Queue.PriorityQueue()
+        self.run_queue = queue.PriorityQueue()
         self._shutdown_now = False
         self._shutdown_when_finished = False
 
+        self.current_task = None
+
         # We don't override `threading.Thread` because debugging this seems unsafe with pydevd.
         # Overriding __len__(self) seems to cause a debugger deadlock.
         self._thread = threading.Thread(target=self.run, name='task_queue')
@@ -32,25 +37,27 @@ class TaskQueue(object):
 
     def run(self):
         while not self._shutdown_now:
+            # Grab the first job from the run queue and do it
             try:
-                # Grab the first job from the run queue and do it
-                try:
-                    task = self.run_queue.get(timeout=0.5)
-                except Queue.Empty:
-                    if self._shutdown_when_finished:
-                        self._shutdown_now = True
-                    continue
-                try:
-                    task.execute()
-                except TaskAbort as e:
-                    log.debug('task %s aborted: %r' % (task.name, e))
-                finally:
-                    self.run_queue.task_done()
+                self.current_task = self.run_queue.get(timeout=0.5)
+            except queue.Empty:
+                if self._shutdown_when_finished:
+                    self._shutdown_now = True
+                continue
+            try:
+                self.current_task.execute()
+            except TaskAbort as e:
+                log.debug('task %s aborted: %r' % (self.current_task.name, e))
             except (ProgrammingError, OperationalError):
-                log.exception('Database error while running a task. Attempting to recover.')
+                log.critical('Database error while running a task. Attempting to recover.')
+                self.current_task.manager.crash_report()
             except Exception:
-                log.exception('BUG: Unhandled exception during task queue run loop.')
-                raise
+                log.critical('BUG: Unhandled exception during task queue run loop.')
+                self.current_task.manager.crash_report()
+            finally:
+                self.run_queue.task_done()
+                self.current_task = None
+
         remaining_jobs = self.run_queue.qsize()
         if remaining_jobs:
             log.warning('task queue shut down with %s tasks remaining in the queue to run.' % remaining_jobs)
@@ -87,6 +94,12 @@ class TaskQueue(object):
         Waits for the thread to exit.
         Allows abortion of task queue with ctrl-c
         """
+        if sys.version_info >= (3, 4):
+            # Due to python bug, Thread.is_alive doesn't seem to work properly under our conditions on python 3.4+
+            # http://bugs.python.org/issue26793
+            # TODO: Is it important to have the clean abortion? Do we need to find a better way?
+            self._thread.join()
+            return
         try:
             while self._thread.is_alive():
                 time.sleep(0.5)
