@@ -1,27 +1,32 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import difflib
 import logging
 import re
+import random
 
 from bs4.element import Tag
 
 from flexget.utils.soup import get_soup
-from flexget.utils.requests import Session
+from flexget.utils.requests import Session, TimedLimiter
 from flexget.utils.tools import str_to_int
-from flexget.plugin import get_plugin_by_name
+from flexget.plugin import get_plugin_by_name, PluginError
 
 log = logging.getLogger('utils.imdb')
 # IMDb delivers a version of the page which is unparsable to unknown (and some known) user agents, such as requests'
 # Spoof the old urllib user agent to keep results consistent
 requests = Session()
 requests.headers.update({'User-Agent': 'Python-urllib/2.6'})
-#requests.headers.update({'User-Agent': random.choice(USERAGENTS)})
+# requests.headers.update({'User-Agent': random.choice(USERAGENTS)})
 
 # this makes most of the titles to be returned in english translation, but not all of them
 requests.headers.update({'Accept-Language': 'en-US,en;q=0.8'})
+requests.headers.update({'X-Forwarded-For': '24.110.%d.%d' % (random.randint(0, 254), random.randint(0, 254))})
 
 # give imdb a little break between requests (see: http://flexget.com/ticket/129#comment:1)
-requests.set_domain_delay('imdb.com', '3 seconds')
+requests.add_domain_limiter(TimedLimiter('imdb.com', '3 seconds'))
 
 
 def is_imdb_url(url):
@@ -32,6 +37,26 @@ def is_imdb_url(url):
     return re.match(r'https?://[^/]*imdb\.com/', url)
 
 
+def is_valid_imdb_title_id(value):
+    """
+    Return True if `value` is a valid IMDB ID for titles (movies, series, etc).
+    """
+    if not isinstance(value, basestring):
+        raise TypeError("is_valid_imdb_title_id expects a string but got {0}".format(type(value)))
+    # IMDB IDs for titles have 'tt' followed by 7 digits
+    return re.match('tt[\d]{7}', value) is not None
+
+
+def is_valid_imdb_person_id(value):
+    """
+    Return True if `value` is a valid IMDB ID for a person.
+    """
+    if not isinstance(value, basestring):
+        raise TypeError("is_valid_imdb_person_id expects a string but got {0}".format(type(value)))
+    # An IMDB ID for a person is formed by 'nm' followed by 7 digits
+    return re.match('nm[\d]{7}', value) is not None
+
+
 def extract_id(url):
     """Return IMDb ID of the given URL. Return None if not valid or if URL is not a string."""
     if not isinstance(url, basestring):
@@ -47,7 +72,6 @@ def make_url(imdb_id):
 
 
 class ImdbSearch(object):
-
     def __init__(self):
         # de-prioritize aka matches a bit
         self.aka_weight = 0.95
@@ -57,14 +81,14 @@ class ImdbSearch(object):
         self.min_diff = 0.01
         self.debug = False
 
-        self.max_results = 10
+        self.max_results = 50
 
     def ireplace(self, text, old, new, count=0):
         """Case insensitive string replace"""
         pattern = re.compile(re.escape(old), re.I)
         return re.sub(pattern, new, text, count)
 
-    def smart_match(self, raw_name):
+    def smart_match(self, raw_name, single_match=True):
         """Accepts messy name, cleans it and uses information available to make smartest and best match"""
         parser = get_plugin_by_name('parsing').instance.parse_movie(raw_name)
         name = parser.name
@@ -73,9 +97,9 @@ class ImdbSearch(object):
             log.critical('Failed to parse name from %s' % raw_name)
             return None
         log.debug('smart_match name=%s year=%s' % (name, str(year)))
-        return self.best_match(name, year)
+        return self.best_match(name, year, single_match)
 
-    def best_match(self, name, year=None):
+    def best_match(self, name, year=None, single_match=True):
         """Return single movie that best matches name criteria or None"""
         movies = self.search(name)
 
@@ -87,11 +111,14 @@ class ImdbSearch(object):
         for movie in movies[:]:
             if year and movie.get('year'):
                 if movie['year'] != str(year):
-                    log.debug('best_match removing %s - %s (wrong year: %s)' % (movie['name'], movie['url'], str(movie['year'])))
+                    log.debug('best_match removing %s - %s (wrong year: %s)' % (
+                        movie['name'],
+                        movie['url'],
+                        str(movie['year'])))
                     movies.remove(movie)
                     continue
             if movie['match'] < self.min_match:
-                log.debug('best_match removing %s (min_match)' % movie['name'])
+                log.debug('best_match removing %s (min_match)', movie['name'])
                 movies.remove(movie)
                 continue
 
@@ -113,43 +140,46 @@ class ImdbSearch(object):
                 log.debug('remain: %s (match: %s) %s' % (m['name'], m['match'], m['url']))
             return None
         else:
-            return movies[0]
+            return movies[0] if single_match else movies
 
     def search(self, name):
         """Return array of movie details (dict)"""
         log.debug('Searching: %s' % name)
         url = u'http://www.imdb.com/find'
-        # This will only include movies searched by title in the results
-        params = {'q': name, 's': 'tt', 'ttype': 'ft'}
+        # This may include Shorts and TV series in the results
+        params = {'q': name, 's': 'tt'}
 
-        log.debug('Serch query: %s' % repr(url))
+        log.debug('Search query: %s' % repr(url))
         page = requests.get(url, params=params)
         actual_url = page.url
 
         movies = []
+        soup = get_soup(page.text)
         # in case we got redirected to movie page (perfect match)
         re_m = re.match(r'.*\.imdb\.com/title/tt\d+/', actual_url)
         if re_m:
+            div = soup.find('div', {'class': 'title_wrapper'})
+            title_wrapper = div.find('h1', {'itemprop': 'name'})
+            title = ''.join(text for text in title_wrapper.find_all(text=True) if text.parent.name not in ['span', 'a'])
+            title = title.strip()
+            year = div.find('a').text
             actual_url = re_m.group(0)
             log.debug('Perfect hit. Search got redirected to %s' % actual_url)
             movie = {}
             movie['match'] = 1.0
-            movie['name'] = name
-            movie['url'] = actual_url
+            movie['name'] = title
             movie['imdb_id'] = extract_id(actual_url)
-            movie['year'] = None  # skips year check
+            movie['url'] = make_url(movie['imdb_id'])
+            movie['year'] = year
             movies.append(movie)
             return movies
 
-        # the god damn page has declared a wrong encoding
-        soup = get_soup(page.text)
-
         section_table = soup.find('table', 'findList')
         if not section_table:
             log.debug('results table not found')
             return
 
-        rows = section_table.find_all('td', 'result_text')
+        rows = section_table.find_all('tr')
         if not rows:
             log.debug('Titles section does not have links')
         for count, row in enumerate(rows):
@@ -157,15 +187,24 @@ class ImdbSearch(object):
             if count > self.max_results:
                 break
 
+            result_text = row.find('td', 'result_text')
             movie = {}
-            additional = re.findall(r'\((.*?)\)', row.text)
+            additional = re.findall(r'\((.*?)\)', result_text.text)
             if len(additional) > 0:
-                movie['year'] = additional[-1]
+                if re.match('^\d{4}$', additional[-1]):
+                    movie['year'] = additional[-1]
+                elif len(additional) > 1:
+                    movie['year'] = additional[-2]
+                    if additional[-1] not in ['TV Movie', 'Video']:
+                        log.debug('skipping %s', result_text.text)
+                        continue
+            primary_photo = row.find('td', 'primary_photo')
+            movie['thumbnail'] = primary_photo.find('a').find('img').get('src')
 
-            link = row.find_next('a')
+            link = result_text.find_next('a')
             movie['name'] = link.text
-            movie['url'] = 'http://www.imdb.com' + link.get('href')
-            movie['imdb_id'] = extract_id(movie['url'])
+            movie['imdb_id'] = extract_id(link.get('href'))
+            movie['url'] = make_url(movie['imdb_id'])
             log.debug('processing name: %s url: %s' % (movie['name'], movie['url']))
 
             # calc & set best matching ratio
@@ -209,6 +248,7 @@ class ImdbParser(object):
         self.languages = []
         self.actors = {}
         self.directors = {}
+        self.writers = {}
         self.score = 0.0
         self.votes = 0
         self.year = 0
@@ -230,150 +270,105 @@ class ImdbParser(object):
         page = requests.get(url)
         soup = get_soup(page.text)
 
-        # get photo
-        tag_photo = soup.find('td', attrs={'id': 'img_primary'})
-        if tag_photo:
-            tag_img = tag_photo.find('img')
-            if tag_img:
-                self.photo = tag_img.get('src')
-                log.debug('Detected photo: %s' % self.photo)
+        title_overview = soup.find('div', attrs={'class': 'title-overview'})
+        if not title_overview:
+            raise PluginError('IMDB parser needs updating, imdb format changed. Please report on Github.')
 
-        # get rating. contentRating <span> in infobar.
-        tag_infobar_div = soup.find('div', attrs={'class': 'infobar'})
-        if tag_infobar_div:
-            tag_mpaa_rating = tag_infobar_div.find('span', attrs={'itemprop': 'contentRating'})
-            if tag_mpaa_rating:
-                if not tag_mpaa_rating.get('class') or not tag_mpaa_rating['class'][0].startswith('us_'):
-                    log.warning('Could not determine mpaa rating for %s' % url)
-                else:
-                    rating_class = tag_mpaa_rating['class'][0]
-                    if rating_class == 'us_not_rated':
-                        self.mpaa_rating = 'NR'
-                    else:
-                        self.mpaa_rating = rating_class.lstrip('us_').replace('_', '-').upper()
-                log.debug('Detected mpaa rating: %s' % self.mpaa_rating)
-            else:
-                log.debug('Unable to match signature of mpaa rating for %s - '
-                          'could be a TV episode, or plugin needs update?' % url)
+        # Parse stuff from the title-overview section
+        name_elem = title_overview.find('h1', attrs={'itemprop': 'name'})
+        if name_elem:
+            self.name = name_elem.contents[0].strip()
         else:
-            # We should match the infobar, it's an integral part of the IMDB page.
-            log.warning('Unable to get infodiv class for %s - plugin needs update?' % url)
+            log.error('Possible IMDB parser needs updating, Please report on Github.')
+            raise PluginError('Unable to set imdb_name for %s from %s' % (self.imdb_id, self.url))
 
-        # get name
-        tag_name = soup.find('h1')
-        if tag_name:
-            tag_name = tag_name.find('span', attrs={'itemprop': 'name'})
-        if tag_name:
-            self.name = tag_name.text
-            log.debug('Detected name: %s' % self.name)
-        else:
-            log.warning('Unable to get name for %s - plugin needs update?' % url)
-
-        tag_original_title_i = soup.find('i', text=re.compile(r'original title'))
-        if tag_original_title_i:
-            span = tag_original_title_i.parent
-            tag_original_title_i.decompose()
-            self.original_name = span.text.strip().strip('"')
-            log.debug('Detected original name: %s' % self.original_name)
-        else:
-            # if title is already in original language, it doesn't have the tag
-            log.debug('Unable to get original title for %s - it probably does not exists' % url)
-
-        star_box = soup.find('div', attrs={'class': 'star-box giga-star'})
-        if star_box:
-            # detect if movie is eligible for ratings
-            rating_ineligible = star_box.find('div', attrs={'class': 'rating-ineligible'})
-            if rating_ineligible:
-                log.debug('movie is not eligible for ratings')
-            else:
-                # get votes
-                tag_votes = star_box.find(itemprop='ratingCount')
-                if tag_votes:
-                    self.votes = str_to_int(tag_votes.string) or 0
-                    log.debug('Detected votes: %s' % self.votes)
-                else:
-                    log.warning('Unable to get votes for %s - plugin needs update?' % url)
-
-                # get score - find the ratingValue item that contains a numerical value
-                span_score = star_box.find(itemprop='ratingValue', text=re.compile('[\d\.]+'))
-                if span_score:
-                    try:
-                        self.score = float(span_score.string)
-                    except (ValueError, TypeError):
-                        log.debug('tag_score %r is not valid float' % span_score.string)
-                    log.debug('Detected score: %s' % self.score)
-                else:
-                    log.warning('Unable to get score for %s - plugin needs update?' % url)
-        else:
-            log.warning('Unable to find score/vote section for %s - plugin needs update?' % url)
-
-        # get genres
-        genres = soup.find('div', itemprop='genre')
-        if genres:
-            for link in genres.find_all('a'):
-                self.genres.append(link.text.strip().lower())
-        else:
-            log.warning('Unable to find genres section for %s - plugin needs update?' % url)
-
-        # get languages
-        for link in soup.find_all('a', href=re.compile('/language/.*')):
-            # skip non-primary languages "(a few words)", etc.
-            m = re.search('(?x) \( [^()]* \\b few \\b', link.next_sibling)
-            if not m:
-                lang = link.text.lower()
-                if not lang in self.languages:
-                    self.languages.append(lang.strip())
-
-        # get year
-        tag_year = soup.find('a', attrs={'href': re.compile('^/year/\d+')})
-        if tag_year:
-            self.year = int(tag_year.text)
-            log.debug('Detected year: %s' % self.year)
-        elif soup.head.title:
-            m = re.search(r'(\d{4})\)', soup.head.title.string)
+        year = title_overview.find('span', attrs={'id': 'titleYear'})
+        if year:
+            m = re.search(r'([0-9]{4})', year.text)
             if m:
                 self.year = int(m.group(1))
-                log.debug('Detected year: %s' % self.year)
-            else:
-                log.warning('Unable to get year for %s (regexp mismatch) - plugin needs update?' % url)
-        else:
-            log.warning('Unable to get year for %s (missing title) - plugin needs update?' % url)
 
-        # get main cast
-        tag_cast = soup.find('table', 'cast_list')
-        if tag_cast:
-            for actor in tag_cast.find_all('a', href=re.compile('/name/nm')):
+        if not self.year:
+            log.debug('No year found for %s' % self.imdb_id)
+
+        mpaa_rating_elem = title_overview.find(itemprop='contentRating')
+        if mpaa_rating_elem:
+            self.mpaa_rating = mpaa_rating_elem['content']
+        else:
+            log.debug('No rating found for %s' % self.imdb_id)
+
+        photo_elem = title_overview.find(itemprop='image')
+        if photo_elem:
+            self.photo = photo_elem['src']
+        else:
+            log.debug('No photo found for %s' % self.imdb_id)
+
+        original_name_elem = title_overview.find(attrs={'class': 'originalTitle'})
+        if original_name_elem:
+            self.original_name = original_name_elem.contents[0].strip().strip('"')
+        else:
+            log.debug('No original title found for %s' % self.imdb_id)
+
+        votes_elem = title_overview.find(itemprop='ratingCount')
+        if votes_elem:
+            self.votes = str_to_int(votes_elem.text)
+        else:
+            log.debug('No votes found for %s' % self.imdb_id)
+
+        score_elem = title_overview.find(itemprop='ratingValue')
+        if score_elem:
+            self.score = float(score_elem.text)
+        else:
+            log.debug('No score found for %s' % self.imdb_id)
+
+        # get director(s)
+        for director in title_overview.select('[itemprop="director"] > a'):
+            director_id = extract_id(director['href'])
+            director_name = director.text
+            # tag instead of name
+            if isinstance(director_name, Tag):
+                director_name = None
+            self.directors[director_id] = director_name
+
+        # get writer(s)
+        for writer in title_overview.select('[itemprop="creator"] > a'):
+            writer_id = extract_id(writer['href'])
+            writer_name = writer.text
+            # tag instead of name
+            if isinstance(writer_name, Tag):
+                writer_name = None
+            self.writers[writer_id] = writer_name
+
+        # Details section
+        title_details = soup.find('div', attrs={'id': 'titleDetails'})
+        if title_details:
+            # get languages
+            for link in title_details.find_all('a', href=re.compile('^/search/title\?title_type=feature'
+                                                                    '&primary_language=')):
+                lang = link.text.strip().lower()
+                if lang not in self.languages:
+                    self.languages.append(lang.strip())
+
+        # Storyline section
+        storyline = soup.find('div', attrs={'id': 'titleStoryLine'})
+        if storyline:
+            plot_elem = storyline.find('p')
+            if plot_elem:
+                # Remove the "Written By" part.
+                if plot_elem.em:
+                    plot_elem.em.replace_with('')
+                self.plot_outline = plot_elem.text.strip()
+            else:
+                log.debug('No storyline found for %s' % self.imdb_id)
+            self.genres = [i.text.strip().lower() for i in storyline.select('[itemprop="genre"] > a')]
+
+        # Cast section
+        cast = soup.find('div', attrs={'id': 'titleCast'})
+        if cast:
+            for actor in cast.select('[itemprop="actor"] > a'):
                 actor_id = extract_id(actor['href'])
                 actor_name = actor.text.strip()
                 # tag instead of name
                 if isinstance(actor_name, Tag):
                     actor_name = None
                 self.actors[actor_id] = actor_name
-
-        # get director(s)
-        h4_director = soup.find('h4', text=re.compile('Director'))
-        if h4_director:
-            for director in h4_director.parent.find_all('a', href=re.compile('/name/nm')):
-                director_id = extract_id(director['href'])
-                director_name = director.text
-                # tag instead of name
-                if isinstance(director_name, Tag):
-                    director_name = None
-                self.directors[director_id] = director_name
-
-        log.debug('Detected genres: %s' % self.genres)
-        log.debug('Detected languages: %s' % self.languages)
-        log.debug('Detected director(s): %s' % ', '.join(self.directors))
-        log.debug('Detected actors: %s' % ', '.join(self.actors))
-
-        # get plot
-        h2_plot = soup.find('h2', text='Storyline')
-        if h2_plot:
-            p_plot = h2_plot.find_next('p')
-            if p_plot and p_plot.next.string:
-                self.plot_outline = p_plot.next.string.strip()
-                log.debug('Detected plot outline: %s' % self.plot_outline)
-            else:
-                log.debug('Plot does not have p-tag')
-        else:
-            log.debug('Failed to find plot')
