@@ -1,4 +1,7 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import basestring
+
 import re
 import copy
 import logging
@@ -8,6 +11,7 @@ log = logging.getLogger('utils.qualities')
 
 class QualityComponent(object):
     """"""
+
     def __init__(self, type, value, name, regexp=None, modifier=None, defaults=None):
         """
         :param type: Type of quality component. (resolution, source, codec, or audio)
@@ -49,8 +53,8 @@ class QualityComponent(object):
     def __hash__(self):
         return hash(self.type + str(self.value))
 
-    def __nonzero__(self):
-        return self.value
+    def __bool__(self):
+        return bool(self.value)
 
     def __eq__(self, other):
         if isinstance(other, basestring):
@@ -112,6 +116,7 @@ class QualityComponent(object):
         # No mutable attributes, return a regular copy
         return copy.copy(self)
 
+
 _resolutions = [
     QualityComponent('resolution', 10, '360p'),
     QualityComponent('resolution', 20, '368p', '368p?'),
@@ -119,9 +124,10 @@ _resolutions = [
     QualityComponent('resolution', 40, '576p', '576p?'),
     QualityComponent('resolution', 45, 'hr'),
     QualityComponent('resolution', 50, '720i'),
-    QualityComponent('resolution', 60, '720p', '(1280x)?720p?x?(50)?'),
+    QualityComponent('resolution', 60, '720p', '(1280x)?720(p|hd)?x?(50)?'),
     QualityComponent('resolution', 70, '1080i'),
-    QualityComponent('resolution', 80, '1080p', '(1920x)?1080p?')
+    QualityComponent('resolution', 80, '1080p', '(1920x)?1080p?x?(50)?'),
+    QualityComponent('resolution', 90, '2160p', '((3840x)?2160p?x?(50)?)|4k')
 ]
 _sources = [
     QualityComponent('source', 10, 'workprint', modifier=-8),
@@ -135,11 +141,11 @@ _sources = [
     QualityComponent('source', 90, 'tvrip', 'tv[\W_]?rip'),
     QualityComponent('source', 100, 'dsr', 'dsr|ds[\W_]?rip'),
     QualityComponent('source', 110, 'sdtv', '(?:[sp]dtv|dvb)(?:[\W_]?rip)?'),
-    QualityComponent('source', 120, 'webrip', 'web[\W_]?rip'),
-    QualityComponent('source', 130, 'dvdscr', '(?:(?:dvd|web)[\W_]?)?scr(?:eener)?', modifier=0),
-    QualityComponent('source', 140, 'bdscr', 'bdscr(?:eener)?'),
+    QualityComponent('source', 120, 'dvdscr', '(?:(?:dvd|web)[\W_]?)?scr(?:eener)?', modifier=0),
+    QualityComponent('source', 130, 'bdscr', 'bdscr(?:eener)?'),
+    QualityComponent('source', 140, 'webrip', 'web[\W_]?rip'),
     QualityComponent('source', 150, 'hdtv', 'a?hdtv(?:[\W_]?rip)?'),
-    QualityComponent('source', 160, 'webdl', 'web(?:[\W_]?(dl|hd))'),
+    QualityComponent('source', 160, 'webdl', 'web(?:[\W_]?(dl|hd))?'),
     QualityComponent('source', 170, 'dvdrip', 'dvd(?:[\W_]?rip)?'),
     QualityComponent('source', 175, 'remux'),
     QualityComponent('source', 180, 'bluray', '(?:b[dr][\W_]?rip|blu[\W_]?ray(?:[\W_]?rip)?)')
@@ -148,16 +154,18 @@ _codecs = [
     QualityComponent('codec', 10, 'divx'),
     QualityComponent('codec', 20, 'xvid'),
     QualityComponent('codec', 30, 'h264', '[hx].?264'),
+    QualityComponent('codec', 35, 'vp9'),
     QualityComponent('codec', 40, 'h265', '[hx].?265|hevc'),
     QualityComponent('codec', 50, '10bit', '10.?bit|hi10p')
 ]
-channels = '(?:(?:[\W_]?5[\W_]?1)|(?:[\W_]?2[\W_]?(?:0|ch)))'
+channels = '(?:(?:[^\w+]?[1-7][\W_]?(?:0|1|ch)))'
 _audios = [
     QualityComponent('audio', 10, 'mp3'),
     # TODO: No idea what order these should go in or if we need different regexps
     QualityComponent('audio', 20, 'aac', 'aac%s?' % channels),
     QualityComponent('audio', 30, 'dd5.1', 'dd%s' % channels),
     QualityComponent('audio', 40, 'ac3', 'ac3%s?' % channels),
+    QualityComponent('audio', 45, 'dd+5.1', 'dd[p+]%s' % channels),
     QualityComponent('audio', 50, 'flac', 'flac%s?' % channels),
     # The DTSs are a bit backwards, but the more specific one needs to be parsed first
     QualityComponent('audio', 60, 'dtshd', 'dts[\W_]?hd(?:[\W_]?ma)?'),
@@ -180,7 +188,6 @@ _UNKNOWNS = {
     print '}}}'
 '''
 
-
 _registry = {}
 for items in (_resolutions, _sources, _codecs, _audios):
     for item in items:
@@ -188,7 +195,7 @@ for items in (_resolutions, _sources, _codecs, _audios):
 
 
 def all_components():
-    return _registry.itervalues()
+    return iter(_registry.values())
 
 
 class Quality(object):
@@ -269,7 +276,7 @@ class Quality(object):
                 return False
         return True
 
-    def __nonzero__(self):
+    def __bool__(self):
         return any(self._comparator)
 
     def __eq__(self, other):
@@ -330,7 +337,7 @@ def get(quality_name):
     if not found_components:
         raise ValueError('No quality specified')
     result = Quality()
-    for type, component in found_components.iteritems():
+    for type, component in found_components.items():
         setattr(result, type, component)
     return result
 
@@ -346,8 +353,8 @@ class RequirementComponent(object):
     def reset(self):
         self.min = None
         self.max = None
-        self.acceptable = []
-        self.none_of = []
+        self.acceptable = set()
+        self.none_of = set()
 
     def allows(self, comp, loose=False):
         if comp.type != self.type:
@@ -377,16 +384,16 @@ class RequirementComponent(object):
             self.min, self.max = min, max
         elif '|' in text:
             quals = text.split('|')
-            quals = [_registry[qual] for qual in quals]
+            quals = {_registry[qual] for qual in quals}
             if any(qual.type != self.type for qual in quals):
                 raise ValueError('Component type mismatch: %s' % text)
-            self.acceptable.extend(quals)
+            self.acceptable |= quals
         else:
             qual = _registry[text.strip('!<>=+')]
             if qual.type != self.type:
                 raise ValueError('Component type mismatch!')
             if text in _registry:
-                self.acceptable.append(qual)
+                self.acceptable.add(qual)
             else:
                 if text[0] == '<':
                     if text[1] != '=':
@@ -397,11 +404,19 @@ class RequirementComponent(object):
                         qual += 1
                     self.min = qual
                 elif text[0] == '!':
-                    self.none_of.append(qual)
+                    self.none_of.add(qual)
+
+    def __eq__(self, other):
+        return ((self.max, self.max, self.acceptable, self.none_of) ==
+                (other.max, other.max, other.acceptable, other.none_of))
+
+    def __hash__(self):
+        return hash(tuple([self.min, self.max, tuple(sorted(self.acceptable)), tuple(sorted(self.none_of))]))
 
 
 class Requirements(object):
     """Represents requirements for allowable qualities. Can determine whether a given Quality passes requirements."""
+
     def __init__(self, req=''):
         self.text = ''
         self.resolution = RequirementComponent('resolution')
@@ -463,6 +478,14 @@ class Requirements(object):
                 return False
         return True
 
+    def __eq__(self, other):
+        if isinstance(other, str):
+            other = Requirements(other)
+        return self.components == other.components
+
+    def __hash__(self):
+        return hash(tuple(self.components))
+
     def __str__(self):
         return self.text or 'any'
 
