@@ -1,10 +1,13 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+
 import logging
 import re
 
 from flexget.utils.titles.parser import TitleParser
 from flexget.utils import qualities
 from flexget.utils.tools import str_to_int
+from datetime import datetime
 
 log = logging.getLogger('movieparser')
 
@@ -19,11 +22,25 @@ def diff_pos(string1, string2):
 
 
 class MovieParser(TitleParser):
+
     def __init__(self):
         self.data = None
         self.reset()
         TitleParser.__init__(self)
 
+    @property
+    def fields(self):
+        """
+        Return a dict of all parser fields
+        """
+        return {
+            'movie_parser': self,
+            'movie_name': self.name,
+            'movie_year': self.year,
+            'proper': self.proper,
+            'proper_count': self.proper_count
+        }
+
     @property
     def valid(self):
         return True
@@ -44,6 +61,7 @@ class MovieParser(TitleParser):
         # parsing results
         self.name = None
         self.year = None
+        self.year_pos = None
         self.quality = qualities.Quality()
         self.proper_count = 0
 
@@ -86,8 +104,13 @@ class MovieParser(TitleParser):
             # check for year
             num = str_to_int(part)
             if num is not None:
-                if 1930 < num < 2050:
+                if 1930 < num <= datetime.now().year:
+                    if self.year_pos == cut_part:
+                        # Looks like a year, but we already set the cutpoint to a year, let's move it forward
+                        cut_part = part_pos
+                        
                     self.year = num
+                    self.year_pos = part_pos
                     cut = True
             # Don't consider all caps words cut words if the whole title has been all caps
             if not part.isupper():
@@ -100,8 +123,10 @@ class MovieParser(TitleParser):
                 cut = True
             # check for propers
             if part.lower() in self.propers:
-                self.proper_count += 1
-                cut = True
+                # 'real' and 'final' are too common in movie titles, only cut if it comes after year
+                if part.lower() not in ['real', 'final'] or self.year:
+                    self.proper_count += 1
+                    cut = True
             # update cut position
             if cut and parts.index(part) < cut_part:
                 cut_part = part_pos
