@@ -1,28 +1,30 @@
 from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from past.builtins import cmp
+
 import logging
+import re
 from datetime import datetime, timedelta
 from string import capwords
 
 from dateutil.parser import parse as parsedate
 
-import re
 from flexget.utils.titles.parser import TitleParser
 from flexget.plugins.parsers import ParseWarning
 from flexget.plugins.parsers.parser_common import default_ignore_prefixes, name_to_re
 from flexget.utils import qualities
 from flexget.utils.tools import ReList
 
-
 log = logging.getLogger('seriesparser')
 
 # Forced to INFO !
 # switch to logging.DEBUG if you want to debug this class (produces quite a bit info ..)
 log.setLevel(logging.INFO)
 
-ID_TYPES = ['ep', 'date', 'sequence', 'id'] # may also be 'special'
+ID_TYPES = ['ep', 'date', 'sequence', 'id']  # may also be 'special'
+
 
 class SeriesParser(TitleParser):
-
     """
     Parse series.
 
@@ -44,14 +46,18 @@ class SeriesParser(TitleParser):
         '(?:series|season)\s?(\d{1,4})\s(\d{1,3})\s?of\s?(?:\d{1,3})',
         '(\d{1,2})\s?x\s?(\d+)(?:\s(\d{1,2}))?',
         '(\d{1,3})\s?of\s?(?:\d{1,3})',
-        '(?:episode|ep|part|pt)\s?(\d{1,3}|%s)' % roman_numeral_re,
-        'part\s(%s)' % '|'.join(map(str, english_numbers))]])
+        '(?:episode|e|ep|part|pt)\s?(\d{1,3}|%s)' % roman_numeral_re,
+        'part\s(%s)' % '|'.join(map(str, english_numbers)),
+    ]])
+    season_pack_regexps = ReList([
+        # S01 or Season 1 but not Season 1 Episode|Part 2
+        r'(?:season\s?|s)(\d{1,})(?:\s|$)(?!(?:(?:.*?\s)?(?:episode|e|ep|part|pt)\s?(?:\d{1,3}|%s)|(?:\d{1,3})\s?of\s?(?:\d{1,3})))' % roman_numeral_re,
+        '(\d{1,3})\s?x\s?all',  # 1xAll
+    ])
     unwanted_regexps = ReList([
         '(\d{1,3})\s?x\s?(0+)[^1-9]',  # 5x0
         'S(\d{1,3})D(\d{1,3})',  # S3D1
-        '(\d{1,3})\s?x\s?(all)',  # 1xAll
-        r'(?:season(?:s)|s|series|\b)\s?\d\s?(?:&\s?\d)?[\s-]*(?:complete|full)',
-        'seasons\s(\d\s){2,}',
+        r'(?:s|series|\b)\s?\d\s?(?:&\s?\d)?[\s-]*(?:complete|full)',
         'disc\s\d'])
     # Make sure none of these are found embedded within a word or other numbers
     date_regexps = ReList([TitleParser.re_not_in_word(regexp) for regexp in [
@@ -146,6 +152,7 @@ class SeriesParser(TitleParser):
         self.special = False
         # TODO: group is only produced with allow_groups
         self.group = None
+        self.season_pack = None
 
         # false if item does not match series
         self.valid = False
@@ -165,7 +172,9 @@ class SeriesParser(TitleParser):
         if match:
             self.identified_by = 'date'
         else:
-            match = self.parse_episode(clean_title)
+            match = self.parse_season_packs(clean_title)
+            if not match:
+                match = self.parse_episode(clean_title)
             self.identified_by = 'ep'
         if not match:
             return
@@ -219,7 +228,8 @@ class SeriesParser(TitleParser):
         # regexp name matching
         if not self.name_regexps:
             # if we don't have name_regexps, generate one from the name
-            self.name_regexps = ReList(name_to_re(name, self.ignore_prefixes, self) for name in [self.name] + self.alternate_names)
+            self.name_regexps = ReList(
+                name_to_re(name, self.ignore_prefixes, self) for name in [self.name] + self.alternate_names)
             # With auto regex generation, the first regex group captures the name
             self.re_from_name = True
         # try all specified regexps on this data
@@ -246,7 +256,7 @@ class SeriesParser(TitleParser):
         if self.allow_groups:
             for group in self.allow_groups:
                 group = group.lower()
-                for fmt in ['[%s]', '-%s']:
+                for fmt in ['[%s]', '-%s', '(%s)']:
                     if fmt % group in data_stripped:
                         log.debug('%s is from group %s', self.data, group)
                         self.group = group
@@ -314,7 +324,7 @@ class SeriesParser(TitleParser):
                     if ep_match['match'].start() > 1:
                         return
 
-                if ep_match['end_episode'] > ep_match['episode'] + 2:
+                if ep_match['end_episode'] and ep_match['end_episode'] > ep_match['episode'] + 2:
                     # This is a pack of too many episodes, ignore it.
                     log.debug('Series pack contains too many episodes (%d). Rejecting',
                               ep_match['end_episode'] - ep_match['episode'])
@@ -331,15 +341,26 @@ class SeriesParser(TitleParser):
                 if not (self.special and self.prefer_specials):
                     return
             else:
-                log.debug('-> no luck with ep_regexps')
+                season_pack_match = self.parse_season_packs(data_stripped)
+                # If a title looks like a special, give it precedence over season pack
+                if season_pack_match and not self.special:
+                    if self.strict_name and season_pack_match['match'].start() > 1:
+                        return
+                    self.season = season_pack_match['season']
+                    self.season_pack = True
+                    self.id = 'S%s' % season_pack_match['season']
+                    self.id_type = 'ep'
+                    self.valid = True
+                else:
+                    log.debug('-> no luck with ep_regexps')
 
-            if self.identified_by == 'ep':
+            if self.identified_by == 'ep' and not self.season_pack:
                 # we should be getting season, ep !
                 # try to look up idiotic numbering scheme 101,102,103,201,202
                 # ressu: Added matching for 0101, 0102... It will fail on
                 #        season 11 though
-                log.debug('expect_ep enabled')
-                match = re.search(self.re_not_in_word(r'(0?\d)(\d\d)'), data_stripped, re.IGNORECASE | re.UNICODE)
+                log.debug('ep identifier expected. Attempting SEE format parsing.')
+                match = re.search(self.re_not_in_word(r'(\d?\d)(\d\d)'), data_stripped, re.IGNORECASE | re.UNICODE)
                 if match:
                     # strict_name
                     if self.strict_name:
@@ -353,7 +374,7 @@ class SeriesParser(TitleParser):
                     self.valid = True
                     return
                 else:
-                    log.debug('-> no luck with the expect_ep')
+                    log.debug('-> no luck with SEE')
 
         # Check id regexps
         if self.identified_by in ['id', 'auto'] and not self.valid:
@@ -545,6 +566,24 @@ class SeriesParser(TitleParser):
                         'match': match}
         return False
 
+    def parse_season_packs(self, data):
+        """Parses data for season packs. Return True if the data contains a hit"""
+        for season_pack_re in self.season_pack_regexps:
+            match = re.search(season_pack_re, data)
+            if match:
+                log.debug('season pack regexp %s match %s', season_pack_re.pattern, match.groups())
+                matches = match.groups()
+                if len(matches) == 1:
+                    # Single season full pack, no parts etc
+                    season = int(matches[0])
+                    return {
+                        'season': season,
+                        'match': match
+                    }
+                elif len(matches) == 2:
+                    # TODO support other formats of season packs: 1xall, s01-PART1, etc.
+                    pass
+
     def roman_to_int(self, roman):
         """Converts roman numerals up to 39 to integers"""
 
@@ -571,7 +610,9 @@ class SeriesParser(TitleParser):
         if not self.valid:
             raise Exception('Series flagged invalid')
         if self.id_type == 'ep':
-            return ['S%02dE%02d' % (self.season, self.episode + x) for x in xrange(self.episodes)]
+            if self.season_pack:
+                return ['S%02d' % self.season]
+            return ['S%02dE%02d' % (self.season, self.episode + x) for x in range(self.episodes)]
         elif self.id_type == 'date':
             return [self.id.strftime('%Y-%m-%d')]
         if self.id is None:
@@ -590,8 +631,11 @@ class SeriesParser(TitleParser):
     def pack_identifier(self):
         """Return a combined identifier for the whole pack if this has more than one episode."""
         # Currently only supports ep mode
-        if self.id_type == 'ep' and self.episodes > 1:
-            return 'S%02dE%02d-E%02d' % (self.season, self.episode, self.episode + self.episodes - 1)
+        if self.id_type == 'ep':
+            if self.episodes > 1:
+                return 'S%02dE%02d-E%02d' % (self.season, self.episode, self.episode + self.episodes - 1)
+            else:
+                return self.identifier
         else:
             return self.identifier
 
@@ -605,9 +649,9 @@ class SeriesParser(TitleParser):
         valid = 'INVALID'
         if self.valid:
             valid = 'OK'
-        return '<SeriesParser(data=%s,name=%s,id=%s,season=%s,episode=%s,quality=%s,proper=%s,status=%s)>' % \
-            (self.data, self.name, str(self.id), self.season, self.episode,
-             self.quality, self.proper_count, valid)
+        return '<SeriesParser(data=%s,name=%s,id=%s,season=%s,season_pack=%s,episode=%s,quality=%s,proper=%s,status=%s)>' % \
+               (self.data, self.name, str(self.id), self.season, self.season_pack, self.episode, self.quality,
+                self.proper_count, valid)
 
     def __cmp__(self, other):
         """Compares quality of parsers, if quality is equal, compares proper_count."""
