@@ -1,27 +1,33 @@
 """Contains miscellaneous helpers"""
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.moves.urllib import request
+from future.utils import PY2
 
-from __future__ import unicode_literals, division, absolute_import, print_function
+import logging
+import ast
 import copy
-import urllib2
-import httplib
+import hashlib
+import locale
+import operator
 import os
-import socket
-import time
 import re
 import sys
-import locale
-import Queue
 from collections import MutableMapping
-from urlparse import urlparse
-from htmlentitydefs import name2codepoint
 from datetime import timedelta, datetime
+from pprint import pformat
+
+import flexget
+import queue
+import requests
+
+from html.entities import name2codepoint
+
+log = logging.getLogger('utils')
 
 
 def str_to_boolean(string):
-    if string.lower() in ['true', '1', 't', 'y', 'yes']:
-        return True
-    else:
-        return False
+    return string.lower() in ['true', '1', 't', 'y', 'yes']
 
 
 def str_to_int(string):
@@ -31,6 +37,16 @@ def str_to_int(string):
         return None
 
 
+if PY2:
+    def native_str_to_text(string, **kwargs):
+        if 'encoding' not in kwargs:
+            kwargs['encoding'] = 'ascii'
+        return string.decode(**kwargs)
+else:
+    def native_str_to_text(string, **kwargs):
+        return string
+
+
 def convert_bytes(bytes):
     """Returns given bytes as prettified string."""
 
@@ -53,7 +69,6 @@ def convert_bytes(bytes):
 
 
 class MergeException(Exception):
-
     def __init__(self, value):
         self.value = value
 
@@ -67,7 +82,7 @@ def strip_html(text):
     try:
         text = ' '.join(BeautifulSoup(text).find_all(text=True))
         return ' '.join(text.split())
-    except:
+    except Exception:
         return text
 
 
@@ -79,10 +94,11 @@ charrefpat = re.compile(r'&(#(\d+|x[\da-fA-F]+)|[\w.:-]+);?')
 def _htmldecode(text):
     """Decode HTML entities in the given text."""
     # From screpe.py - licensed under apache 2.0 .. should not be a problem for a MIT afaik
-    if type(text) is unicode:
-        uchr = unichr
+    if isinstance(text, str):
+        uchr = chr
     else:
-        uchr = lambda value: value > 127 and unichr(value) or chr(value)
+        def uchr(value):
+            value > 127 and chr(value) or chr(value)
 
     def entitydecode(match, uchr=uchr):
         entity = match.group(1)
@@ -94,6 +110,7 @@ def _htmldecode(text):
             return uchr(name2codepoint[entity])
         else:
             return match.group(0)
+
     return charrefpat.sub(entitydecode, text)
 
 
@@ -137,17 +154,17 @@ def merge_dict_from_to(d1, d2):
     """Merges dictionary d1 into dictionary d2. d1 will remain in original form."""
     for k, v in d1.items():
         if k in d2:
-            if type(v) == type(d2[k]):
+            if isinstance(v, type(d2[k])):
                 if isinstance(v, dict):
                     merge_dict_from_to(d1[k], d2[k])
                 elif isinstance(v, list):
                     d2[k].extend(copy.deepcopy(v))
-                elif isinstance(v, (basestring, bool, int, float, type(None))):
+                elif isinstance(v, (str, bool, int, float, type(None))):
                     pass
                 else:
                     raise Exception('Unknown type: %s value: %s in dictionary' % (type(v), repr(v)))
-            elif (isinstance(v, (basestring, bool, int, float, type(None))) and
-                    isinstance(d2[k], (basestring, bool, int, float, type(None)))):
+            elif (isinstance(v, (str, bool, int, float, type(None))) and
+                      isinstance(d2[k], (str, bool, int, float, type(None)))):
                 # Allow overriding of non-container types with other non-container types
                 pass
             else:
@@ -157,103 +174,18 @@ def merge_dict_from_to(d1, d2):
             d2[k] = copy.deepcopy(v)
 
 
-class SmartRedirectHandler(urllib2.HTTPRedirectHandler):
-
+class SmartRedirectHandler(request.HTTPRedirectHandler):
     def http_error_301(self, req, fp, code, msg, headers):
-        result = urllib2.HTTPRedirectHandler.http_error_301(self, req, fp, code, msg, headers)
+        result = request.HTTPRedirectHandler.http_error_301(self, req, fp, code, msg, headers)
         result.status = code
         return result
 
     def http_error_302(self, req, fp, code, msg, headers):
-        result = urllib2.HTTPRedirectHandler.http_error_302(self, req, fp, code, msg, headers)
+        result = request.HTTPRedirectHandler.http_error_302(self, req, fp, code, msg, headers)
         result.status = code
         return result
 
 
-def urlopener(url_or_request, log, **kwargs):
-    """
-    Utility function for pulling back a url, with a retry of 3 times, increasing the timeout, etc.
-    Re-raises any errors as URLError.
-
-    .. warning:: This is being replaced by requests library.
-                 flexget.utils.requests should be used going forward.
-
-    :param str url_or_request: URL or Request object to get.
-    :param log: Logger to log debug info and errors to
-    :param kwargs: Keyword arguments to be passed to urlopen
-    :return: The file-like object returned by urlopen
-    """
-    from flexget.utils.requests import is_unresponsive, set_unresponsive
-
-    if isinstance(url_or_request, urllib2.Request):
-        url = url_or_request.get_host()
-    else:
-        url = url_or_request
-    if is_unresponsive(url):
-        msg = '%s is known to be unresponsive, not trying again.' % urlparse(url).hostname
-        log.warning(msg)
-        raise urllib2.URLError(msg)
-
-    retries = kwargs.get('retries', 3)
-    timeout = kwargs.get('timeout', 15.0)
-
-    # get the old timeout for sockets, so we can set it back to that when done. This is NOT threadsafe by the way.
-    # In order to avoid requiring python 2.6, we're not using the urlopen timeout parameter. That really should be used
-    # after checking for python 2.6.
-    oldtimeout = socket.getdefaulttimeout()
-    try:
-        socket.setdefaulttimeout(timeout)
-
-        handlers = [SmartRedirectHandler()]
-        if urllib2._opener:
-            handlers.extend(urllib2._opener.handlers)
-        if kwargs.get('handlers'):
-            handlers.extend(kwargs['handlers'])
-        if len(handlers) > 1:
-            handler_names = [h.__class__.__name__ for h in handlers]
-            log.debug('Additional handlers have been specified for this urlopen: %s' % ', '.join(handler_names))
-        opener = urllib2.build_opener(*handlers).open
-        for i in range(retries):  # retry getting the url up to 3 times.
-            if i > 0:
-                time.sleep(3)
-            try:
-                retrieved = opener(url_or_request, kwargs.get('data'))
-            except urllib2.HTTPError as e:
-                if e.code < 500:
-                    # If it was not a server error, don't keep retrying.
-                    log.warning('Could not retrieve url (HTTP %s error): %s' % (e.code, e.url))
-                    raise
-                log.debug('HTTP error (try %i/%i): %s' % (i + 1, retries, e.code))
-            except (urllib2.URLError, socket.timeout) as e:
-                if hasattr(e, 'reason'):
-                    reason = str(e.reason)
-                else:
-                    reason = 'N/A'
-                if reason == 'timed out':
-                    set_unresponsive(url)
-                log.debug('Failed to retrieve url (try %i/%i): %s' % (i + 1, retries, reason))
-            except httplib.IncompleteRead as e:
-                log.critical('Incomplete read - see python bug 6312')
-                break
-            else:
-                # make the returned instance usable in a with statement by adding __enter__ and __exit__ methods
-
-                def enter(self):
-                    return self
-
-                def exit(self, exc_type, exc_val, exc_tb):
-                    self.close()
-
-                retrieved.__class__.__enter__ = enter
-                retrieved.__class__.__exit__ = exit
-                return retrieved
-
-        log.warning('Could not retrieve url: %s' % url_or_request)
-        raise urllib2.URLError('Could not retrieve url after %s tries.' % retries)
-    finally:
-        socket.setdefaulttimeout(oldtimeout)
-
-
 class ReList(list):
     """
     A list that stores regexps.
@@ -275,7 +207,7 @@ class ReList(list):
 
     def __getitem__(self, k):
         item = list.__getitem__(self, k)
-        if isinstance(item, basestring):
+        if isinstance(item, str):
             item = re.compile(item, re.IGNORECASE | re.UNICODE)
             self[k] = item
         return item
@@ -306,14 +238,6 @@ else:
         io_encoding = 'ascii'
 
 
-def console(text):
-    """Print to console safely."""
-    if isinstance(text, str):
-        print(text)
-        return
-    print(unicode(text).encode(io_encoding, 'replace'))
-
-
 def parse_timedelta(value):
     """Parse a string like '5 days' into a timedelta object. Also allows timedeltas to pass through."""
     if isinstance(value, timedelta):
@@ -332,11 +256,20 @@ def parse_timedelta(value):
     except TypeError:
         raise ValueError('Invalid time format \'%s\'' % value)
 
+
+def timedelta_total_seconds(td):
+    """replaces python 2.7+ timedelta.total_seconds()"""
+    # TODO: Remove this when we no longer support python 2.6
+    try:
+        return td.total_seconds()
+    except AttributeError:
+        return (td.days * 24 * 3600) + td.seconds + (td.microseconds / 1000000)
+
+
 def multiply_timedelta(interval, number):
-    """timedeltas can not normally be multiplied by floating points. This does that."""
-    # Python 2.6 doesn't have total seconds
-    total_seconds = interval.seconds + interval.days * 24 * 3600
-    return timedelta(seconds=total_seconds*number)
+    """`timedelta`s can not normally be multiplied by floating points. This does that."""
+    return timedelta(seconds=timedelta_total_seconds(interval) * number)
+
 
 if os.name == 'posix':
     def pid_exists(pid):
@@ -372,22 +305,66 @@ else:
         # process is still running.
         return is_running or exit_code.value == STILL_ACTIVE
 
+_binOps = {
+    ast.Add: operator.add,
+    ast.Sub: operator.sub,
+    ast.Mult: operator.mul,
+    ast.Div: operator.truediv,
+    ast.Mod: operator.mod
+}
+
+
+def arithmeticEval(s):
+    """
+    A safe eval supporting basic arithmetic operations.
+
+    :param s: expression to evaluate
+    :return: value
+    """
+    node = ast.parse(s, mode='eval')
+
+    def _eval(node):
+        if isinstance(node, ast.Expression):
+            return _eval(node.body)
+        elif isinstance(node, ast.Str):
+            return node.s
+        elif isinstance(node, ast.Num):
+            return node.n
+        elif isinstance(node, ast.BinOp):
+            return _binOps[type(node.op)](_eval(node.left), _eval(node.right))
+        else:
+            raise Exception('Unsupported type {}'.format(node))
+
+    return _eval(node.body)
+
 
 class TimedDict(MutableMapping):
     """Acts like a normal dict, but keys will only remain in the dictionary for a specified time span."""
+
     def __init__(self, cache_time='5 minutes'):
         self.cache_time = parse_timedelta(cache_time)
         self._store = dict()
+        self._last_prune = datetime.now()
+
+    def _prune(self):
+        """Prune all expired keys."""
+        for key, (add_time, _) in list(self._store.items()):
+            if add_time < datetime.now() - self.cache_time:
+                del self._store[key]
+        self._last_prune = datetime.now()
 
     def __getitem__(self, key):
         add_time, value = self._store[key]
-        # Prune data and raise KeyError when expired
+        # Prune data and raise KeyError if expired
         if add_time < datetime.now() - self.cache_time:
             del self._store[key]
             raise KeyError(key, 'cache time expired')
         return value
 
     def __setitem__(self, key, value):
+        # Make sure we clear periodically, even if old keys aren't accessed again
+        if self._last_prune < datetime.now() - (2 * self.cache_time):
+            self._prune()
         self._store[key] = (datetime.now(), value)
 
     def __delitem__(self, key):
@@ -395,19 +372,20 @@ class TimedDict(MutableMapping):
 
     def __iter__(self):
         # Uses our getitem to skip expired items
-        return (key for key in self._store.keys() if key in self)
+        return (key for key in list(self._store.keys()) if key in self)
 
     def __len__(self):
         return len(list(self.__iter__()))
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, dict(zip(self._store, (v[1] for v in self._store.values()))))
+        return '%s(%r)' % (
+            self.__class__.__name__, dict(list(zip(self._store, (v[1] for v in list(self._store.values()))))))
 
 
-class BufferQueue(Queue.Queue):
+class BufferQueue(queue.Queue):
     """Used in place of a file-like object to capture text and access it safely from another thread."""
     # Allow access to the Empty error from here
-    Empty = Queue.Empty
+    Empty = queue.Empty
 
     def write(self, line):
         self.put(line)
@@ -420,4 +398,181 @@ def singleton(cls):
         if cls not in instances:
             instances[cls] = cls(*args, **kwargs)
         return instances[cls]
+
     return getinstance
+
+
+def split_title_year(title):
+    """Splits title containing a year into a title, year pair."""
+    if not title:
+        return
+    if not re.search(r'\d{4}', title):
+        return title, None
+    match = re.search(r'(.*?)\(?(\d{4})?\)?$', title)
+
+    title = match.group(1).strip()
+    year_match = match.group(2)
+
+    if year_match and not title:
+        # title looks like a year, '2020' for example
+        title = year_match
+        year = None
+    elif title and not year_match:
+        year = None
+    else:
+        year = int(year_match)
+    return title, year
+
+
+def get_latest_flexget_version_number():
+    """
+    Return latest Flexget version from https://pypi.python.org/pypi/FlexGet/json
+    """
+    try:
+        data = requests.get('https://pypi.python.org/pypi/FlexGet/json').json()
+        return data.get('info', {}).get('version')
+    except requests.RequestException:
+        return
+
+
+def get_current_flexget_version():
+    return flexget.__version__
+
+
+def parse_filesize(text_size, si=True):
+    """
+    Parses a data size and returns its value in mebibytes
+
+    :param string text_size: string containing the data size to parse i.e. "5 GB"
+    :param bool si: If True, possibly ambiguous units like KB, MB, GB will be assumed to be base 10 units,
+    rather than the default base 2. i.e. if si then 50 GB = 47684 else 50GB = 51200
+
+    :returns: an float with the data size in mebibytes
+    """
+    prefix_order = {'': 0, 'k': 1, 'm': 2, 'g': 3, 't': 4, 'p': 5}
+
+    parsed_size = re.match('(\d+(?:[.,\s]\d+)*)(?:\s*)((?:[ptgmk]i?)?b)', text_size.strip().lower(), flags=re.UNICODE)
+    if not parsed_size:
+        raise ValueError('%s does not look like a file size' % text_size)
+    amount = parsed_size.group(1)
+    unit = parsed_size.group(2)
+    if not unit.endswith('b'):
+        raise ValueError('%s does not look like a file size' % text_size)
+    unit = unit.rstrip('b')
+    if unit.endswith('i'):
+        si = False
+        unit = unit.rstrip('i')
+    if unit not in prefix_order:
+        raise ValueError('%s does not look like a file size' % text_size)
+    order = prefix_order[unit]
+    amount = float(amount.replace(',', '').replace(' ', ''))
+    base = 1000 if si else 1024
+    return (amount * (base ** order)) / 1024 ** 2
+
+
+def get_config_hash(config):
+    """
+    :param dict config: Configuration
+    :return: MD5 hash for *config*
+    """
+    if isinstance(config, dict) or isinstance(config, list):
+        # this does in fact support nested dicts, they're sorted too!
+        return hashlib.md5(pformat(config).encode('utf-8')).hexdigest()
+    else:
+        return hashlib.md5(str(config).encode('utf-8')).hexdigest()
+
+
+def get_config_as_array(config, key):
+    """
+    Return configuration key as array, even if given as a single string
+    :param dict config: Configuration
+    :param string key: Configuration
+    :return: Array
+    """
+    v = config.get(key, [])
+    if isinstance(v, str):
+        return [v]
+    return v
+
+
+def parse_episode_identifier(ep_id, identify_season=False):
+    """
+    Parses series episode identifier, raises ValueError if it fails
+
+    :param ep_id: Value to parse
+    :return: Return identifier type: `sequence`, `ep` or `date`
+    :raises ValueError: If ep_id does not match any valid types
+    """
+    error = None
+    identified_by = None
+    entity_type = 'episode'
+    if isinstance(ep_id, int):
+        if ep_id <= 0:
+            error = 'sequence type episode must be higher than 0'
+        identified_by = 'sequence'
+    elif re.match(r'(?i)^S\d{1,4}E\d{1,3}$', ep_id):
+        identified_by = 'ep'
+    elif re.match(r'(?i)^S\d{1,4}$', ep_id) and identify_season:
+        identified_by = 'ep'
+        entity_type = 'season'
+    elif re.match(r'\d{4}-\d{2}-\d{2}', ep_id):
+        identified_by = 'date'
+    else:
+        # Check if a sequence identifier was passed as a string
+        try:
+            ep_id = int(ep_id)
+            if ep_id <= 0:
+                error = 'sequence type episode must be higher than 0'
+            identified_by = 'sequence'
+        except ValueError:
+            error = '`%s` is not a valid episode identifier.' % ep_id
+    if error:
+        raise ValueError(error)
+    return (identified_by, entity_type)
+
+
+def aggregate_inputs(task, inputs):
+    from flexget import plugin
+
+    entries = []
+    entry_titles = set()
+    entry_urls = set()
+    entry_locations = set()
+    for item in inputs:
+        for input_name, input_config in item.items():
+            input = plugin.get_plugin_by_name(input_name)
+            if input.api_ver == 1:
+                raise plugin.PluginError('Plugin %s does not support API v2' % input_name)
+            method = input.phase_handlers['input']
+            try:
+                result = method(task, input_config)
+            except plugin.PluginError as e:
+                log.warning('Error during input plugin %s: %s', input_name, e)
+                continue
+
+            if not result:
+                log.warning('Input %s did not return anything', input_name)
+                continue
+
+            for entry in result:
+                urls = ([entry['url']] if entry.get('url') else []) + entry.get('urls', [])
+
+                if any(url in entry_urls for url in urls):
+                    log.debug('URL for `%s` already in entry list, skipping.', entry['title'])
+                    continue
+
+                if entry['title'] in entry_titles:
+                    log.debug('Ignored duplicate title `%s`', entry['title'])  # TODO: should combine?
+                    continue
+
+                if entry.get('location') and entry['location'] in entry_locations:
+                    log.debug('Ignored duplicate location `%s`', entry['location'])  # TODO: should combine?
+                    continue
+
+                entries.append(entry)
+                entry_titles.add(entry['title'])
+                entry_urls.update(urls)
+                if entry.get('location'):
+                    entry_locations.add(entry['location'])
+
+    return entries
