@@ -1,4 +1,6 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import unicode_literals, division, absolute_import
+from builtins import *  # noqa pylint: disable=unused-import, redefined-builtin
+from future.utils import with_metaclass
 
 import re
 
@@ -22,7 +24,7 @@ class Errors(object):
 
     def add(self, msg):
         """Add new error message to current path."""
-        path = [unicode(p) for p in self.path]
+        path = [str(p) for p in self.path]
         msg = '[/%s] %s' % ('/'.join(path), msg)
         self.messages.append(msg)
 
@@ -40,7 +42,7 @@ class Errors(object):
         """Removes level from path by depth number"""
         if self.path_level is None:
             raise Exception('no path level')
-        del(self.path[self.path_level])
+        del (self.path[self.path_level])
         self.path_level -= 1
 
     def path_update_value(self, value):
@@ -49,6 +51,7 @@ class Errors(object):
             raise Exception('no path level')
         self.path[self.path_level] = value
 
+
 # A registry mapping validator names to their class
 registry = {}
 
@@ -73,18 +76,19 @@ def any_schema(schemas):
         return {'anyOf': schemas}
 
 
-class Validator(object):
+class ValidatorType(type):
+    """Automatically adds subclasses to the registry."""
+
+    def __init__(cls, name, bases, dict):
+        type.__init__(cls, name, bases, dict)
+        if 'name' not in dict:
+            raise Exception('Validator %s is missing class-attribute name' % name)
+        registry[dict['name']] = cls
+
+
+class Validator(with_metaclass(ValidatorType)):
     name = 'validator'
 
-    class __metaclass__(type):
-        """Automatically adds subclasses to the registry."""
-
-        def __init__(cls, name, bases, dict):
-            type.__init__(cls, name, bases, dict)
-            if not 'name' in dict:
-                raise Exception('Validator %s is missing class-attribute name' % name)
-            registry[dict['name']] = cls
-
     def __init__(self, parent=None, message=None, **kwargs):
         self.valid = []
         self.message = message
@@ -175,14 +179,14 @@ class ChoiceValidator(Validator):
         self.valid_ic = []
         Validator.__init__(self, parent, **kwargs)
 
-    def accept(self, value, ignore_case=False):
+    def accept(self, value, ignore_case=False):  # pylint: disable=W0221
         """
         :param value: accepted text, int or boolean
         :param bool ignore_case: Whether case matters for text values
         """
-        if not isinstance(value, (basestring, int, float)):
+        if not isinstance(value, (str, int, float)):
             raise Exception('Choice validator only accepts strings and numbers')
-        if isinstance(value, basestring) and ignore_case:
+        if isinstance(value, str) and ignore_case:
             self.valid_ic.append(value.lower())
         else:
             self.valid.append(value)
@@ -199,7 +203,7 @@ class ChoiceValidator(Validator):
         if self.valid_ic:
             schemas.append(any_schema({"type": "string", "pattern": "(?i)^%s$" % p} for p in self.valid_ic))
         s = any_schema(schemas)
-        s['error'] = 'Must be one of the following: %s' % ', '.join(map(unicode, self.valid + self.valid_ic))
+        s['error'] = 'Must be one of the following: %s' % ', '.join(map(str, self.valid + self.valid_ic))
         return s
 
 
@@ -343,7 +347,7 @@ class PathValidator(TextValidator):
     def __init__(self, parent=None, allow_replacement=False, allow_missing=False, **kwargs):
         self.allow_replacement = allow_replacement
         self.allow_missing = allow_missing
-        Validator.__init__(self, parent, **kwargs)
+        TextValidator.__init__(self, parent, **kwargs)
 
     def _schema(self):
         if self.allow_missing:
@@ -359,7 +363,7 @@ class UrlValidator(TextValidator):
             self.protocols = protocols
         else:
             self.protocols = ['ftp', 'http', 'https', 'file']
-        Validator.__init__(self, parent, **kwargs)
+            TextValidator.__init__(self, parent, **kwargs)
 
     def _schema(self):
         return {'type': 'string', 'format': 'url'}
@@ -417,7 +421,7 @@ class DictValidator(Validator):
 
     def require_key(self, key):
         """Flag key as mandatory"""
-        if not key in self.required_keys:
+        if key not in self.required_keys:
             self.required_keys.append(key)
 
     def accept_any_key(self, value, **kwargs):
@@ -443,7 +447,7 @@ class DictValidator(Validator):
             # Make sure errors show up in our list
             key_validator.add_parent(self)
         elif key_type:
-            if isinstance(key_type, basestring):
+            if isinstance(key_type, str):
                 key_type = [key_type]
             key_validator = self.get_validator('root')
             for key_type in key_type:
@@ -457,7 +461,7 @@ class DictValidator(Validator):
     def _schema(self):
         schema = {'type': 'object'}
         properties = schema['properties'] = {}
-        for key, validators in self.valid.iteritems():
+        for key, validators in self.valid.items():
             if not validators:
                 continue
             properties[key] = any_schema(v.schema() for v in validators)
@@ -471,8 +475,8 @@ class DictValidator(Validator):
         else:
             schema['additionalProperties'] = False
         # TODO: implement this
-        #if self.reject_keys:
-        #    schema['reject_keys'] = self.reject
+        # if self.reject_keys:
+        #     schema['reject_keys'] = self.reject
 
         return schema
 
@@ -490,6 +494,7 @@ class QualityRequirementsValidator(TextValidator):
     def _schema(self):
         return {'type': 'string', 'format': 'qualityRequirements'}
 
+
 # ---- TESTING ----
 
 
@@ -538,19 +543,11 @@ def build_options_validator(options):
 
 
 def complex_test():
-
     def build_list(series):
         """Build series list to series."""
         series.accept('text')
         series.accept('number')
         bundle = series.accept('dict')
-        # prevent invalid indentation level
-        """
-        bundle.reject_keys(['set', 'path', 'timeframe', 'name_regexp',
-            'ep_regexp', 'id_regexp', 'watched', 'quality', 'min_quality',
-            'max_quality', 'qualities', 'exact', 'from_group'],
-            'Option \'$key\' has invalid indentation level. It needs 2 more spaces.')
-        """
         bundle.accept_any_key('path')
         options = bundle.accept_any_key('dict')
         build_options_validator(options)
@@ -564,28 +561,13 @@ def complex_test():
     simple = root.accept('list')
     build_list(simple)
 
-    # advanced format:
-    #   settings:
-    #     group: {...}
-    #   group:
-    #     {...}
-
-    """
-    advanced = root.accept('dict')
-    settings = advanced.accept('dict', key='settings')
-    settings_group = settings.accept_any_key('dict')
-    build_options_validator(settings_group)
-
-    group = advanced.accept_any_key('list')
-    build_list(group)
-    """
-
     return root
 
 
 if __name__ == '__main__':
     from flexget.plugins.input.rss import InputRSS
-    #v = complex_test()
+
+    # v = complex_test()
     v = InputRSS().validator()
     schema = v.schema()
 
@@ -593,12 +575,9 @@ if __name__ == '__main__':
 
     print(json.dumps(schema, sort_keys=True, indent=4))
 
-    """
-    root = factory()
-    list = root.accept('list')
-    list.accept('text')
-    list.accept('regexp')
-    list.accept('choice').accept_choices(['foo', 'bar'])
-
-    print root.schema()
-    """
+    # root = factory()
+    # list = root.accept('list')
+    # list.accept('text')
+    # list.accept('regexp')
+    # list.accept('choice').accept_choices(['foo', 'bar'])
+    # print root.schema()
